package jflowchart.engine.arithmetics;

import java.util.LinkedList;

import jflowchart.engine.common.Const;
import jflowchart.engine.common.Functions;
import jflowchart.engine.runtime.Program;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;
import jflowchart.ui.canvas.graphnodes.GraphNode;



public class Assignment extends BinaryOperation implements GraphNodeOperation {

	private Variable op1;
	private GraphNode parent;
	
	public Assignment() {
		super();
		op1 = null;
		super.setOperator(Const.ASSIGN_OPERATION_CODE);
	}
	
	public void setFirstOperand(Variable v) throws IncompatibleDataTypesException {
		super.setFirstOperand(v);
		op1 = v;
	}
	
	public void setOperator(final int opCode) {
	}
	
	public Variable getFirstOperand() {
		return op1;
	}
	
	@Override
	public Value evaluate() {
		try {
			op1.setValue(op2.getValue());
		} catch (IncompatibleDataTypesException e) {
			return null;
		}
		return op1.getValue();
	}

	@Override
	public Assignment deepCopy() {
		Assignment copy = new Assignment();
		try {
			copy.setFirstOperand(getFirstOperand());
			copy.setSecondOperand(getSecondOperand());
		} catch (IncompatibleDataTypesException e) {}
		copy.setGraphNode(getGraphNode());
		return copy;
	}

	@Override
	public String print() {
		return op1.print() + Const.SPACE + Const.ASSIGN_OPERATION_SIGN + Const.SPACE + op2.print();
	}

	@Override
	public String copy() {
		return Const.ASSIGNMENT_CODE + Const.SPACE + getFirstOperand().copy() + Const.SPACE + getSecondOperand().copy();
	}

	@Override
	public int paste(String[] data, int pointer) {
		if (! data[pointer].equals(Const.ASSIGNMENT_CODE)) return Const.PASTE_ERROR;
		else {
			try {
				setFirstOperand((Variable)Functions.parseOperand(data, pointer+1, parent.getCanvas().getVariables(), this));
			} catch (IncompatibleDataTypesException e) {
				return Const.PASTE_ERROR;
			}
			int ptr = getFirstOperand().paste(data, pointer+1);
			if (ptr == Const.PASTE_ERROR) return ptr;
			else {
				try {
					setSecondOperand(Functions.parseOperand(data, ptr, parent.getCanvas().getVariables(), this));
				} catch (IncompatibleDataTypesException e) {
					return Const.PASTE_ERROR;
				}
				return getSecondOperand().paste(data, ptr);
			}
		}
	}

	@Override
	public GraphNode getGraphNode() {
		return parent;
	}

	@Override
	public void setGraphNode(GraphNode n) {
		parent = n;
	}
	
	public Variable checkVariables(Program p) {
		return (Variable)checkOperand(getSecondOperand(), p);
	}

	@Override
	public void save(XMLWriter h) throws XMLWriterException {
		h.openExpressionElem(Const.ASSIGNMENT_CODE, Const.ASSIGN_OPERATION_SIGN);
		op1.save(h);
		op2.save(h);
		h.closeElem();
	}
	
	public void collectVariables(LinkedList<Variable> vars) {
		getFirstOperand().collectVariables(vars);
		getSecondOperand().collectVariables(vars);
	}

}
