package jflowchart.engine.arithmetics;

import jflowchart.engine.common.Const;
import jflowchart.engine.runtime.Program;

public abstract class BinaryOperation implements Operation {

	protected Operand op1;
	protected Operand op2;
	protected int operationCode;
	
	protected BinaryOperation() {
		op1 = null;
		op2 = null;
		operationCode = Const.UNSELECTED_CODE;
	}
	
	public void setFirstOperand(Operand o) throws IncompatibleDataTypesException {
		if ((o == null) || ((op2 != null) && (! o.getDataType().equals(op2.getDataType())))) 
			throw new IncompatibleDataTypesException();
		else op1 = o;
	}
	
	public void setSecondOperand(Operand o) throws IncompatibleDataTypesException {
		if ((o == null) || ((op1 != null) && (! o.getDataType().equals(op1.getDataType())))) 
			throw new IncompatibleDataTypesException();
		else op2 = o;
	}
	
	public Operand getFirstOperand() {
		return op1;
	}
	
	public Operand getSecondOperand() {
		return op2;
	}
	
	public boolean usable() {
		return ((op1 != null) && (operationCode != Const.UNSELECTED_CODE) && (op2 != null));
	}
	
	public void setOperator(final int opCode) {
		operationCode = opCode;
	}
	
	public int getOperator() {
		return operationCode;
	}
	
	protected Operand checkOperand(Operand o, Program p) {
		if (o.isVariable() && (! p.isSet((Variable)o)))
			return o;
		else if (o.isExpression()) {
			NumberExpression ne = (NumberExpression)o;
			return (Operand)ne.checkVariables(p);
		} else return null;
	}
	
	public Variable checkVariables(Program p) {
		Variable v = (Variable)checkOperand(getFirstOperand(), p);
		if (v != null) return v;
		else {
			v = (Variable)checkOperand(getSecondOperand(), p);
			if (v != null) return v;
			else return null;
		}
	}
	
}
