package jflowchart.engine.arithmetics;

import java.util.LinkedList;

import jflowchart.engine.common.Const;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;

public class ComparableVariable extends GeneralVariable implements ComparableOperand {
	
	@SuppressWarnings("rawtypes")
	protected ComparableVariable(Class type, String name, ComparableValue defaultValue, boolean num) {
		super(type, name, defaultValue, true, num);
		value = defaultValue;
	}
	
	@SuppressWarnings("rawtypes")
	public ComparableVariable(Class type, String name, ComparableValue defaultValue) {
		super(type, name, defaultValue, true, false);
		value = defaultValue;
	}
	
	public ComparableValue getValue() {
		return (ComparableValue)value;
	}
	
	public void setValue(ComparableValue v) throws IncompatibleDataTypesException {
		try {
			super.setValue(v);
			value = v;
		} catch (IncompatibleDataTypesException e) {
			throw e;
		}
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public int compareTo(ComparableOperand o) {
		return getValue().getInnerValue().compareTo(o.getValue().getInnerValue());
	}
	
	protected String generalCopy() {
		return super.copy();
	}

	@Override
	public String copy() {
		return Const.COMPARABLE_VARIABLE_CODE + generalCopy();
	}
	
	@Override
	public int paste(String[] data, int pointer) {
		if (! data[pointer].equals(Const.COMPARABLE_VARIABLE_CODE))
			return Const.PASTE_ERROR;
		else return pointer + 3;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public Comparable getInnerValue() {
		ComparableValue c = (ComparableValue)value;
		return c.getInnerValue();
	}

	@Override
	public boolean hasFixValue() {
		return false;
	}

	@Override
	public boolean contains(Variable v) {
		return this.equals(v);
	}

	@Override
	public void save(XMLWriter h) throws XMLWriterException {
		h.writeVariable(Const.COMPARABLE_VARIABLE_CODE, super.printType(), super.getName());
	}

	@Override
	public void collectVariables(LinkedList<Variable> vars) {
		if (!vars.contains(this)) 
			vars.add(this);
	}

}
