package jflowchart.engine.arithmetics;

import java.util.LinkedList;

import jflowchart.engine.common.Const;
import jflowchart.engine.common.Functions;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;
import jflowchart.ui.canvas.graphnodes.GraphNode;



public class Comparison extends ComparableBinaryOperation implements GraphNodeOperation {
	
	private GraphNode parent;
	
	public Comparison() {
		super();
	}
	
	public Comparison deepCopy() {
		Comparison copy = new Comparison();
		try {
			copy.setFirstOperand(getFirstOperand());
			copy.setSecondOperand(getSecondOperand());
		} catch (IncompatibleDataTypesException e) {}
		copy.setOperator(operationCode);
		copy.setGraphNode(getGraphNode());
		return copy;
	}
	
	public Boolean evaluate() {
		boolean ret = false;
		if (operationCode == Const.EQUAL_OPERATION_CODE) ret = (getFirstOperand().compareTo(getSecondOperand()) == 0);
		else if (operationCode == Const.NONEQUAL_OPERATION_CODE) ret = (getFirstOperand().compareTo(getSecondOperand()) != 0);
		else if (operationCode == Const.LESS_OPERATION_CODE) ret = (getFirstOperand().compareTo(getSecondOperand()) < 0);
		else if (operationCode == Const.GREATER_OPERATION_CODE) ret = (getFirstOperand().compareTo(getSecondOperand()) > 0);
		else if (operationCode == Const.LESSEQUAL_OPERATION_CODE) ret = (getFirstOperand().compareTo(getSecondOperand()) <= 0);
		else if (operationCode == Const.GREATEREQUAL_OPERATION_CODE) ret = (getFirstOperand().compareTo(getSecondOperand()) >= 0);
		return ret;
	}
	
	private String getOperatorSign() {
		if ((operationCode == Const.EQUAL_OPERATION_CODE) || (operationCode == Const.NONEQUAL_OPERATION_CODE) 
				|| (operationCode == Const.LESS_OPERATION_CODE) || (operationCode == Const.GREATER_OPERATION_CODE) 
				|| (operationCode == Const.LESSEQUAL_OPERATION_CODE) || (operationCode == Const.GREATEREQUAL_OPERATION_CODE)) 
			return Const.COMPARISON_OPERATION_SIGNS[operationCode];
		else return "";
	}
	
	private String printOperator() {
		return Const.SPACE.concat(getOperatorSign()).concat(Const.SPACE);
	}
	
	public String print() {
		return getFirstOperand().print() + printOperator() + getSecondOperand().print();
	}

	@Override
	public String copy() {
		return Const.COMPARISON_CODE + Const.SPACE + getFirstOperand().copy() + printOperator() + getSecondOperand().copy();
	}

	@Override
	public int paste(String[] data, int pointer) {
		if (! data[pointer].equals(Const.COMPARISON_CODE)) return Const.PASTE_ERROR;
		else {
			try {
				setFirstOperand((ComparableOperand)Functions.parseOperand(data, pointer+1, Functions.filterNonComparableVariables(parent.getCanvas().getVariables()), this));
			} catch (IncompatibleDataTypesException e) {
				return Const.PASTE_ERROR;
			}
			int ptr = getFirstOperand().paste(data, pointer+1);
			if (ptr == Const.PASTE_ERROR) return ptr;
			else {
				try {
					setOperator(Functions.parseComparisonOperatorCode(data[ptr]));
				} catch (InvalidOperatorException e) {
					return Const.PASTE_ERROR;
				}
				ptr++;
				try {
					setSecondOperand((ComparableOperand)Functions.parseOperand(data, ptr, Functions.filterNonComparableVariables(parent.getCanvas().getVariables()), this));
				} catch (IncompatibleDataTypesException e) {
					return Const.PASTE_ERROR;
				}
				return getSecondOperand().paste(data, ptr);
			}
		}
	}

	@Override
	public GraphNode getGraphNode() {
		return parent;
	}

	@Override
	public void setGraphNode(GraphNode n) {
		parent = n;
	}
	
	public ConditionValue isPassing() {
		if (getFirstOperand().isUserSet() || getSecondOperand().isUserSet()
				|| getFirstOperand().isIterating() || getSecondOperand().isIterating())
			return ConditionValue.UNKNOWN;
		else if (evaluate()) return ConditionValue.TRUE;
		else return ConditionValue.FALSE;
	}
	
	public ConditionValue isContradiction() {
		if (getFirstOperand().isUserSet() || getSecondOperand().isUserSet()
				|| getFirstOperand().isIterating() || getSecondOperand().isIterating())
			return ConditionValue.UNKNOWN;
		else if (!evaluate()) return ConditionValue.TRUE;
		else return ConditionValue.FALSE;
	}

	@Override
	public void save(XMLWriter h) throws XMLWriterException {
		h.openExpressionElem(Const.COMPARISON_CODE, this.getOperatorSign());
		getFirstOperand().save(h);
		getSecondOperand().save(h);
		h.closeElem();
	}
	
	public void collectVariables(LinkedList<Variable> vars) {
		getFirstOperand().collectVariables(vars);
		getSecondOperand().collectVariables(vars);
	}
	
}
