package jflowchart.engine.arithmetics;

import jflowchart.engine.common.Const;

public abstract class GeneralVariable implements Variable {

	@SuppressWarnings("rawtypes")
	private Class type;
	private final Value defaultValue;
	protected Value value;
	private String name;
	
	private boolean comparableFlag;
	private boolean numberFlag;
	private boolean userSet;
	private boolean iterating;
	
	@SuppressWarnings("rawtypes")
	protected GeneralVariable(Class type, String name, Value defaultValue, boolean comp, boolean num) {
		this.type = type;
		this.name = name;
		this.defaultValue = defaultValue;
		value = defaultValue;
		comparableFlag = comp;
		numberFlag = num;
		userSet = false;
		iterating = false;
	}

	public String getName() {
		return name;
	}
	
	@Override
	public void setValue(Value v) throws IncompatibleDataTypesException {
		if (! v.getDataType().equals(getDataType())) throw new IncompatibleDataTypesException();
		else {
			userSet = false;
			iterating = false;
			value = v;
		}
	}
	
	@Override
	public void discardValue() {
		userSet = false;
		iterating = false;
		value = defaultValue;
	}
	
	public String printType() {
		int typeCode = Const.UNSELECTED_CODE;
		if (type.equals(Integer.class)) typeCode = Const.INTEGER_TYPE_LABEL_CODE;
		else if (type.equals(Float.class)) typeCode = Const.FLOAT_TYPE_LABEL_CODE;
		else if (type.equals(String.class)) typeCode = Const.STRING_TYPE_LABEL_CODE;
		return Const.TYPE_LABELS[typeCode];
	}
	
	public String print() {
		 return name;
	}
	
	public String copy() {
		return Const.SPACE + printType() + Const.SPACE + getName();
	}

	@Override
	public Value getValue() {
		return value;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public Class getDataType() {
		return type;
	}

	@Override
	public boolean isValue() {
		return false;
	}

	@Override
	public boolean isVariable() {
		return true;
	}

	@Override
	public boolean isExpression() {
		return false;
	}

	@Override
	public boolean isComparable() {
		return comparableFlag;
	}

	@Override
	public boolean isNumber() {
		return numberFlag;
	}
	
	@Override
	public void setUserSet() {
		userSet = true;
	}

	@Override
	public boolean isUserSet() {
		return userSet;
	}
	
	@Override
	public boolean isIterating() {
		return iterating;
	}

	@Override
	public void setIterating() {
		iterating = true;
	}

}
