package jflowchart.engine.arithmetics;

import java.util.LinkedList;

import jflowchart.engine.common.Const;
import jflowchart.engine.common.Functions;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;


public class NumberExpression extends ComparableBinaryOperation implements NumberOperand {
	
	private NumberOperand op1;
	private NumberOperand op2;
	
	private GraphNodeOperation parent;
	
	public NumberExpression() {
		super();
		op1 = null;
		op2 = null;
	}
	
	public void setFirstOperand(NumberOperand o) throws IncompatibleDataTypesException {
		try {
			super.setFirstOperand(o);
			op1 = o;
		} catch (IncompatibleDataTypesException e) {
			throw e;
		}
	}
	
	public void setSecondOperand(NumberOperand o) throws IncompatibleDataTypesException {
		try {
			super.setSecondOperand(o);
			op2 = o;
		} catch (IncompatibleDataTypesException e) {
			throw e;
		}
	}
	
	public NumberOperand getFirstOperand() {
		return op1;
	}
	
	public NumberOperand getSecondOperand() {
		return op2;
	}
	
	private String printOperator() {
		String ret = Const.SPACE;
		if ((operationCode == Const.ADD_OPERATION_CODE) || (operationCode == Const.SUB_OPERATION_CODE) 
				|| (operationCode == Const.MUL_OPERATION_CODE) || (operationCode == Const.DIV_OPERATION_CODE)) 
			ret = ret.concat(Const.NUMBER_OPERATION_SIGNS[operationCode]);
		return ret.concat(Const.SPACE);
	}
	
	private String brackets(String s) {
		return Const.OPEN_BRACKET + s + Const.CLOSE_BRACKET;
	}
	
	private String printOperand(NumberOperand o) {
		if (Functions.numberExpression(o)) return brackets(o.print());
		else return o.print();
	}
	
	@Override
	public String print() {
		return printOperand(op1) + printOperator() + printOperand(op2);
	}

	@Override
	public NumberValue evaluate() {
		NumberValue n = null;
		if (operationCode == Const.ADD_OPERATION_CODE) n = op1.add(op2);
		else if (operationCode == Const.SUB_OPERATION_CODE) n = op1.sub(op2);
		else if (operationCode == Const.MUL_OPERATION_CODE) n = op1.mul(op2);
		else if (operationCode == Const.DIV_OPERATION_CODE) n = op1.div(op2);
		return n;
	}

	@Override
	public NumberValue getValue() {
		return evaluate();
	}

	@SuppressWarnings("rawtypes")
	@Override
	public Class getDataType() {
		return evaluate().getDataType();
	}

	@SuppressWarnings("unchecked")
	@Override
	public int compareTo(ComparableOperand o) {
		return getValue().getInnerValue().compareTo(o.getValue().getInnerValue());
	}

	@Override
	public Number getNumber() {
		return evaluate().getNumber();
	}

	@Override
	public NumberValue add(NumberOperand o) {
		return Functions.add(getNumber(), o.getNumber());
	}

	@Override
	public NumberValue sub(NumberOperand o) {
		return Functions.sub(getNumber(), o.getNumber());
	}

	@Override
	public NumberValue mul(NumberOperand o) {
		return Functions.mul(getNumber(), o.getNumber());
	}

	@Override
	public NumberValue div(NumberOperand o) {
		return Functions.div(getNumber(), o.getNumber());
	}

	@Override
	public NumberExpression deepCopy() {
		NumberExpression copy = new NumberExpression();
		try {
			copy.setFirstOperand(getFirstOperand());
			copy.setSecondOperand(getSecondOperand());
			copy.setOperator(getOperator());
		} catch (IncompatibleDataTypesException e) {}
		
		return copy;
	}

	@Override
	public boolean isValue() {
		return false;
	}

	@Override
	public boolean isVariable() {
		return false;
	}

	@Override
	public boolean isExpression() {
		return true;
	}

	@Override
	public boolean isComparable() {
		return true;
	}

	@Override
	public boolean isNumber() {
		return true;
	}
	
	public void setParent(GraphNodeOperation par) {
		parent = par;
	}

	@Override
	public String copy() {
		return Const.NUMBER_EXPRESSION_CODE + Const.SPACE + op1.copy() + Const.SPACE + Const.NUMBER_OPERATION_SIGNS[operationCode] + Const.SPACE + op2.copy();
	}
	
	@Override
	public int paste(String[] data, int pointer) {
		if (! data[pointer].equals(Const.NUMBER_EXPRESSION_CODE)) return Const.PASTE_ERROR;
		else {
			try {
				setFirstOperand((NumberOperand)Functions.parseOperand(data, pointer+1, Functions.filterNonNumberVariables(parent.getGraphNode().getCanvas().getVariables()), parent));
			} catch (IncompatibleDataTypesException e) {
				return Const.PASTE_ERROR;
			}
			int ptr = op1.paste(data, pointer+1);
			if (ptr == Const.PASTE_ERROR) return ptr;
			else {
				try {
					setOperator(Functions.parseNumberOperatorCode(data[ptr]));
				} catch (InvalidOperatorException e) {
					return Const.PASTE_ERROR;
				}
				ptr++;
				try {
					setSecondOperand((NumberOperand)Functions.parseOperand(data, ptr, Functions.filterNonNumberVariables(parent.getGraphNode().getCanvas().getVariables()), parent));
				} catch (IncompatibleDataTypesException e) {
					return Const.PASTE_ERROR;
				}
				return op2.paste(data, ptr);
			}
		}
	}

	@SuppressWarnings("rawtypes")
	@Override
	public Comparable getInnerValue() {
		return getValue().getInnerValue();
	}

	@Override
	public boolean isUserSet() {
		if ((op1 == null) || (op2 == null)) return false;
		else return (op1.isUserSet() || op2.isUserSet());
	}

	@Override
	public boolean hasFixValue() {
		return (op1.hasFixValue() && op2.hasFixValue());
	}

	@Override
	public boolean contains(Variable v) {
		return ((op1.isVariable() && v.equals((Variable)op1))
				|| (op2.isVariable() && v.equals((Variable)op2)));
	}

	@Override
	public boolean isIterating() {
		if ((op1 == null) || (op2 == null)) return false;
		else return (op1.isIterating() || op2.isIterating());
	}

	@Override
	public void save(XMLWriter h) throws XMLWriterException {
		h.openExpressionElem(Const.NUMBER_EXPRESSION_CODE, Const.NUMBER_OPERATION_SIGNS[operationCode]);
		op1.save(h);
		op2.save(h);
		h.closeElem();
	}

	@Override
	public void collectVariables(LinkedList<Variable> vars) {
		op1.collectVariables(vars);
		op2.collectVariables(vars);
	}

}
