package jflowchart.engine.arithmetics;

import jflowchart.engine.common.Const;
import jflowchart.engine.common.Functions;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;

public class NumberValue extends ComparableValue implements NumberOperand {

	private Integer intValue;
	private Float floatValue;
	
	public NumberValue(Integer i) {
		super(i, true);
		intValue = i;
		floatValue = null;
	}
	
	public NumberValue(Float f) {
		super(f, true);
		intValue = null;
		floatValue = f;
	}
	
	@Override
	public NumberValue add(NumberOperand o) {
		return Functions.add(getNumber(), o.getNumber());
	}

	@Override
	public NumberValue sub(NumberOperand o) {
		return Functions.sub(getNumber(), o.getNumber());
	}

	@Override
	public NumberValue mul(NumberOperand o) {
		return Functions.mul(getNumber(), o.getNumber());
	}

	@Override
	public NumberValue div(NumberOperand o) {
		return Functions.div(getNumber(), o.getNumber());
	}

	@Override
	public Number getNumber() {
		if (intValue == null) return floatValue;
		else return intValue;
	}
	
	@Override
	public String copy() {
		String ret = Const.NUMBER_VALUE_CODE + Const.SPACE;
		if (intValue == null) return ret + floatValue.toString();
		else return ret + intValue.toString();
	}
	
	@Override
	public int paste(String[] data, int pointer) {
		if (! data[pointer].equals(Const.NUMBER_VALUE_CODE))
			return Const.PASTE_ERROR;
		else return pointer + 2;
	}
	
	@Override
	public void save(XMLWriter h) throws XMLWriterException {
		h.writeValue(Const.NUMBER_VALUE_CODE, getNumber().toString());
	}

}
