package jflowchart.engine.arithmetics;

import jflowchart.engine.common.Const;
import jflowchart.engine.common.Functions;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;

public class NumberVariable extends ComparableVariable implements NumberOperand {
	
	@SuppressWarnings("rawtypes")
	public NumberVariable(Class type, String name, NumberValue defaultValue) {
		super(type, name, defaultValue, true);
		value = defaultValue;
	}
	
	@Override
	public Number getNumber() {
		NumberValue n = (NumberValue)value;
		return n.getNumber();
	}
	
	public NumberValue getValue() {
		return (NumberValue)value;
	}
	
	public void setValue(NumberValue v) throws IncompatibleDataTypesException {
		try {
			super.setValue(v);
			value = v;
		} catch (IncompatibleDataTypesException e) {
			throw e;
		}
	}

	@Override
	public NumberValue add(NumberOperand o) {
		return Functions.add(getNumber(), o.getNumber());
	}

	@Override
	public NumberValue sub(NumberOperand o) {
		return Functions.sub(getNumber(), o.getNumber());
	}

	@Override
	public NumberValue mul(NumberOperand o) {
		return Functions.mul(getNumber(), o.getNumber());
	}

	@Override
	public NumberValue div(NumberOperand o) {
		return Functions.div(getNumber(), o.getNumber());
	}
	
	@Override
	public String copy() {
		return Const.NUMBER_VARIABLE_CODE + generalCopy();
	}
	
	@Override
	public int paste(String[] data, int pointer) {
		if (! data[pointer].equals(Const.NUMBER_VARIABLE_CODE))
			return Const.PASTE_ERROR;
		else return pointer + 3;
	}
	
	@Override
	public void save(XMLWriter h) throws XMLWriterException {
		h.writeVariable(Const.NUMBER_VARIABLE_CODE, super.printType(), super.getName());
	}

}
