package jflowchart.engine.codegen.c;

import java.util.LinkedList;

import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.codegen.CodeGenerator;
import jflowchart.engine.common.Const;
import jflowchart.engine.runtime.Program;

public class CGenerator implements CodeGenerator {
	
	private int indent;
	private String code;
	
	public CGenerator() {
		indent = 0;
		code = "";
	}
	
	public void incrIndent() {
		indent++;
	}
	
	public void decrIndent() {
		indent--;
	}
	
	public void newLine(String content) {
		String newLine = "\n";
		for (int i = 0; i < indent; i++)
			newLine = newLine.concat("\t");
		code = code.concat(newLine).concat(content);
	}
	
	private void genHeader() {
		code = "#include <stdio.h>";
		newLine("");
		newLine("int main() {");
		incrIndent();
	}
	
	private void genFooter() {
		decrIndent();
		newLine("}");
	}
	
	@SuppressWarnings("rawtypes")
	public static String getFormatString(Class type) {
		if (type.equals(Integer.class))
			return "%i";
		else if (type.equals(Float.class))
			return "%f";
		else return "%s";
	}
	
	@SuppressWarnings("rawtypes")
	private String getTypeLabel(Class type) {
		if (type.equals(Integer.class))
			return "int";
		else if (type.equals(Float.class))
			return "float";
		else return "char *";
	}
	
	public String generate(Program p) {
		indent = 0;
		genHeader();
		LinkedList<Variable> vars = p.collectVariables();
		for (Variable v : vars)
			newLine(getTypeLabel(v.getDataType()) + " " + v.getName() + Const.SEMICOLON);
		p.generateCCode(this);
		genFooter();
		return code;
	}

}
