package jflowchart.engine.history;

import jflowchart.engine.arithmetics.ComparableOperand;
import jflowchart.engine.arithmetics.Comparison;
import jflowchart.engine.arithmetics.IncompatibleDataTypesException;
import jflowchart.ui.canvas.FlowChartCanvas;

public class EditingConditionAction extends GeneralAction {

	private Comparison condition;
	
	private ComparableOperand oldFirstOperand;
	private int oldOperator;
	private ComparableOperand oldSecondOperand;
	
	private ComparableOperand newFirstOperand;
	private int newOperator;
	private ComparableOperand newSecondOperand;
	
	public EditingConditionAction(FlowChartCanvas c, Comparison cond) {
		super(c);
		condition = cond;
		oldFirstOperand = condition.getFirstOperand();
		oldOperator = condition.getOperator();
		oldSecondOperand = condition.getSecondOperand();
	}
	
	public void recordNewData() {
		newFirstOperand = condition.getFirstOperand();
		newOperator = condition.getOperator();
		newSecondOperand = condition.getSecondOperand();
	}
	
	private void writeData(ComparableOperand firstOperand, int operator, ComparableOperand secondOperand) {
		try {
			condition.setFirstOperand(firstOperand);
			condition.setSecondOperand(secondOperand);
		} catch (IncompatibleDataTypesException e) {}
		condition.setOperator(operator);
		
		canvas.getMainShell().setSavedFlag();
		canvas.updateSemaphore();
		canvas.redraw();
	}
	
	@Override
	public void undo() {
		writeData(oldFirstOperand, oldOperator, oldSecondOperand);
	}

	@Override
	public void redo() {
		writeData(newFirstOperand, newOperator, newSecondOperand);
	}

}
