package jflowchart.engine.nodes;

import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;


public class AssistantConnectPoint extends ConnectPoint {
	
	private ConnectPoint origin;

	public AssistantConnectPoint(int x, int y, ConnectPoint orig) {
		super(false, false);
		super.setPosition(x, y);
		super.setOffset(0);
		origin = orig;
	}
	
	public boolean isAssistant() {
		return true;
	}
	
	public int[] getOffsetPoint(int customOffset) {
		return super.computeOffsetPoint(0);
	}
	
	private void removeOpposites() {
		opposites.clear();
	}
	
	public void removeAssistant(boolean in) {
		if (in) {
			removeOpposites();
			origin.removeOpposite(this);
			origin.removeAssistant(in);
		}
		else if (!in && hasOpposites()) {
			opposites.element().removeAssistant(in);
			opposites.element().removeOpposite(this);
			removeOpposites();
		}
		
	}
	
	public ConnectPoint getOrigin() {
		return origin;
	}
	
	public boolean isReachable(ConnectPoint p, boolean in) {
		if (in) {
			if (origin.equals(p))
				return true;
			else return origin.isReachable(p, in);
		} else {
			if (opposites.element().equals(p))
				return true;
			else return opposites.element().isReachable(p, in);
		}
	}
	
	public boolean isFromDecision() {
		return origin.isFromDecision();
	}
	
	public void save(XMLWriter h) throws XMLWriterException {
		h.saveAssistantConnectPoint(id, x, y);
	}

}
