package jflowchart.engine.nodes;

import java.util.LinkedList;

import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;
import jflowchart.ui.canvas.FlowChartCanvas;

public class Connection {
	
	private ConnectPoint from;
	private ConnectPoint to;
	private LinkedList<AssistantConnectPoint> path;
	
	public Connection(ConnectPoint from, ConnectPoint to) {
		this.from = from;
		this.to = to;
		this.path = new LinkedList<AssistantConnectPoint>();
	}
	
	public Connection(ConnectPoint from, LinkedList<AssistantConnectPoint> path, ConnectPoint to) {
		this.from = from;
		this.to = to;
		this.path = path;
	}
	
	public boolean fits(ConnectPoint from, ConnectPoint to) {
		return (((this.from.getID().equals(from.getID())) && (this.to.getID().equals(to.getID()))) 
				|| ((this.to.getID().equals(from.getID())) && (this.from.getID().equals(to.getID()))));
	}
	
	public void setTo(ConnectPoint to) {
		this.to = to;
	}
	
	public void setPath(LinkedList<AssistantConnectPoint> path) {
		this.path = path;
	}
	
	public void addPathPoint(AssistantConnectPoint a) {
		this.path.add(a);
	}
	
	public ConnectPoint getLastOutput() {
		if (path.isEmpty()) return from;
		else return path.getLast();
	}
	
	public void save(XMLWriter h) throws XMLWriterException {
		h.saveConnection(from.getID(), to.getID(), path);
	}
	
	public void addTo(FlowChartCanvas canvas) {
		canvas.addConnection(from, path, to);
	}

}
