package jflowchart.engine.nodes;

import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.codegen.c.CGenerator;
import jflowchart.engine.common.Const;
import jflowchart.engine.runtime.Program;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;

public class DisplayNode extends InputOutputNode {
	
	protected DisplayNode(boolean ghost) {
		super(Const.DISPLAY_NODE_WIDTH, Const.DISPLAY_NODE_HEIGHT, ghost);
	}
	
	protected DisplayNode(int posX, int posY, boolean ghost) {
		super(posX, posY, Const.DISPLAY_NODE_WIDTH, Const.DISPLAY_NODE_HEIGHT, ghost);
	}
	
	public int getWestBorder() {
		return positionX - Math.round(Const.INPUT_OUTPUT_NODE_WIDTH / 2);
	}
	
	public int getEastBorder() {
		return positionX + Math.round((width - Const.INPUT_OUTPUT_NODE_WIDTH) / 2);
	}
	
	public String copy(String textData) {
		textData += Const.DISPLAY_CODE + Const.SPACE;
		return super.copy(textData);
	}
	
	@Override
	public boolean paste(String[] data) {
		if (! data[0].equals(Const.DISPLAY_CODE)) return false;
		else return super.paste(data);
	}

	@Override
	public Variable checkVariables(Program p) {
		if (! p.isSet(var)) return var;
		else return connectPoints[1].checkVariables(p);
	}

	@Override
	public void execute(Program p) {
		if (hasBreakpoint && p.isRunning()) p.pause();
		else {
			p.getCanvas().getMainShell().displayVariable(var);
			connectPoints[1].nextStep(p);
		}
	}

	@Override
	public void save(XMLWriter h) throws XMLWriterException {
		h.openNodeElem(Const.DISPLAY_CODE, positionX, positionY);
		h.saveConnectPoints(connectPoints);
		var.save(h);
		h.closeElem();
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void generateCCode(CGenerator g) {
		Class varType = var.getDataType();
		String format = CGenerator.getFormatString(varType);
		g.newLine("printf(\"" + format + "\\n\", " + var.getName() + ");");
		connectPoints[1].generateCCode(g);
	}

}
