package jflowchart.engine.nodes;

import java.util.LinkedList;

import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.codegen.c.CGenerator;
import jflowchart.engine.common.Const;
import jflowchart.engine.runtime.Program;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;

public abstract class EndNode extends StartEndNode {
	
	protected EndNode(boolean ghost) {
		super(ghost, false);
	}
	
	protected EndNode(int x, int y, boolean ghost) {
		super(x, y, ghost, false);
	}
	
	public ConnectPoint connect(int x, int y) {
		if (isInside(x, y)) return connectPoints[0];
		else return null;
	}
	
	public ConnectPoint connect(int x, int y, boolean input) {
		if (isInside(x, y) && input) return connectPoints[0];
		else return null;
	}
	
	public boolean isReachableEnd(Program p) {
		return true;
	}
	
	@Override
	public boolean hasInfiniteLoop(Program p, PreparationNode origin) {
		return false;
	}
	
	@Override
	public Variable checkVariables(Program p) {
		return null;
	}
	
	@Override
	public void execute(Program p) {
		if (hasBreakpoint && p.isRunning()) p.pause();
		else p.terminate();
	}
	
	@Override
	public void save(XMLWriter h) throws XMLWriterException {
		h.openNodeElem(Const.END_NODE_CODE, positionX, positionY);
		h.saveConnectPoints(connectPoints);
		h.closeElem();
	}
	
	@Override
	public void generateCCode(CGenerator g) {
		g.newLine("return 0;");
	}
	
	@Override
	public void collectVariables(LinkedList<Variable> vars) {
	}

}
