package jflowchart.engine.nodes;

import java.util.LinkedList;

import jflowchart.engine.arithmetics.Assignment;
import jflowchart.engine.arithmetics.IncompatibleDataTypesException;
import jflowchart.engine.arithmetics.Operation;
import jflowchart.engine.arithmetics.Value;
import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.common.Const;
import jflowchart.engine.common.Functions;
import jflowchart.engine.runtime.Program;

public abstract class InputOutputNode extends GeneralNode {

	protected Variable var;
	
	private void init() {
		var = null;
		operation = new Assignment();
		connectPoints[0] = new ConnectPoint(this, true, false, -Const.NODE_CONNECTION_OFFSET);
		connectPoints[1] = new ConnectPoint(this, false, false, Const.NODE_CONNECTION_OFFSET);
		setConnectPoints();
	}
	
	protected InputOutputNode(int w, int h, boolean ghost) {
		super(w, h, 2, ghost);
		init();
	}
	
	protected InputOutputNode(int posX, int posY, int w, int h, boolean ghost) {
		super(posX, posY, w, h, 2, ghost);
		init();
	}

	public void setVariable(Variable v) {
		var = v;
	}
	
	public Variable getVariable() {
		return var;
	}
	
	public void assignValue(Value v) throws IncompatibleDataTypesException {
		var.setValue(v);
	}
	
	@Override
	public Operation getOperation() {
		return null;
	}
	
	public String copy(String textData) {
		textData = super.copy(textData) + Const.SPACE;
		textData += var.copy();
		textData = separateClipboardData(textData);
		return textData;
	}
	
	@Override
	public boolean paste(String[] data) {
		if (! super.paste(data)) return false;
		setVariable((Variable)Functions.parseOperand(data, 3, operation.getGraphNode().getCanvas().getVariables(), operation));
		return (! (var.paste(data, 3) == Const.PASTE_ERROR));
	}
	
	@Override
	public void setConnectPoints() {
		connectPoints[0].setPosition(positionX, getNorthBorder());
		connectPoints[1].setPosition(positionX, getSouthBorder());
	}
	
	public ConnectPoint connect(int x, int y) {
		if (isInside(x, y) && isBetween(y, this.getNorthBorder(), this.positionY)) return connectPoints[0];
		else if (isInside(x, y) && isBetween(y, this.positionY, this.getSouthBorder())) return connectPoints[1];
		else return null;
	}
	
	@Override
	public ConnectPoint connect(int x, int y, boolean input) {
		if (isInside(x, y)) {
			if (input) return connectPoints[0];
			else return connectPoints[1];
		} else return null;
	}
	
	public boolean isReachableEnd(Program p) {
		return connectPoints[1].isReachableEnd(p);
	}
	
	@Override
	public void addLoopOrigin(PreparationNode n) {
		connectPoints[1].addLoopOrigin(n);
	}
	
	@Override
	public void discardLoopOrigin(PreparationNode n) {
		connectPoints[1].removeLoopOrigin(n);
	}
	
	@Override
	public boolean hasInfiniteLoop(Program p, PreparationNode origin) {
		return connectPoints[1].hasInfiniteLoop(p, origin);
	}
	
	@Override
	public boolean isFromDecision() {
		return connectPoints[0].isFromDecision();
	}
	
	@Override
	public boolean checkLoops(PreparationNode origin) {
		return connectPoints[1].checkLoops(origin);
	}
	
	@Override
	public void collectVariables(LinkedList<Variable> vars) {
		var.collectVariables(vars);
		connectPoints[1].collectVariables(vars);
	}

}
