package jflowchart.engine.nodes;

import java.util.LinkedList;

import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.codegen.c.CGenerator;
import jflowchart.engine.runtime.Program;

public class LoopConnectPoint extends ConnectPoint {
	
	private PreparationNode origin;
	
	public LoopConnectPoint(PreparationNode n, boolean in, boolean hor, int offset) {
		super(n, in, hor, offset, in);
		origin = n;
		if (!in) super.addLoopOrigin(origin); 
	}
	
	public void addOpposite(ConnectPoint cp) {
		if (! super.input) {
			super.addOpposite(cp);
			super.addLoopOrigin(origin);
		} else if (cp.hasLoopOrigins())
			super.addOpposite(cp);
	}
	
	public void removeOpposite(ConnectPoint cp) {
		super.removeOpposite(cp);
		if (! super.input) super.addLoopOrigin(origin);
		else super.removeLoopOrigin(origin);
	}
	
	public boolean isLoopConnectPoint() {
		return true;
	}
	
	public void nextStep(Program p) {
		if (input) origin.checkLooping();
		super.nextStep(p);
	}
	
	public void generateCCode(CGenerator g) {
		if (!input) super.generateCCode(g);
	}
	
	public void collectVariables(LinkedList<Variable> vars) {
		if (!input) super.collectVariables(vars);
	}

}
