package jflowchart.engine.nodes;

import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.codegen.c.CGenerator;
import jflowchart.engine.common.Const;
import jflowchart.engine.runtime.Program;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;

public class ManualInputNode extends InputOutputNode {
	
	protected ManualInputNode(boolean ghost) {
		super(Const.MANUAL_INPUT_NODE_WIDTH, Const.MANUAL_INPUT_NODE_HEIGHT, ghost);
	}
	
	protected ManualInputNode(int posX, int posY, boolean ghost) {
		super(posX, posY, Const.MANUAL_INPUT_NODE_WIDTH, Const.MANUAL_INPUT_NODE_HEIGHT, ghost);
	}
	
	public int getWestBorder() {
		return positionX - Math.round(Const.INPUT_OUTPUT_NODE_WIDTH / 2);
	}
	
	public int getEastBorder() {
		return positionX + Math.round((width - Const.INPUT_OUTPUT_NODE_WIDTH) / 2);
	}
	
	public String copy(String textData) {
		textData += Const.MANUAL_INPUT_CODE + Const.SPACE;
		return super.copy(textData);
	}
	
	@Override
	public boolean paste(String[] data) {
		if (! data[0].equals(Const.MANUAL_INPUT_CODE)) return false;
		else return super.paste(data);
	}
	
	private void recordVariable(Program p) {
		p.addVariable(var);
		p.setUserSet(var);
	}
	
	public boolean hasInfiniteLoop(Program p, PreparationNode origin) {
		recordVariable(p);
		return super.hasInfiniteLoop(p, origin);
	}
	
	public boolean isReachableEnd(Program p) {
		recordVariable(p);
		return super.isReachableEnd(p);
	}

	@Override
	public Variable checkVariables(Program p) {
		recordVariable(p);
		return connectPoints[1].checkVariables(p);
	}

	@Override
	public void execute(Program p) {
		if (hasBreakpoint && p.isRunning()) p.pause();
		else {
			p.addVariable(var);
			p.getCanvas().getMainShell().showManualInputShell(p, var);
			connectPoints[1].nextStep(p);
		}
	}
	
	@Override
	public void save(XMLWriter h) throws XMLWriterException {
		h.openNodeElem(Const.MANUAL_INPUT_CODE, positionX, positionY);
		h.saveConnectPoints(connectPoints);
		var.save(h);
		h.closeElem();
	}

	@Override
	public void generateCCode(CGenerator g) {
		@SuppressWarnings("rawtypes")
		Class varType = var.getDataType();
		String format = CGenerator.getFormatString(varType);
		String content = "scanf(\"" + format + "\", ";
		if (!varType.equals(String.class)) content = content.concat("&");
		g.newLine(content + var.getName() + ");");
		connectPoints[1].generateCCode(g);
	}

}
