package jflowchart.engine.nodes;

import java.util.LinkedList;

import jflowchart.engine.arithmetics.Operation;
import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.codegen.c.CGenerator;
import jflowchart.engine.runtime.Program;
import jflowchart.engine.xml.XMLReaderException;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;

public interface Node {
	
	public void setPosition(int x, int y);
	public void setSelected(boolean val);
	public void setGhost(boolean val);
	public void setMoving(boolean val);
	public void move(int dx, int dy);
	void setConnectPoints();
	
	public int getPositionX();
	public int getPositionY();
	
	public int getWidth();
	public int getHeight();
	
	public int getWestBorder();
	public int getEastBorder();
	public int getNorthBorder();
	public int getSouthBorder();
	
	public boolean isInside(int x, int y);
	public boolean isInRectangle(int x, int y, int width, int height);
	public boolean isGhost();
	public boolean isSelected();
	public boolean isMoving();
	public Operation getOperation();
	
	public String copy(String textData);
	public boolean paste(String[] data);
	
	public ConnectPoint[] getConnectPoints();
	
	public boolean isStart();
	public boolean isReachableEnd(Program p);
	public ConnectPoint connect(int x, int y);
	public ConnectPoint connect(int x, int y, boolean input);
	
	public void addLoopOrigin(PreparationNode n);
	public void discardLoopOrigin(PreparationNode n);
	
	public boolean checkLoops(PreparationNode origin);
	public boolean hasInfiniteLoop(Program p, PreparationNode origin);
	public Variable checkVariables(Program p);
	public boolean isFromDecision();
	
	public void toggleBreakpoint();
	public boolean hasBreakpoint();
	
	public void setActive(boolean val);
	public boolean isActive();
	public void execute(Program p);
	
	public void save(XMLWriter h) throws XMLWriterException;
	
	public void setConnectPointIDs(LinkedList<String> IDs) throws XMLReaderException;
	public ConnectPoint getConnectPointByID(String id);
	
	public void collectVariables(LinkedList<Variable> vars);
	
	public void generateCCode(CGenerator g);

}
