package jflowchart.engine.nodes;

import jflowchart.engine.arithmetics.Operation;
import jflowchart.engine.common.Const;

public abstract class StartEndNode extends GeneralNode {
	
	protected boolean start;
	private String nodeClipboardCode;
	
	private void init(boolean start) {
		this.start = start;
		if (start) {
			nodeClipboardCode = Const.START_NODE_CODE;
			connectPoints[0] = new ConnectPoint(this, false, false, Const.NODE_CONNECTION_OFFSET);
		} else {
			nodeClipboardCode = Const.END_NODE_CODE;
			connectPoints[0] = new ConnectPoint(this, true, false, -Const.NODE_CONNECTION_OFFSET);
		}
		setConnectPoints();
	}
	
	protected StartEndNode(boolean ghost, boolean start) {
		super(Const.START_END_NODE_WIDTH, Const.START_END_NODE_HEIGHT, 1, ghost);
		init(start);
	}
	
	protected StartEndNode(int x, int y, boolean ghost, boolean start) {
		super(x, y, Const.START_END_NODE_WIDTH, Const.START_END_NODE_HEIGHT, 1, ghost);
		init(start);
	}
	
	public String copy(String textData) {
		textData += nodeClipboardCode + Const.SPACE;
		textData = super.copy(textData);
		textData = separateClipboardData(textData);
		return textData;
	}
	
	@Override
	public boolean paste(String[] data) {
		if (! data[0].equals(nodeClipboardCode)) return false;
		else {
			try {
				positionX = Integer.parseInt(data[1]);
				positionY = Integer.parseInt(data[2]);
			} catch(Exception e) {
				return false;
			}
		}
		return true;
	}

	@Override
	public Operation getOperation() {
		return null;
	}
	
	public void setConnectPoints() {
		if (start) connectPoints[0].setPosition(positionX, getSouthBorder());
		else connectPoints[0].setPosition(positionX, getNorthBorder());
	}
	
	@Override
	public void addLoopOrigin(PreparationNode n) {	
	}
	
	@Override
	public void discardLoopOrigin(PreparationNode n) {
	}
	
	@Override
	public boolean isFromDecision() {
		return false;
	}
	
	@Override
	public boolean checkLoops(PreparationNode origin) {
		return true;
	}

}
