package jflowchart.engine.nodes;

import java.util.LinkedList;

import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.codegen.c.CGenerator;
import jflowchart.engine.common.Const;
import jflowchart.engine.runtime.Program;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.engine.xml.XMLWriterException;

public abstract class StartNode extends StartEndNode {

	protected StartNode(boolean ghost) {
		super(ghost, true);
	}
	
	protected StartNode(int x, int y, boolean ghost) {
		super(x, y, ghost, true);
	}
	
	public boolean isStart() {
		return true;
	}
	
	public ConnectPoint connect(int x, int y) {
		if (isInside(x, y)) return connectPoints[0];
		else return null;
	}
	
	public ConnectPoint connect(int x, int y, boolean input) {
		if (isInside(x, y) && (! input)) return connectPoints[0];
		else return null;
	}
	
	public boolean isReachableEnd(Program p) {
		return connectPoints[0].isReachableEnd(p);
	}
	
	@Override
	public boolean hasInfiniteLoop(Program p, PreparationNode origin) {
		return connectPoints[0].hasInfiniteLoop(p, origin);
	}
	
	@Override
	public Variable checkVariables(Program p) {
		return connectPoints[0].checkVariables(p);
	}
	
	@Override
	public void execute(Program p) {
		if (hasBreakpoint && p.isRunning()) p.pause();
		else connectPoints[0].nextStep(p);
	}
	
	@Override
	public void save(XMLWriter h) throws XMLWriterException {
		h.openNodeElem(Const.START_NODE_CODE, positionX, positionY);
		h.saveConnectPoints(connectPoints);
		h.closeElem();
	}
	
	@Override
	public void generateCCode(CGenerator g) {
		connectPoints[0].generateCCode(g);
	}
	
	@Override
	public void collectVariables(LinkedList<Variable> vars) {
		connectPoints[0].collectVariables(vars);
	}

}
