package jflowchart.engine.runtime;

import jflowchart.engine.arithmetics.Variable;

public class Semaphore {

	private Program program;
	private SemaphoreState state;
	
	private boolean hasInfiniteLoop;
	
	public Semaphore(Program p) {
		program = p;
		hasInfiniteLoop = false;
		updateState();
	}
	
	public void updateState() {
		hasInfiniteLoop = false;
		if (! program.hasStart()) {
			state = SemaphoreState.MISSING_START;
		} else if (! program.checkVariables()) {
			state = SemaphoreState.UNSET_VARIABLE;
		} else if (! program.isReachableEnd()) {
			state = SemaphoreState.UNREACHABLE_END;
		} else {
			state = SemaphoreState.REACHABLE_END;
			hasInfiniteLoop = (program.hasInfiniteLoop());
		}
	}
	
	public SemaphoreState getState() {
		return state;
	}
	
	public Variable getUnsetVariable() {
		return program.getUnsetVariable();
	}
	
	public boolean greenLight() {
		if ((state == SemaphoreState.MISSING_START) || (state == SemaphoreState.UNREACHABLE_END) 
				|| (state == SemaphoreState.UNSET_VARIABLE))
			return false;
		else return true;
	}
	
	public boolean hasInfiniteLoop() {
		return hasInfiniteLoop;
	}
	
}
