package jflowchart.engine.xml;

import java.io.OutputStream;
import java.util.LinkedList;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import jflowchart.engine.nodes.AssistantConnectPoint;
import jflowchart.engine.nodes.ConnectPoint;
import jflowchart.engine.nodes.Connection;
import jflowchart.ui.canvas.FlowChartCanvas;
import jflowchart.ui.canvas.graphnodes.GraphNode;

public class XMLWriter {
	
	private XMLOutputFactory xmlOutputFactory;
	private XMLStreamWriter xmlWriter;
	
	public XMLWriter() {
		xmlOutputFactory = XMLOutputFactory.newInstance();
		xmlWriter = null;
	}
	
	public void openNodeElem(String label, int x, int y) throws XMLWriterException {
		try {
			xmlWriter.writeStartElement(label);
			xmlWriter.writeAttribute(Labels.X, Integer.toString(x));
			xmlWriter.writeAttribute(Labels.Y, Integer.toString(y));
		} catch (XMLStreamException e) {
			throw new XMLWriterException(e.getMessage());
		}
	}
	
	public void writeValue(String label, String value) throws XMLWriterException {
		try {
			xmlWriter.writeStartElement(label);
			xmlWriter.writeCharacters(value);
			closeElem();
		} catch (XMLStreamException e) {
			throw new XMLWriterException(e.getMessage());
		}
	}
	
	public void writeVariable(String label, String typeLabel, String name) throws XMLWriterException {
		try {
			xmlWriter.writeStartElement(label);
			xmlWriter.writeAttribute(Labels.TYPE, typeLabel);
			xmlWriter.writeCharacters(name);
			closeElem();
		} catch (XMLStreamException e) {
			throw new XMLWriterException(e.getMessage());
		}
	}
	
	public void openExpressionElem(String label, String operator) throws XMLWriterException {
		try {
			xmlWriter.writeStartElement(label);
			xmlWriter.writeAttribute(Labels.OPERATOR, operator);
		} catch (XMLStreamException e) {
			throw new XMLWriterException(e.getMessage());
		}
	}
	
	public void saveConnectPoint(String id) throws XMLWriterException {
		try {
			xmlWriter.writeStartElement(Labels.CONNECT_POINT);
			xmlWriter.writeCharacters(id);
			closeElem();
		} catch (XMLStreamException e) {
			throw new XMLWriterException(e.getMessage());
		}
	}
	
	public void saveConnectPoints(ConnectPoint[] cps) throws XMLWriterException {
		try {
			xmlWriter.writeStartElement(Labels.CONNECT_POINTS);
			for (ConnectPoint cp : cps)
				cp.save(this);
			closeElem();
		} catch (XMLStreamException e) {
			throw new XMLWriterException(e.getMessage());
		}
	}
	
	public void saveAssistantConnectPoint(String id, int x, int y) throws XMLWriterException {
		try {
			xmlWriter.writeStartElement(Labels.ASSISTANT_CONNECT_POINT);
			xmlWriter.writeAttribute(Labels.X, Integer.toString(x));
			xmlWriter.writeAttribute(Labels.Y, Integer.toString(y));
			xmlWriter.writeCharacters(id);
			closeElem();
		} catch (XMLStreamException e) {
			throw new XMLWriterException(e.getMessage());
		}
	}
	
	public void saveConnection(String fromId, String toId, LinkedList<AssistantConnectPoint> path) throws XMLWriterException {
		try {
			xmlWriter.writeStartElement(Labels.CONNECTION);
			xmlWriter.writeAttribute(Labels.FROM, fromId);
			xmlWriter.writeAttribute(Labels.TO, toId);
			for (AssistantConnectPoint a : path)
				a.save(this);
			closeElem();
		} catch (XMLStreamException e) {
			throw new XMLWriterException(e.getMessage());
		}
	}
	
	public void closeElem() throws XMLWriterException {
		try {
			xmlWriter.writeEndElement();
		} catch (XMLStreamException e) {
			throw new XMLWriterException(e.getMessage());
		}
	}
	
	public void save(OutputStream out, FlowChartCanvas c) throws XMLWriterException {
		
		try {
			
			xmlWriter = xmlOutputFactory.createXMLStreamWriter(out, "UTF-8");
			LinkedList<GraphNode> nodes = c.getNodes();
			LinkedList<Connection> connections = c.getConnections();
			
			xmlWriter.writeStartDocument();
			xmlWriter.writeStartElement(Labels.CANVAS);
			xmlWriter.writeAttribute(Labels.CANVAS_SIZE_X, Integer.toString(c.getClientArea().width));
			xmlWriter.writeAttribute(Labels.CANVAS_SIZE_Y, Integer.toString(c.getClientArea().height));
			
			xmlWriter.writeStartElement(Labels.NODES);
			for (GraphNode n : nodes)
				n.save(this);
			closeElem();
			
			xmlWriter.writeStartElement(Labels.CONNECTIONS);
			for (Connection cn : connections)
				cn.save(this);
			closeElem();
			
			closeElem();
			xmlWriter.writeEndDocument();
		
		} catch (XMLStreamException e) {
			throw new XMLWriterException(e.getMessage());
		}
		
		
	}

}
