package jflowchart.ui.canvas;

import java.io.File;
import java.util.LinkedList;
import java.util.Stack;

import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.history.Action;
import jflowchart.engine.nodes.Connection;
import jflowchart.ui.canvas.graphnodes.GraphNode;

public class CanvasEnvironment {
	
	private int canvasWidth;
	private int canvasHeight;
	
	private boolean savedFlag;
	private File activeFile;
	
	private LinkedList<Connection> connections; 
	private LinkedList<Variable> variables;
	private LinkedList<GraphNode> nodes;
	
	private Stack<Action> undoStack;
	private Stack<Action> redoStack;
	
	public CanvasEnvironment(int w, int h) {
		
		canvasWidth =w;
		canvasHeight = h;
		
		savedFlag = true;
		activeFile = null;
		
		connections = new LinkedList<Connection>();
		variables = new LinkedList<Variable>();
		nodes = new LinkedList<GraphNode>();
		
		undoStack = new Stack<Action>();
		redoStack = new Stack<Action>();
	
	}
	
	public void setSavedFlag(boolean val) {
		savedFlag = val;
	}
	
	public void setCurrentFile(File f) {
		activeFile = f;
	}
	
	public void addConnection(Connection c) {
		connections.add(c);
	}
	
	public void addVariable(Variable v) {
		variables.add(v);
	}
	
	public void addNode(GraphNode n) {
		nodes.add(n);
	}
	
	public void pushUndoItem(Action a) {
		undoStack.push(a);
	}
	
	public void pushRedoItem(Action a) {
		redoStack.push(a);
	}
	
	public int getCanvasWidth() {
		return canvasWidth;
	}
	
	public int getCanvasHeight() {
		return canvasHeight;
	}
	
	public boolean getSavedFlag() {
		return savedFlag;
	}
	
	public File getActiveFile() {
		return activeFile;
	}
	
	public LinkedList<Connection> getConnections() {
		return connections;
	}
	
	public LinkedList<Variable> getVariables() {
		return variables;
	}
	
	public LinkedList<GraphNode> getNodes() {
		return nodes;
	}
	
	public Stack<Action> getUndoStack() {
		return undoStack;
	}
	
	public Stack<Action> getRedoStack() {
		return redoStack;
	}

}
