package jflowchart.ui.canvas;

import java.util.LinkedList;
import java.util.Stack;

import jflowchart.engine.arithmetics.Assignment;
import jflowchart.engine.arithmetics.Comparison;
import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.common.Const;
import jflowchart.engine.common.Language;
import jflowchart.engine.history.Action;
import jflowchart.engine.history.AddingAction;
import jflowchart.engine.history.AddingConnectionAction;
import jflowchart.engine.history.EditingConnectionAction;
import jflowchart.engine.history.MovingAction;
import jflowchart.engine.history.RemovingAction;
import jflowchart.engine.history.RemovingConnectionAction;
import jflowchart.engine.history.ResizingAction;
import jflowchart.engine.nodes.AssistantConnectPoint;
import jflowchart.engine.nodes.ConnectPoint;
import jflowchart.engine.nodes.Connection;
import jflowchart.engine.nodes.PreparationNode;
import jflowchart.engine.nodes.StartNode;
import jflowchart.engine.runtime.Program;
import jflowchart.engine.runtime.Semaphore;
import jflowchart.ui.canvas.graphnodes.DecisionGraphNode;
import jflowchart.ui.canvas.graphnodes.DisplayGraphNode;
import jflowchart.ui.canvas.graphnodes.EndGraphNode;
import jflowchart.ui.canvas.graphnodes.GraphNode;
import jflowchart.ui.canvas.graphnodes.ManualInputGraphNode;
import jflowchart.ui.canvas.graphnodes.PreparationGraphNode;
import jflowchart.ui.canvas.graphnodes.ProcessGraphNode;
import jflowchart.ui.canvas.graphnodes.StartGraphNode;
import jflowchart.ui.windows.MainShell;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.wb.swt.SWTResourceManager;


public class FlowChartCanvas extends Canvas {
	
	private Display display;
	private MainShell mainShell;
	private Language lang;
	
	private boolean transparented;
	private int origAlpha;
	
	private boolean dotted;
	private int origStyle;
	
	private StartGraphNode instantStartGhost;
	private EndGraphNode instantEndGhost;
	private ProcessGraphNode instantProcessGhost;
	private DecisionGraphNode instantDecisionGhost;
	private PreparationGraphNode instantPreparationGhost;
	private ManualInputGraphNode instantManualInputGhost;
	private DisplayGraphNode instantDisplayGhost;
	
	private int centerX;
	private int centerY;
	private int dynamicX;
	private int dynamicY;
	
	private boolean verGridLine;
	private boolean horGridLine;
	
	private GraphNode ghost;
	private LinkedList<GraphNode> nodes;
	
	private boolean dragging;
	private GraphNode onCursor;
	private int dragFromX;
	private int dragFromY;
	
	private Menu contextMenu;
	private Point contextMenuAxis;
	private Menu instantNodeContextMenu;
	private Menu instantOperationNodeContextMenu;
	private Menu instantCanvasContextMenu;
	
	private GraphNode selectedNode;
	private boolean multipleSelection;
	private int multipleIndicatorX;
	private int multipleIndicatorY;
	
	private boolean marqueeSelection;
	private int marqueeRectWidth;
	private int marqueeRectHeight;
	
	private int minimalWidth;
	private int minimalHeight;
	
	private int dndResize;
	
	private Stack<Action> undoStack;
	private Stack<Action> redoStack;
	
	private MovingAction movingAction;
	private ResizingAction resizingAction;
	
	private LinkedList<Variable> variables;
	
	private boolean locked;
	private PreparationNode preparationSelected;
	
	private int mouseX;
	private int mouseY;
	
	private boolean drawingGhostConnection;
	private ConnectPoint ghostConnectPoint;
	
	private ConnectPoint activeFreeConnectPoint;
	private ConnectPoint connectingFrom;
	private ConnectPoint connectingTo;
	
	private ConnectPoint activeRemoveConnectPoint;
	private ConnectPoint activeEditingConnectPoint;
	private ConnectPoint editIndicator;
	
	private LinkedList<AssistantConnectPoint> pathBackup;
	private LinkedList<Connection> connections;
	
	private Program program;
	private Semaphore semaphore;
	
	private CanvasPlayer player;
	
	private boolean validActiveConnectPoint;
	
	public LinkedList<GraphNode> getNodes() {
		return nodes;
	}
	
	public void setStartGhost(int x, int y) {
		instantStartGhost.setPosition(x, y);
		ghost = instantStartGhost;
	}
	
	public void setEndGhost(int x, int y) {
		instantEndGhost.setPosition(x, y);
		ghost = instantEndGhost;
	}
	
	public void setProcessGhost(int x, int y) {
		instantProcessGhost.setPosition(x, y);
		ghost = instantProcessGhost;
	}
	
	public void setDecisionGhost(int x, int y) {
		instantDecisionGhost.setPosition(x, y);
		ghost = instantDecisionGhost;
	}
	
	public void setPreparationGhost(int x, int y) {
		instantPreparationGhost.setPosition(x, y);
		ghost = instantPreparationGhost;
	}
	
	public void setManualInputGhost(int x, int y) {
		instantManualInputGhost.setPosition(x, y);
		ghost = instantManualInputGhost;
	}
	
	public void setDisplayGhost(int x, int y) {
		instantDisplayGhost.setPosition(x, y);
		ghost = instantDisplayGhost;
	}
	
	public void discardGhost() {
		ghost = null;
	}
	
	private int max(int[] a) {
		int ret = Integer.MIN_VALUE;
		for (int i = 0; i < a.length; i++) 
			if (a[i] > ret) ret = a[i];
		return ret;
	}
	
	private void adjustMinimalSize() {
		
		int n = nodes.size();
		int[] eastBorders = new int[n];
		int[] southBorders = new int[n];
		
		for (int i = 0; i < n; i++) {
			eastBorders[i] = nodes.get(i).getEastBorder();
			southBorders[i] = nodes.get(i).getSouthBorder();
		}
		
		minimalWidth = max(eastBorders);
		minimalHeight = max(southBorders);
		
		mainShell.adjustImageExportItems();
		
	}
	
	public void addNode(GraphNode n) {
		nodes.add(n);
		if (n.isStart() && (! program.hasStart()))
			program.setStart((StartNode)n);
		mainShell.setSavedFlag();
		adjustMinimalSize();
		semaphore.updateState();
		this.redraw();
	}
	
	private void addNodesRecorded(LinkedList<GraphNode> l, AddingAction a) {
		for (int i = 0; i < l.size(); i++) {
			GraphNode n = l.get(i);
			addNode(n);
			a.addAddingNode(n);
		}
		recordAction(a);
	}
	
	private void addGraphNode(GraphNode n) {
		LinkedList<GraphNode> l = new LinkedList<GraphNode>();
		l.add(n);
		AddingAction a = new AddingAction(this);
		addNodesRecorded(l, a);
	}
	
	public void addStartGraphNode(int x, int y) {
		if (! program.hasStart()) {
			StartGraphNode n = new StartGraphNode(this, x, y, false);
			addGraphNode(n);
		}
	}
	
	public void addEndGraphNode(int x, int y) {
		EndGraphNode n = new EndGraphNode(this, x, y, false);
		addGraphNode(n);
	}
	
	public void addDecisionGraphNode(int x, int y) {
		DecisionGraphNode n = new DecisionGraphNode(this, x, y, false);
		Comparison c = (Comparison)n.getOperation();
		mainShell.showConditionSettingsShell(mainShell.getShell(), c, false);
		if (c.usable()) addGraphNode(n);
	}
	
	public void addProcessGraphNode(int x, int y) {
		ProcessGraphNode n = new ProcessGraphNode(this, x, y, false);
		Assignment a = n.getOperation();
		mainShell.showAssignmentSettingsShell(mainShell.getShell(), a, true, false);
		if (a.usable()) addGraphNode(n);
	}
	
	public void addPreparationGraphNode(int x, int y) {
		PreparationGraphNode n = new PreparationGraphNode(this, x, y, false);
		Comparison c = n.getCondition();
		Assignment a = n.getOperation();
		mainShell.showPreparationSettingsShell(c, a, false);
		if (c.usable() && a.usable()) addGraphNode(n);
	}
	
	public void addManualInputGraphNode(int x, int y) {
		ManualInputGraphNode n = new ManualInputGraphNode(this, x, y, false);
		mainShell.showInputOutputSettingsShell(n, true, false);
		if (n.getVariable() != null) addGraphNode(n);
	}
	
	public void addDisplayGraphNode(int x, int y) {
		DisplayGraphNode n = new DisplayGraphNode(this, x, y, false);
		mainShell.showInputOutputSettingsShell(n, false, false);
		if (n.getVariable() != null) addGraphNode(n);
	}
	
	private void transparency(GC gc) {
		if (! transparented) {
			origAlpha = gc.getAlpha();
			gc.setAlpha((int)(Const.GHOST_TRANSPARENCY_RATIO * origAlpha));
			transparented = true;
		}
	}
	
	private void untransparency(GC gc) {
		if (transparented) {
			gc.setAlpha(origAlpha);
			transparented = false;
		}
	}
	
	private void dot(GC gc) {
		if (! dotted) {
			origStyle = gc.getLineStyle();
			gc.setLineStyle(SWT.LINE_DOT);
			dotted = true;
		}
	}
	
	private void undot(GC gc) {
		if (dotted) {
			gc.setLineStyle(origStyle);
			dotted = false;
		}
	}
	
	private void drawSelection(GC gc, int north, int west, int width, int height) {
		transparency(gc);
		dot(gc);
		gc.drawRectangle(west, north, width, height);
		undot(gc);
		untransparency(gc);
	}
	
	private void drawActiveNode(GC gc, int north, int west, int width, int height) {
		
		Color defColor = gc.getForeground();
		gc.setForeground(display.getSystemColor(SWT.COLOR_DARK_GREEN));
		dot(gc);
		gc.drawRectangle(west, north, width, height);
		undot(gc);
		gc.setForeground(defColor);
		
		defColor = gc.getBackground();
		gc.setBackground(display.getSystemColor(SWT.COLOR_DARK_GREEN));
		int origAlpha = gc.getAlpha();
		gc.setAlpha(34);
		gc.fillRectangle(west, north, width, height);
		gc.setAlpha(origAlpha);
		gc.setBackground(defColor);
		
	}
	
	private void drawNode(GraphNode node, GC gc) {
		if (node.isGhost()) transparency(gc);
		node.draw(gc);
		if (node.isGhost()) untransparency(gc);
		if (node.isActive()) drawActiveNode(gc, node.getNorthBorder(), node.getWestBorder(), node.getWidth(), node.getHeight());
		if (node.isSelected() && (! dragging)) drawSelection(gc, node.getNorthBorder(), node.getWestBorder(), node.getWidth(), node.getHeight());
	}
	
	public void drawNodes(GC gc) {
		if (ghost != null) drawNode(ghost, gc);
		for (int i = 0; i < nodes.size(); i++)
			drawNode(nodes.get(i), gc);
	}
	
	private int getHorizontalDirection(int fromX, int toX) {
		int ret = Const.AB_DIRECTION_SAME_POINT;
		int dir = toX - fromX;
		if (dir < 0) ret = Const.AB_DIRECTION_WEST;
		else if (dir > 0) ret = Const.AB_DIRECTION_EAST;
		return ret;
	}
	
	private int getVerticalDirection(int fromY, int toY) {
		int ret = Const.AB_DIRECTION_SAME_POINT;
		int dir = toY - fromY;
		if (dir < 0) ret = Const.AB_DIRECTION_NORTH;
		else if (dir > 0) ret = Const.AB_DIRECTION_SOUTH;
		return ret;
	}
	
	private int getDirection(int fromX, int fromY, int toX, int toY) {
		int ret = getHorizontalDirection(fromX, toX);
		int multiplicator = 1;
		if (ret != 0) multiplicator = 10;
		return ret + (getVerticalDirection(fromY, toY) * multiplicator);
	}
	
	private void drawArrow(int peakX, int peakY, GC gc, int direction, boolean ghost) {
		int[] pts;
		if (direction == Const.AB_DIRECTION_NORTH) pts = new int[] { peakX, peakY, peakX-Const.ARROW_HALFBASE, peakY+Const.ARROW_HEIGHT, peakX+Const.ARROW_HALFBASE, peakY+Const.ARROW_HEIGHT };
		else if (direction == Const.AB_DIRECTION_SOUTH) pts = new int[] { peakX, peakY, peakX-Const.ARROW_HALFBASE, peakY-Const.ARROW_HEIGHT, peakX+Const.ARROW_HALFBASE, peakY-Const.ARROW_HEIGHT };
		else if (direction == Const.AB_DIRECTION_WEST) pts = new int[] { peakX, peakY, peakX+Const.ARROW_HEIGHT, peakY-Const.ARROW_HALFBASE, peakX+Const.ARROW_HEIGHT, peakY+Const.ARROW_HALFBASE };
		else if (direction == Const.AB_DIRECTION_EAST) pts = new int[] { peakX, peakY, peakX-Const.ARROW_HEIGHT, peakY-Const.ARROW_HALFBASE, peakX-Const.ARROW_HEIGHT, peakY+Const.ARROW_HALFBASE };
		else pts = new int[] {};
		if (ghost) transparency(gc);
		Color originalBG = gc.getBackground();
		gc.setBackground(display.getSystemColor(SWT.COLOR_BLACK));
		gc.fillPolygon(pts);
		gc.setBackground(originalBG);
		if (ghost) untransparency(gc);
	}
	
	private void drawPath(int fromX, int fromY, int toX, int toY, GC gc, boolean isFinal, boolean ghost) {
		
		Stack<int[]> s = new Stack<int[]>();
		s.push(new int[] {fromX, fromY, toX, toY});
		
		int[] step; 
		int dir;
		
		while (! s.empty()) {
			step = s.pop();
			dir = getDirection(step[0], step[1], step[2], step[3]);
			if ((dir == Const.AB_DIRECTION_NORTH) || (dir == Const.AB_DIRECTION_SOUTH) 
					|| (dir == Const.AB_DIRECTION_WEST) || (dir == Const.AB_DIRECTION_EAST)) {
				if (ghost) {
					dot(gc);
					transparency(gc);
				}
				gc.drawLine(step[0], step[1], step[2], step[3]);
				if (ghost) {
					undot(gc);
					untransparency(gc);
				}
				if (isFinal && s.empty()) drawArrow(toX, toY, gc, dir, ghost);
			} else if (dir != Const.AB_DIRECTION_SAME_POINT) {
				s.push(new int[] {step[2], step[1], step[2], step[3]});
				s.push(new int[] {step[0], step[1], step[2], step[1]});
			}
		}
		
	}
	
	private boolean bothHorizontal(ConnectPoint from, ConnectPoint to) {
		return (from.isHorizontal() && to.isHorizontal());
	}
	
	private boolean bothVertical(ConnectPoint from, ConnectPoint to) {
		return ((! from.isHorizontal()) && (! to.isHorizontal()));
	}
	
	private void drawConnection(ConnectPoint from, ConnectPoint to, GC gc, boolean ghost) {
		
		boolean arrow = (!to.isAssistant());
		
		int[] fromOffset = from.getOffsetPoint();
		int[] toOffset = to.getOffsetPoint();
			
		int diffX = Math.abs(from.getX() - to.getX());
		int diffY = Math.abs(from.getY() - to.getY());
			
		if ((diffX < 2*Const.NODE_CONNECTION_OFFSET) && bothHorizontal(from, to)) {
			fromOffset = from.getOffsetPoint(diffX/2);
			toOffset = to.getOffsetPoint(diffX/2);
			if (diffX/2 <= Const.ARROW_HEIGHT) arrow = false;
		}
			
		if ((diffY < 2*Const.NODE_CONNECTION_OFFSET) && bothVertical(from, to)) {
			fromOffset = from.getOffsetPoint(diffY/2);
			toOffset = to.getOffsetPoint(diffY/2);
			if (diffY/2 <= Const.ARROW_HEIGHT) arrow = false;
		}
			
		if ((diffX < Const.NODE_CONNECTION_OFFSET) && (! bothVertical(from, to)) && (! bothHorizontal(from, to))) {
			if (from.isHorizontal()) fromOffset = from.getOffsetPoint(diffX);
			if (to.isHorizontal()) toOffset = to.getOffsetPoint(diffX);
			if ((diffX <= Const.ARROW_HEIGHT) && to.isHorizontal()) 
				arrow = false;
		}
			
		if ((diffY < Const.NODE_CONNECTION_OFFSET) && (! bothVertical(from, to)) && (! bothHorizontal(from, to))) {
			if (! from.isHorizontal()) fromOffset = from.getOffsetPoint(diffY);
			if (! to.isHorizontal()) toOffset = to.getOffsetPoint(diffY);
			if ((diffY <= Const.ARROW_HEIGHT) && (! to.isHorizontal())) 
				arrow = false;
		}
		
		drawPath(from.getX(), from.getY(), fromOffset[0], fromOffset[1], gc, false, ghost);
		drawPath(fromOffset[0], fromOffset[1], toOffset[0], toOffset[1], gc, false, ghost);
		drawPath(toOffset[0], toOffset[1], to.getX(), to.getY(), gc, arrow, ghost);
		
		if (to.isAssistant() && to.hasOpposites())
			drawConnection(to, to.getOpposites().element(), gc, ghost);
	
	}
	
	public void drawConnections(GC gc) {
		ConnectPoint[] cps;
		if (drawingGhostConnection)
			drawConnection(connectingFrom, ghostConnectPoint, gc, true);
		for (int i = 0; i < nodes.size(); i++) {
			cps = nodes.get(i).getConnectPoints();
			for (int j = 0; j < cps.length; j++) 
				if ((! cps[j].isInput()) && (cps[j].hasOpposites()))
					for (ConnectPoint cp : cps[j].getOpposites())
						drawConnection(cps[j], cp, gc, false);
		}
	}
	
	private void drawGridLine(GC gc, boolean vertical) {
		
		if ((ghost != null) || (onCursor != null)) {
			
			dot(gc);
			transparency(gc);
			
			if (vertical) gc.drawLine(dynamicX, 0, dynamicX, super.getClientArea().height);
			else gc.drawLine(0, dynamicY, super.getClientArea().width, dynamicY);
			
			untransparency(gc);
			undot(gc);
		
		}
		
	}
	
	public void drawVerticalGridLine(GC gc) {
		if (verGridLine) drawGridLine(gc, true);
	}
	
	public void drawHorizontalGridLine(GC gc) {
		if (horGridLine) drawGridLine(gc, false);
	}
	
	private boolean checkNode(GraphNode node, int dynamic, boolean vertical) {
		if (node.isMoving()) return false;
		else if (vertical) return (node.getPositionX() == dynamic);
		else return (node.getPositionY() == dynamic);
	}
	
	private boolean checkNodes(int dynamic, boolean vertical) {
		for (int i = 0; i < nodes.size(); i++) {
			if (checkNode(nodes.get(i), dynamic, vertical))
				return true;
		}
		return false;
	}
	
	public void toDrawVerticalGridLine(int x) {
		verGridLine = ((x == centerX) || checkNodes(x, true));
		dynamicX = x;
		this.redraw();
	}
	
	public void toDrawHorizontalGridLine(int y) {
		horGridLine = ((y == centerY) || checkNodes(y, false));
		dynamicY = y;
		this.redraw();
	}
	
	public void drawGridLines(int x, int y) {
		toDrawVerticalGridLine(x);
		toDrawHorizontalGridLine(y);
	}
	
	public void recenter() {
		Rectangle area = super.getClientArea();
		centerX = Math.round(area.width / 2);
		centerY = Math.round(area.height / 2);
	}
	
	private void discardSelection() {
		preparationSelected = null;
		for (int i = 0; i < nodes.size(); i++)
			nodes.get(i).setSelected(false);
		mainShell.setEditNodeMenuItem(false);
	}
	
	private GraphNode find(int x, int y, boolean selected) {
		GraphNode ret = null;
		for (int i = 0; i < nodes.size(); i++) {
			GraphNode n = nodes.get(i);
			if (n.isInside(x, y) 
					&& ((! selected) || (selected && n.isSelected()))) {
				ret = n;
				break;
			}
		}
		return ret;
	}
	
	private boolean editingNode(GraphNode n) {
		return ((n.getOperation() != null) && (! multipleSelection));
	}
	
	private void selectNode(GraphNode n) {
		n.setSelected(true);
		if (n.getClass().equals(PreparationGraphNode.class))
			preparationSelected = (PreparationGraphNode)n;
		selectedNode = n;
		mainShell.setEditNodeMenuItem(editingNode(selectedNode));
	}
	
	public GraphNode select(int x, int y) {
		
		if (! multipleSelection) discardSelection();
		
		GraphNode n = find(x, y, false);
		if (n != null) {
			if (! n.isSelected()) {
				selectNode(n);
			} else if (multipleSelection) {
				n.setSelected(false);
			}
		}
		
		this.redraw();
		return n;
		
	}
	
	private void setDragFrom(int x, int y) {
		dragFromX = x;
		dragFromY = y;
	}
	
	private void setMovingNodes(boolean val) {
		for (int i = 0; i < nodes.size(); i++) { 
			GraphNode n = nodes.get(i);
			if (n.isSelected()) {
				n.setGhost(val);
				n.setMoving(val);
				if (val) movingAction.addMovingNode(n);
			}
		}
	}
	
	private void setMultipleMarqueeSelection(boolean val) {
		marqueeSelection = val;
		multipleSelection = val;
	}
	
	private void setDragging(boolean val) {
		
		if (val) {	
			onCursor = find(dragFromX, dragFromY, true);
			if ((onCursor == null) && (dndResize == Const.CANVAS_DND_RESIZE_NONE)) 
				setMultipleMarqueeSelection(val);
			else {
				dragging = val;
				if (onCursor != null) {
					movingAction = new MovingAction(this, dragFromX, dragFromY);
					setMovingNodes(val);
				} else 
					resizingAction = new ResizingAction(this, this.getClientArea().width, this.getClientArea().height);
			}
		} else {
			onCursor = null;
			if (dragging) {
				dragging = val;
				setMovingNodes(val);
				if (movingAction != null) {
					recordAction(movingAction);
					movingAction = null;
				}
				if (resizingAction != null) {
					recordAction(resizingAction);
					resizingAction = null;
				}
				adjustMinimalSize();
			} else if (marqueeSelection) setMultipleMarqueeSelection(val);
			
			if (dndResize != Const.CANVAS_DND_RESIZE_NONE) {
				dndResize = Const.CANVAS_DND_RESIZE_NONE;
				setCanvasCursor();
			}
		}
		this.redraw();
		
	}
	
	public void dragOn(int x, int y) {
		setDragFrom(x, y);
		setDragging(true);
	}
	
	public void dragOff() {
		setDragging(false);
	}
	
	public boolean isDraggingOn() {
		return dragging;
	}
	
	
	public boolean isMarqueeOn() {
		return marqueeSelection;
	}
	
	private boolean isInside(int x, int y) {
		return (((0 < x) && (x < this.getClientArea().width)) 
				&& ((0 < y) && (y < this.getClientArea().height)));
	}
	
	public boolean nodesSelected() {
		for (int i = 0; i < nodes.size(); i++) {
			if (nodes.get(i).isSelected()) return true;
		}
		return false;
	}
	
	public void moveNode(GraphNode n, int dx, int dy) {
		for (int i = 0; i < nodes.size(); i++) {
			GraphNode x = nodes.get(i);
			if (x.equals(n)) {
				x.move(dx, dy);
				adjustMinimalSize();
				mainShell.setSavedFlag();
				this.redraw();
				return;
			}
		}
	}
	
	public void moveSelected(int x, int y) {
		
		if (nodesSelected() && (onCursor != null)) {
			
			int dx = x - dragFromX;
			int dy = y - dragFromY;
			
			for (int i = 0; i < nodes.size(); i++) {
				GraphNode n = nodes.get(i);
				if (n.isSelected() && isInside(n.getPositionX() + dx, n.getPositionY() + dy)) 
					n.move(dx, dy);
			}
			
			dragFromX += dx;
			dragFromY += dy;
			
			movingAction.updateDestination(dragFromX, dragFromY);
			
			drawGridLines(onCursor.getPositionX(), onCursor.getPositionY());
			
		}
		
		this.redraw();
		
	}
	
	public void removeNode(GraphNode n) {
		nodes.remove(n);
		if (n.equals(program.getStart()))
			program.discardStart();
		for (ConnectPoint cp : n.getConnectPoints())
			if (cp.hasOpposites())
				for (ConnectPoint opp : cp.getOpposites()) {
					while (opp.isAssistant()) {
						if (cp.isInput()) opp = opp.getOrigin();
						else opp = opp.getOpposites().element();
					}
					removeConnection(cp, opp);
				}
		mainShell.setSavedFlag();
		semaphore.updateState();
		adjustMinimalSize();
		this.redraw();
	}
	
	public void deleteSelected() {
		int i = 0;
		RemovingAction a = new RemovingAction(this);
		while (nodesSelected()) {
			GraphNode n = nodes.get(i);
			if (n.isSelected()) {
				for (ConnectPoint cp : n.getConnectPoints())
					if (cp.hasOpposites())
						for (ConnectPoint opp : cp.getOpposites()) {
							LinkedList<AssistantConnectPoint> path = new LinkedList<AssistantConnectPoint>();
							ConnectPoint p = opp;
							boolean in = cp.isInput();
							while (p.isAssistant()) {
								if (in) path.addFirst((AssistantConnectPoint)p);
								else path.add((AssistantConnectPoint)p);
								if (in) p = p.getOrigin();
								else p = p.getOpposites().element();
							}
							if (in) a.addConnection(p, path, cp);
							else a.addConnection(cp, path, p);
						}
				removeNode(n);
				a.addRemovedNode(n);
			} else i++;
		}
		recordAction(a);
	}
	
	private Menu generalMenu(Menu m) {
		
		MenuItem mntmCut = new MenuItem(m, SWT.NONE);
		mntmCut.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				mainShell.cut();
			}
		});
		mntmCut.setText(lang.cut);
		
		MenuItem mntmCopy = new MenuItem(m, SWT.NONE);
		mntmCopy.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				mainShell.copy();
			}
		});
		mntmCopy.setText(lang.copy);
		
		MenuItem mntmPaste = new MenuItem(m, SWT.NONE);
		mntmPaste.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				mainShell.paste();
			}
		});
		mntmPaste.setText(lang.paste);
		
		return m;
		
	}
	
	public void setCopyClip(boolean val) {
		instantNodeContextMenu.getItem(Const.NODE_CONTEXT_MENU_CUT_ITEM_INDEX).setEnabled(val);
		instantNodeContextMenu.getItem(Const.NODE_CONTEXT_MENU_COPY_ITEM_INDEX).setEnabled(val);
		instantCanvasContextMenu.getItem(Const.CANVAS_CONTEXT_MENU_CUT_ITEM_INDEX).setEnabled(val);
		instantCanvasContextMenu.getItem(Const.CANVAS_CONTEXT_MENU_COPY_ITEM_INDEX).setEnabled(val);
	}
	
	public void setPasteClip(boolean val) {
		instantNodeContextMenu.getItem(Const.NODE_CONTEXT_MENU_PASTE_ITEM_INDEX).setEnabled(val);
		instantCanvasContextMenu.getItem(Const.CANVAS_CONTEXT_MENU_PASTE_ITEM_INDEX).setEnabled(val);
	}
	
	public void setUndoRedo() {
		instantCanvasContextMenu.getItem(Const.CANVAS_CONTEXT_MENU_UNDO_ITEM_INDEX).setEnabled(! undoStack.empty());
		instantCanvasContextMenu.getItem(Const.CANVAS_CONTEXT_MENU_REDO_ITEM_INDEX).setEnabled(! redoStack.empty());
	}
	
	public void setUndoRedo(boolean val) {
		instantCanvasContextMenu.getItem(Const.CANVAS_CONTEXT_MENU_UNDO_ITEM_INDEX).setEnabled(val);
		instantCanvasContextMenu.getItem(Const.CANVAS_CONTEXT_MENU_REDO_ITEM_INDEX).setEnabled(val);
	}
	
	public void setToggleBreakpoint(boolean val) {
		instantNodeContextMenu.getItem(Const.NODE_CONTEXT_MENU_TOGGLEBREAKPOINT_ITEM_INDEX).setEnabled(val);
	}
	
	private void addDeleteItem(Menu m) {
		new MenuItem(m, SWT.SEPARATOR);
		MenuItem mntmDeleteItem = new MenuItem(m, SWT.NONE);
		mntmDeleteItem.setImage(SWTResourceManager.getImage(this.getClass(), "/icons/delete.png"));
		mntmDeleteItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				deleteSelected();
			}
		});
		mntmDeleteItem.setText(lang.delete);
	}
	
	public void toggleBreakpoint() {
		for (GraphNode n : nodes)
			if (n.isSelected()) {
				n.toggleBreakpoint();
				this.redraw();
				return;
			}
	}
	
	private void addToggleBreakpointItem(Menu m) {
		new MenuItem(m, SWT.SEPARATOR);
		MenuItem mntmToggleBreakpointItem = new MenuItem(m, SWT.NONE);
		mntmToggleBreakpointItem.setText(lang.toggleBreakpoint);
		mntmToggleBreakpointItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				toggleBreakpoint();
			}
		});
	}
	
	private void createNodeContextMenu() {
		instantNodeContextMenu = generalMenu(new Menu(this));
		addToggleBreakpointItem(instantNodeContextMenu);
		addDeleteItem(instantNodeContextMenu);
	}
	
	public void showSettings() {
		if (selectedNode != null) {
			if (selectedNode.getClass().equals(DecisionGraphNode.class)) 
				mainShell.showConditionSettingsShell(mainShell.getShell(), (Comparison)selectedNode.getOperation(), true);
			else if (selectedNode.getClass().equals(PreparationGraphNode.class)) 
				mainShell.showPreparationSettingsShell(preparationSelected.getCondition(), (Assignment)selectedNode.getOperation(), true);
			else if (selectedNode.getClass().equals(ProcessGraphNode.class)) 
				mainShell.showAssignmentSettingsShell(mainShell.getShell(), (Assignment)selectedNode.getOperation(), true, true);
			else if (selectedNode.getClass().equals(ManualInputGraphNode.class)) 
				mainShell.showInputOutputSettingsShell((ManualInputGraphNode)selectedNode, true, true);
			else if (selectedNode.getClass().equals(DisplayGraphNode.class)) 
				mainShell.showInputOutputSettingsShell((DisplayGraphNode)selectedNode, false, true);
		}
	}
	
	private void createOperationContextMenu() {
		instantOperationNodeContextMenu = generalMenu(new Menu(this));
		this.addToggleBreakpointItem(instantOperationNodeContextMenu);
		MenuItem mntmEditOperationItem = new MenuItem(instantOperationNodeContextMenu, SWT.NONE);
		mntmEditOperationItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				showSettings();
			}
		});
		mntmEditOperationItem.setText(lang.editNode + Const.ELLIPSIS);
		addDeleteItem(instantOperationNodeContextMenu);
	}
	
	private void createCanvasContextMenu() {
		
		instantCanvasContextMenu = new Menu(this);
		
		MenuItem mntmUndo = new MenuItem(instantCanvasContextMenu, SWT.NONE);
		mntmUndo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				mainShell.undo();
			}
		});
		mntmUndo.setText(lang.undo);
		
		MenuItem mntmRedo = new MenuItem(instantCanvasContextMenu, SWT.NONE);
		mntmRedo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				mainShell.redo();
			}
		});
		mntmRedo.setText(lang.redo);
		
		new MenuItem(instantCanvasContextMenu, SWT.SEPARATOR);
		instantCanvasContextMenu = generalMenu(instantCanvasContextMenu);
		
		new MenuItem(instantCanvasContextMenu, SWT.SEPARATOR);
		
		MenuItem mntmCanvasSizeItem = new MenuItem(instantCanvasContextMenu, SWT.NONE);
		mntmCanvasSizeItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				mainShell.showCanvasSizeShell();
			}
		});
		mntmCanvasSizeItem.setText(lang.canvasSize + Const.ELLIPSIS);
	
	}
	
	private void locateShowContextMenu() {
		contextMenu.setLocation(contextMenuAxis.x, contextMenuAxis.y);
		contextMenu.setVisible(true);
	}
	
	public boolean oneNodeSelected() {
		boolean oneSelected = false;
		for (int i = 0; i < nodes.size(); i++) 
			if (nodes.get(i).isSelected() && (! oneSelected)) oneSelected = true;
			else if (nodes.get(i).isSelected() && oneSelected) return false; 
		return oneSelected;
	}
	
	public void showContextMenu(int x, int y) {
		GraphNode n = find(x, y, true);
		if (n != null) {
			n.setContextMenu(! oneNodeSelected());
			locateShowContextMenu();
		} else if (select(x, y) != null) showContextMenu(x, y);
		else {
			setCanvasContextMenu();
			locateShowContextMenu();
		}
	}
	
	public void hideContextMenu() {
		if (contextMenu != null) contextMenu.setVisible(false);
	}
	
	public void setNodeContextMenu() {
		contextMenu = instantNodeContextMenu;
	}
	
	public void setEditOperationNodeContextMenu() {
		contextMenu = instantOperationNodeContextMenu;
	}
	
	public void setCanvasContextMenu() {
		contextMenu = instantCanvasContextMenu;
	}
	
	public void setContextMenuAxis(Point pt) {
		contextMenuAxis = pt;
	}
	
	public void setMultipleSelection(boolean val) {
		multipleSelection = val;
		this.redraw();
	}
	
	public void updateMultipleIndicator(int x, int y) {
		multipleIndicatorX = x + Const.MULTIPLE_INDICATOR_X_OFFSET;
		multipleIndicatorY = y + Const.MULTIPLE_INDICATOR_Y_OFFSET;
		this.redraw();
	}
	
	public void indicateMultipleSelection(GC gc) {
		if (multipleSelection && (! marqueeSelection)) 
			gc.drawText("±", multipleIndicatorX, multipleIndicatorY, true);
	}
	
	public void marqueeSelect(int x, int y) {
		
		if (marqueeSelection) {
			
			marqueeRectWidth = x - dragFromX;
			marqueeRectHeight = y - dragFromY;
			
			for (int i = 0; i < nodes.size(); i++) {
				GraphNode n = nodes.get(i);
				if ((n.isInRectangle(dragFromX, dragFromY, marqueeRectWidth, marqueeRectHeight)) 
						&& (! n.isSelected()))
					selectNode(n);
			}
			
			this.redraw();
		
		}
		
	}
	
	public void drawMarqueeRect(GC gc) {
		if (marqueeSelection) {
			transparency(gc);
			dot(gc);
			gc.drawRectangle(dragFromX, dragFromY, marqueeRectWidth, marqueeRectHeight);
			undot(gc);
			untransparency(gc);
		}
	}
	
	public int getMinimalWidth() {
		return minimalWidth;
	}
	
	public int getMinimalHeight() {
		return minimalHeight;
	}
	
	private void setCanvasCursor() {
		
		if (dndResize == Const.CANVAS_DND_RESIZE_NONE) 
			this.setCursor(display.getSystemCursor(SWT.CURSOR_ARROW));
		else if (dndResize == Const.CANVAS_DND_RESIZE_EAST)
			this.setCursor(display.getSystemCursor(SWT.CURSOR_SIZEE));
		else if (dndResize == Const.CANVAS_DND_RESIZE_SOUTH)
			this.setCursor(display.getSystemCursor(SWT.CURSOR_SIZES));
		else if (dndResize == Const.CANVAS_DND_RESIZE_SOUTHEAST)
			this.setCursor(display.getSystemCursor(SWT.CURSOR_SIZESE));
	
	}
	
	public void setDndResize(int x, int y) {
		if (! dragging) {
			dndResize = Const.CANVAS_DND_RESIZE_NONE;
			if (((this.getClientArea().width - Const.DND_RESIZE_TOLERANCES) < x) && (x < this.getClientArea().width)) 
				dndResize += Const.CANVAS_DND_RESIZE_EAST;
			if (((this.getClientArea().height - Const.DND_RESIZE_TOLERANCES) < y) && (y < this.getClientArea().height)) 
				dndResize += Const.CANVAS_DND_RESIZE_SOUTH;
			setCanvasCursor();
		}
	}
	
	public int getDndResize() {
		return dndResize;
	}
	
	public String copy() {
		String ret = "";
		for (int i = 0; i < nodes.size(); i++) {
			GraphNode n = nodes.get(i);
			if (n.isSelected()) ret = n.copy(ret);
		}
		return ret;
	}
	
	private void generateNode(String[] pasteData, LinkedList<GraphNode> l) {
		GraphNode n = null;
		if ((pasteData[0].equals(Const.START_NODE_CODE)) && (! program.hasStart())) 
			n = new StartGraphNode(this, false);
		else if (pasteData[0].equals(Const.END_NODE_CODE))
			n = new EndGraphNode(this, false);
		else if (pasteData[0].equals(Const.DECISION_CODE))
			n = new DecisionGraphNode(this, false);
		else if (pasteData[0].equals(Const.PROCESS_CODE))
			n = new ProcessGraphNode(this, false);
		else if (pasteData[0].equals(Const.PREPARATION_CODE))
			n = new PreparationGraphNode(this, false);
		else if (pasteData[0].equals(Const.MANUAL_INPUT_CODE))
			n = new ManualInputGraphNode(this, false);
		else if (pasteData[0].equals(Const.DISPLAY_CODE))
			n = new DisplayGraphNode(this, false);
		if (n != null) {
			Rectangle canvasRect = this.getClientArea();
			if (n.paste(pasteData) 
					&& n.isInRectangle(canvasRect.x, canvasRect.y, canvasRect.width, canvasRect.height)) {
				l.add(n);
			}
		}
	}
	
	public void trackMousePosition(int x, int y) {
		mouseX = x;
		mouseY = y;
	}
	
	public void paste(String s) {
		
		String[] items = s.split("\\" + Const.CLIPBOARD_TEXTDATA_SEPARATOR);
		if (items.length != 0) discardSelection();
		LinkedList<GraphNode> pasted = new LinkedList<GraphNode>();
		for (int i = 0; i < items.length; i++) {
			items[i] = items[i].trim();
			String pasteData[] = items[i].split(Const.SPACE);
			generateNode(pasteData, pasted);
		}
		
		AddingAction a = new AddingAction(this);
		for (int i = 0; i < pasted.size(); i++) {
			GraphNode n = pasted.get(i);
			n.setPosition(mouseX, mouseY);
			selectNode(n);
		}
		if (!pasted.isEmpty()) addNodesRecorded(pasted, a);
	
	}
	
	public void setSize(int width, int height) {
		mainShell.changeCanvasSize(width, height);
		mainShell.setSavedFlag();
	}
	
	private void setSizeRecorded(int width, int height) {
		setSize(width, height);
		resizingAction.setNewSize(width, height);
	}
	
	public void dndResize(int x, int y) {
		if (this.isDraggingOn()) {
			if (this.getDndResize() == Const.CANVAS_DND_RESIZE_EAST)
				setSizeRecorded(x, this.getClientArea().height);
			else if (this.getDndResize() == Const.CANVAS_DND_RESIZE_SOUTH)
				setSizeRecorded(this.getClientArea().width, y);
			else if (this.getDndResize() == Const.CANVAS_DND_RESIZE_SOUTHEAST)
				setSizeRecorded(x, y);
		}
	}
	
	public void undo() {
		if (! undoStack.isEmpty()) {
			Action a = undoStack.pop();
			a.undo();
			redoStack.push(a);
		}
	}
	
	public void redo() {
		if (! redoStack.isEmpty()) {
			Action a = redoStack.pop();
			a.redo();
			undoStack.push(a);
		}
	}
	
	public void recordAction(Action a) {
		undoStack.push(a);
		redoStack.clear();
		mainShell.setSavedFlag();
		mainShell.adjustUndoRedo();
	}
	
	public boolean isUndoStackEmpty() {
		return undoStack.empty();
	}
	
	public boolean isRedoStackEmpty() {
		return redoStack.empty();
	}
	
	public Display getDisplay() {
		return display;
	}
	
	public LinkedList<Variable> getVariables() {
		return variables;
	}
	
	public void addVariable(Variable v) {
		variables.add(v);
	}
	
	public void lock() {
		locked = true;
		setMultipleSelection(false);
	}
	
	public void unlock() {
		locked = false;
	}
	
	public boolean locked() {
		return locked;
	}
	
	public void drawBreakpoint(int x, int y, GC gc) {
		Color defColor = gc.getBackground();
		gc.setAntialias(SWT.ON);
		gc.setBackground(display.getSystemColor(SWT.COLOR_RED));
		gc.fillOval(x-4, y-4, 8, 8);
		gc.setBackground(defColor);
	}
	
	public void drawConnectPoint(GC gc) {
		transparency(gc);
		if (activeFreeConnectPoint != null) {
			dot(gc);
			if (! validActiveConnectPoint) gc.setForeground(display.getSystemColor(SWT.COLOR_RED));
			gc.drawOval(activeFreeConnectPoint.getX()-4, activeFreeConnectPoint.getY()-4, 8, 8);
			gc.setForeground(display.getSystemColor(SWT.COLOR_BLACK));
			undot(gc);
			
		} else if (activeRemoveConnectPoint != null) {
			int x = activeRemoveConnectPoint.getX(), y = activeRemoveConnectPoint.getY();
			gc.drawLine(x-4, y-4, x+4, y+4);
			gc.drawLine(x+4, y-4, x-4, y+4);
		} else if (editIndicator != null) {
			int x = editIndicator.getX(), y = editIndicator.getY();
			gc.drawLine(x+4, y-4, x-4, y+4);
		}
		untransparency(gc);
	}
	
	private boolean toIndicateFreeConnectPoint(ConnectPoint cp) {
		return ((((connectingFrom == null) && (! cp.isInput())) || ((connectingFrom != null) && cp.isInput())) && ((!cp.hasOpposites()) || ((connectingFrom != null) && connectingFrom.isFromDecision())));
	}
	
	private boolean toIndicateRemoveConnectPoint(ConnectPoint cp) {
		return (((connectingFrom == null) && (! cp.isInput()) && (cp.hasOpposites())));
	}
	
	private boolean toIndicateEditConnectPoint(ConnectPoint cp) {
		return ((connectingFrom == null) && cp.isInput() && cp.hasOpposites() && (cp.getOpposites().size() == 1));
	}
	
	private boolean findRemoveEdit(GraphNode gn, int x, int y) {
		ConnectPoint cp = gn.connect(x, y);
		if (cp != null) {
			if (toIndicateRemoveConnectPoint(cp)) {
				activeRemoveConnectPoint = cp;
				return true;
			} else if (toIndicateEditConnectPoint(cp)) {
				editIndicator = cp;
				return true;
			}
		}
		return false;
	}
	
	public void findConnectPoint(int x, int y) {
		
		ConnectPoint cp;
		
		activeFreeConnectPoint = null;
		activeRemoveConnectPoint = null;
		editIndicator = null;
		validActiveConnectPoint = true;
		
		for (GraphNode gn : nodes) {
			cp = gn.connect(x, y, (connectingFrom != null));
			if (cp != null) {
				if (toIndicateFreeConnectPoint(cp)) {
					activeFreeConnectPoint = cp;
					if (connectingFrom != null)
						if (! activeFreeConnectPoint.connectable(connectingFrom))
							validActiveConnectPoint = false;
					break;
				} else if (findRemoveEdit(gn, x, y)) break;
			} else if (findRemoveEdit(gn, x, y)) break;
		}
		this.redraw();
	}
	
	public void setConnectingFrom() {
		connectingFrom = activeFreeConnectPoint;
		drawingGhostConnection = true;
	}
	
	public void discardConnectingFrom() {
		if ((connectingFrom != null) && connectingFrom.isAssistant())
			connectingFrom.removeAssistant(true);
		connectingFrom = null;
		activeFreeConnectPoint = null;
		activeRemoveConnectPoint = null;
		drawingGhostConnection = false;
		this.redraw();
	}
	
	public ConnectPoint getConnectingFrom() {
		return connectingFrom;
	}
	
	public void setConnectingTo() {
		if (validActiveConnectPoint)
			connectingTo = activeFreeConnectPoint;
			if ((connectingFrom != null) && (connectingTo != null)) {
				LinkedList<AssistantConnectPoint> path = new LinkedList<AssistantConnectPoint>();
				ConnectPoint p = connectingFrom;
				while (p.isAssistant()) {
					path.addFirst((AssistantConnectPoint)p);
					p = p.getOrigin();
					p.removeOpposite(path.element());
				}
				addConnection(p, path, connectingTo);
				AddingConnectionAction a = new AddingConnectionAction(this, p, path, connectingTo);
				recordAction(a);
				connectingFrom = null;
				connectingTo = null;
				drawingGhostConnection = false;
				mainShell.discardNotification();
				this.redraw();
			}
	}
	
	public ConnectPoint getConnectingTo() {
		return connectingTo;
	}
	
	public ConnectPoint getPointedToRemove() {
		return activeRemoveConnectPoint;
	}
	
	public ConnectPoint getPointedToConnect() {
		return activeFreeConnectPoint;
	}
	
	public ConnectPoint getPointedToEdit() {
		return editIndicator;
	}
	
	public ConnectPoint getEditingConnectPoint() {
		return activeEditingConnectPoint;
	}
	
	public void removePointedConnection() {
		if (activeRemoveConnectPoint.equals(connectingFrom))
			discardConnectingFrom();
		else if ((activeRemoveConnectPoint != null) && (activeRemoveConnectPoint.hasOpposites())) {
			LinkedList<AssistantConnectPoint> path = new LinkedList<AssistantConnectPoint>();
			ConnectPoint to = activeRemoveConnectPoint.getOpposites().element();
			while (to.isAssistant()) {
				path.add((AssistantConnectPoint)to);
				to = to.getOpposites().element();
			}
			RemovingConnectionAction a = new RemovingConnectionAction(this, activeRemoveConnectPoint, path, to);
			removeConnection(activeRemoveConnectPoint, to);
			recordAction(a);
		}
	}
	
	public void editConnectionMode() {
		activeEditingConnectPoint = editIndicator;
		ConnectPoint editingOpposite = activeEditingConnectPoint.getOpposites().element();
		ConnectPoint opposite = editingOpposite;
		while (opposite.isAssistant()) {
			pathBackup.addFirst((AssistantConnectPoint)opposite);
			opposite = opposite.getOrigin();
		}
		connectingFrom = opposite;
		editingOpposite.removeAssistant(true);
		connectingFrom.removeOpposite(activeEditingConnectPoint);
		activeEditingConnectPoint.removeOpposite(editingOpposite);
		drawingGhostConnection = true;
		this.redraw();
	}
	
	public void commitConnectionEdit() {
		
		if (this.validActiveConnectPoint) {
			
			LinkedList<AssistantConnectPoint> newPath = new LinkedList<AssistantConnectPoint>();
			ConnectPoint p = connectingFrom;
			while (p.isAssistant()) {
				newPath.addFirst((AssistantConnectPoint)p);
				p = p.getOrigin();
			}
			
			for (Connection c : connections)
				if (c.fits(p, activeEditingConnectPoint)) {
					c.setPath(newPath);
					c.setTo(activeFreeConnectPoint);
				}
			
			connectingFrom.addOpposite(activeFreeConnectPoint);
			activeFreeConnectPoint.addOpposite(connectingFrom);
			
			EditingConnectionAction a = new EditingConnectionAction(this, p, pathBackup, newPath, activeEditingConnectPoint, activeFreeConnectPoint);
			recordAction(a);
			
			pathBackup.clear();
			connectingFrom = null;
			activeEditingConnectPoint = null;
			activeFreeConnectPoint = null;
			drawingGhostConnection = false;
			this.redraw();
		
		}
	}
	
	public void discardEditConnectionMode() {
		
		while (connectingFrom.isAssistant()) {
			connectingFrom.getOrigin().removeOpposite(connectingFrom);
			connectingFrom = connectingFrom.getOrigin();
		}
		
		ConnectPoint p = connectingFrom;
		for (AssistantConnectPoint a : pathBackup) {
			p.addOpposite(a);
			p = a;
		}
		
		p.addOpposite(activeEditingConnectPoint);
		activeEditingConnectPoint.addOpposite(p);
		pathBackup.clear();
		connectingFrom = null;
		activeEditingConnectPoint = null;
		drawingGhostConnection = false;
		this.redraw();
	
	}
	
	public boolean editingConnection() {
		return (activeEditingConnectPoint != null);
	}
	
	public void addConnection(ConnectPoint from, LinkedList<AssistantConnectPoint> path, ConnectPoint to) {
		connections.add(new Connection(from, path, to));
		ConnectPoint p = from;
		for (AssistantConnectPoint a : path) {
			p.addOpposite(a);
			p = a;
		}
		p.addOpposite(to);
		to.addOpposite(p);
		mainShell.setSavedFlag();
		semaphore.updateState();
		this.redraw();
	}
	
	public void removeConnection(ConnectPoint from, ConnectPoint to) {
		
		for (Connection c : connections)
			if (c.fits(from, to)) {
				connections.remove(c);
				break;
			}
		
		boolean in = from.isInput();
		for (ConnectPoint cp : from.getOpposites())
			if (cp.isReachable(to, in)) {
				cp.removeAssistant(in);
				from.removeOpposite(cp);
				break;
			}
		
		to.removeOpposite(from);
		mainShell.setSavedFlag();
		semaphore.updateState();
		this.redraw();
	
	}
	
	public boolean validActiveConnectPoint() {
		return validActiveConnectPoint;
	}
	
	public void updateGhostConnectPoint(int x, int y) {
		ghostConnectPoint.setPosition(x, y);
		if (connectingFrom != null)
			if (((ghostConnectPoint.getY() > connectingFrom.getY()) 
					&& (ghostConnectPoint.getOffset() >= 0))
				|| ((ghostConnectPoint.getY() < connectingFrom.getY()) 
					&& (ghostConnectPoint.getOffset() < 0)))
				ghostConnectPoint.inverseOffset();
		this.redraw();
	}
	
	public void placeAssistantConnectPoint() {
		AssistantConnectPoint a = new AssistantConnectPoint(ghostConnectPoint.getX(), ghostConnectPoint.getY(), connectingFrom);
		connectingFrom.addOpposite(a);
		connectingFrom = a;
	}
	
	public MainShell getMainShell() {
		return mainShell;
	}
	
	public boolean hasStart() {
		return program.hasStart();
	}
	
	public void updateSemaphore() {
		semaphore.updateState();
		redraw();
	}
	
	public Semaphore getSemaphore() {
		return semaphore;
	}
	
	public boolean isRunning() {
		return program.isRunning();
	}
	
	public boolean isPaused() {
		return program.isPaused();
	}
	
	public boolean isIdle() {
		return program.isIdle();
	}
	
	public void animate() {
		if (program.isRunning())
			this.nextStep();
	}
	
	public void nextStep() {
		program.nextStep();
		this.redraw();
	}
	
	public void play() {
		program.initRun();
		this.getMainShell().clearConsole();
		this.getMainShell().adjustSteppingItems();
		this.getMainShell().adjustNonSteppingItems(false);
		this.discardSelection();
		this.discardGhost();
		this.redraw();
		player.play();
	}
	
	public void pause() {
		program.pause();
	}
	
	public void stop() {
		program.terminate();
	}
	
	public void stopPlayer() {
		player.stop();
		this.getMainShell().adjustSteppingItems();
	}
	
	public LinkedList<Connection> getConnections() {
		return connections;
	}
	
	public CanvasEnvironment empty() {
		
		CanvasEnvironment e = new CanvasEnvironment(this.getClientArea().width, this.getClientArea().height);
		
		for (Connection c : connections)
			e.addConnection(c);
		
		while (!nodes.isEmpty()) {
			e.addNode(nodes.element());
			removeNode(nodes.element());
		}
		
		for (Variable v : variables)
			e.addVariable(v);
		variables.clear();
		
		this.setSize(Const.DEFAULT_CANVAS_WIDTH, Const.DEFAULT_CANVAS_HEIGHT);
		
		while (!undoStack.empty())
			e.pushUndoItem(undoStack.pop());
		
		while (!redoStack.empty())
			e.pushRedoItem(redoStack.pop());
		mainShell.adjustUndoRedo();
		
		this.redraw();
		return e;
	
	}
	
	public void recover(CanvasEnvironment e) {
		
		this.setSize(e.getCanvasWidth(), e.getCanvasHeight());
		
		for (Variable v : e.getVariables())
			addVariable(v);
		for (GraphNode gn : e.getNodes())
			addNode(gn);
		for (Connection c : e.getConnections())
			c.addTo(this);
		while (!e.getUndoStack().empty())
			undoStack.push(e.getUndoStack().pop());
		while (!e.getRedoStack().empty())
			redoStack.push(e.getRedoStack().pop());
		
		mainShell.adjustUndoRedo();
		this.redraw();
	
	}
	
	public Program getProgram() {
		return program;
	}
	
	public FlowChartCanvas(Display d, MainShell main, Composite parent, int style, Language lang) {
		
		super(parent, style);
		
		display = d;
		mainShell = main;
		this.lang = lang;
		
		instantStartGhost = new StartGraphNode(this, true);
		instantEndGhost = new EndGraphNode(this, true);
		instantProcessGhost = new ProcessGraphNode(this, true);
		instantDecisionGhost = new DecisionGraphNode(this, true);
		instantPreparationGhost = new PreparationGraphNode(this, true);
		instantManualInputGhost = new ManualInputGraphNode(this, true);
		instantDisplayGhost = new DisplayGraphNode(this, true);
		
		recenter();
		dynamicX = -1;
		dynamicY = -1;
		
		verGridLine = false;
		horGridLine = false;
		
		ghost = null;
		nodes = new LinkedList<GraphNode>();
		
		dragging = false;
		onCursor = null;
		dragFromX = -1;
		dragFromY = -1;
		
		selectedNode = null;
		contextMenu = null;
		contextMenuAxis = new Point(-1, -1);
		createNodeContextMenu();
		createCanvasContextMenu();
		createOperationContextMenu();
		
		multipleSelection = false;
		multipleIndicatorX = -1;
		multipleIndicatorY = -1;
		
		marqueeSelection = false;
		marqueeRectWidth = -1;
		marqueeRectHeight = -1;
		
		minimalWidth = 0;
		minimalHeight = 0;
		
		dndResize = Const.CANVAS_DND_RESIZE_NONE;
		
		undoStack = new Stack<Action>();
		redoStack = new Stack<Action>();
		
		movingAction = null;
		resizingAction = null;
		
		variables = new LinkedList<Variable>();
		
		locked = false;
		preparationSelected = null;
		
		mouseX = 0;
		mouseY = 0;
		
		drawingGhostConnection = false;
		ghostConnectPoint = new ConnectPoint(true, false);
		
		activeFreeConnectPoint = null;
		connectingFrom = null;
		connectingTo = null;
		
		activeRemoveConnectPoint = null;
		activeEditingConnectPoint = null;
		editIndicator = null;
		
		validActiveConnectPoint = true;
		
		pathBackup = new LinkedList<AssistantConnectPoint>();
		connections = new LinkedList<Connection>();
		
		program = new Program(this);
		semaphore = new Semaphore(program);
		
		player = new CanvasPlayer(this.getDisplay(), this);
	
	}

}
