package jflowchart.ui.canvas.graphnodes;


import jflowchart.engine.common.Const;
import jflowchart.engine.nodes.ProcessNode;
import jflowchart.ui.canvas.FlowChartCanvas;

import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;

public class ProcessGraphNode extends ProcessNode implements GraphNode {

	private FlowChartCanvas canvas;
	
	private void init(FlowChartCanvas c) {
		operation.setGraphNode(this);
		canvas = c;
	}
	
	public ProcessGraphNode(FlowChartCanvas c, boolean ghost) {
		super(ghost);
		init(c);
	}
	
	public ProcessGraphNode(FlowChartCanvas c, int x, int y, boolean ghost) {
		super(x, y, ghost);
		init(c);
	}
	
	@Override
	public void setContextMenu(boolean multipleSelection) {
		if (multipleSelection) canvas.setNodeContextMenu();
		else canvas.setEditOperationNodeContextMenu();
	}

	@Override
	public void draw(GC gc) {
		if (operation.usable()) {
			Font defFont = gc.getFont();
			FontData[] fd = defFont.getFontData();
			Font font = new Font(canvas.getDisplay(), fd[0].getName(), Const.DEFAULT_DRAWING_FONT_HEIGHT, fd[0].getStyle());
			gc.setFont(font);
			String s = operation.print();
			int fontWidth = gc.getFontMetrics().getAverageCharWidth() * s.length();
			int tolerance = Math.round(2 * Const.PROCESS_NODE_WIDTH / 3);
			if (fontWidth > tolerance) {
				float reductionRatio = (float)tolerance / fontWidth;
				int newHeight = Math.round(Const.DEFAULT_DRAWING_FONT_HEIGHT * reductionRatio);
				font.dispose();
				font = new Font(canvas.getDisplay(), fd[0].getName(), newHeight, fd[0].getStyle());
				gc.setFont(font);
			}
			gc.drawText(s, positionX - fontWidth/3, positionY - gc.getFontMetrics().getHeight() / 2, true);
			gc.setFont(defFont);
			font.dispose();
		}
		Image processImg = new Image(canvas.getDisplay(), getClass().getResourceAsStream("/flowchartSymbols/csnIso5807/process.png"));
		gc.drawImage(processImg, 0, 0, processImg.getBounds().width, processImg.getBounds().height, getWestBorder(), getNorthBorder(), width, height);
		processImg.dispose();
		if (hasBreakpoint) canvas.drawBreakpoint(connectPoints[0].getX(), connectPoints[0].getY(), gc);
	}

	@Override
	public FlowChartCanvas getCanvas() {
		return canvas;
	}

}
