package jflowchart.ui.windows;

import java.util.LinkedList;

import jflowchart.engine.arithmetics.Assignment;
import jflowchart.engine.arithmetics.ComparableValue;
import jflowchart.engine.arithmetics.ComparableVariable;
import jflowchart.engine.arithmetics.IncompatibleDataTypesException;
import jflowchart.engine.arithmetics.NumberExpression;
import jflowchart.engine.arithmetics.Operand;
import jflowchart.engine.arithmetics.Value;
import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.common.Const;
import jflowchart.engine.common.Functions;
import jflowchart.engine.common.Language;
import jflowchart.engine.history.EditingAssignmentAction;
import jflowchart.ui.canvas.FlowChartCanvas;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import swing2swt.layout.BorderLayout;
import swing2swt.layout.FlowLayout;

public class AssignmentSettingsShell extends Shell implements VariableSetting, NumberExpressionSetting {
	
	private Combo firstVariableCombo;
	private Text secondValueText;
	private Combo secondVariableCombo;
	private Button btnOk;
	private Button btnCancel;
	private Label lblMessagesLabel;
	
	private MainShell mainShell;
	private FlowChartCanvas canvas;
	private LinkedList<Variable> variables;
	private Language lang;
	private boolean edit;
	private Assignment assignmentOrig;
	private Assignment assignmentCopy;
	private EditingAssignmentAction editAction;
	private Text secondExpressionText;
	private Button btnCreateEditSecondExpression;
	
	private Button btnSecondVariable;
	private Button btnSecondValue;
	private Button btnSecondExpression;
	
	private Variable finalOp1;
	private Operand finalOp2;
	
	private Variable variableOp1;
	private Variable variableOp2;
	
	private Value valueOp2;
	
	private NumberExpression expressionOp2;
	private Label lblVariable;
	
	private boolean newVariable;
	
	private void setMessage(String msg) {
		lblMessagesLabel.setText(msg);
	}
	
	private boolean emptyFinalOp1() {
		return (finalOp1 == null);
	}
	
	private boolean emptyFinalOp2() {
		return (finalOp2 == null);
	}
	
	private boolean adjustOkButton() {
		if (emptyFinalOp1() || emptyFinalOp2()) {
			btnOk.setEnabled(false);
			setMessage(lang.missingOperandsMsg);
			return false;
		} else {
			btnOk.setEnabled(true);
			setMessage(lang.allRightMsg);
			return true;
		}
	}
	
	private void incompatibleOperandsError() {
		btnOk.setEnabled(false);
		setMessage(lang.incomparableTypesMsg);
	}
	
	private void setFinalOp1() {
		try {
			assignmentCopy.setFirstOperand(finalOp1);
		} catch (IncompatibleDataTypesException e) {
			incompatibleOperandsError();
		}
	}
	
	private void setFinalOp2() {
		try {
			assignmentCopy.setSecondOperand(finalOp2);
		} catch (IncompatibleDataTypesException e) {
			incompatibleOperandsError();
		}
	}
	
	private void setSecondValueOperand() {
		valueOp2 = Functions.parseComparableValue(secondValueText.getText());
		updateSecondFinalOp();
		if (! emptyFinalOp2()) setFinalOp2();
	}
	
	private boolean lastSelected(Combo c) {
		return (c.getSelectionIndex() == (c.getItemCount() - 1));
	}
	
	private void setFirstVariableOperand() {
		boolean lastSelected = lastSelected(firstVariableCombo);
		if (! newVariable) lastSelected = false;
		if (! lastSelected) {
			int selected = firstVariableCombo.getSelectionIndex();
			if (selected > 0) {
				variableOp1 = variables.get(selected - 1);
				updateFirstFinalOp();
				if (! emptyFinalOp1()) setFinalOp1();
			}
		}
	}
	
	private void setSecondVariableOperand() {
		int selected = secondVariableCombo.getSelectionIndex();
		if (selected > 0) {
			variableOp2 = variables.get(selected - 1);
			updateSecondFinalOp();
			if (! emptyFinalOp2()) setFinalOp2();
		}
	}
	
	private void setChanges() {
		try {
			assignmentOrig.setFirstOperand(assignmentCopy.getFirstOperand());
			assignmentOrig.setSecondOperand(assignmentCopy.getSecondOperand());
		} catch (IncompatibleDataTypesException e) {}
		assignmentOrig.setOperator(assignmentCopy.getOperator());
		
	}
	
	private void selectVariable(Combo c, Text t, Button b) {
		c.setEnabled(true);
		t.setEnabled(false);
		b.setEnabled(false);
	}
	
	private void selectValue(Combo c, Text t, Button b) {
		c.setEnabled(false);
		t.setEnabled(true);
		b.setEnabled(false);
	}
	
	private void selectExpression(Combo c, Text t, Button b) {
		c.setEnabled(false);
		t.setEnabled(false);
		b.setEnabled(true);
	}
	
	private void firstVariableSelection() {
		int selected = firstVariableCombo.getSelectionIndex();
		if (selected == 0) {
			variableOp1 = null;
			updateFirstFinalOp();
		} else if (lastSelected(firstVariableCombo) && newVariable) 
			mainShell.showNewVariableShell(this, this, firstVariableCombo, variables);
		else setFirstVariableOperand();
	}
	
	private void secondVariableSelection() {
		int selected = secondVariableCombo.getSelectionIndex();
		if (selected == 0) {
			variableOp2 = null;
			updateSecondFinalOp();
		} else setSecondVariableOperand();
	}
	
	private void updateFirstFinalOp() {
		finalOp1 = variableOp1;
		adjustOkButton();
	}
	
	private void updateSecondFinalOp() {
		if (btnSecondVariable.getSelection()) finalOp2 = variableOp2;
		else if (btnSecondValue.getSelection()) finalOp2 = valueOp2;
		else if (btnSecondExpression.getSelection()) finalOp2 = expressionOp2;
		adjustOkButton();
	}
	
	private void updateFinalOperands() {
		updateFirstFinalOp();
		updateSecondFinalOp();
	}
	
	private void showNumberExpressionShell(NumberExpression e) {
		mainShell.showNewNumberExpressionShell(this, this, e);
	}

	/**
	 * Create the shell.
	 * @param display
	 */
	public AssignmentSettingsShell(MainShell main, Shell parent, Assignment a, boolean newVar, boolean editing) {
		
		super(parent, SWT.CLOSE | SWT.TITLE | SWT.APPLICATION_MODAL);
		
		setMinimumSize(new Point(Const.ASSIGNMENT_SETTINGS_SHELL_WIDTH, Const.ASSIGNMENT_SETTINGS_SHELL_HEIGHT));
		
		mainShell = main;
		canvas = mainShell.getCanvas();
		variables = canvas.getVariables();
		lang = mainShell.getLanguage();
		edit = editing;
		assignmentOrig = a;
		assignmentCopy = assignmentOrig.deepCopy();
		
		finalOp1 = null;
		finalOp2 = null;
		
		variableOp1 = null;
		variableOp2 = null;
		
		valueOp2 = null;
		
		expressionOp2 = null;
		
		newVariable = newVar;
		
		if (edit) editAction = new EditingAssignmentAction(canvas, assignmentOrig);
		else editAction = null;
		
		setLayout(new BorderLayout(0, 0));
		
		Composite confirmComposite = new Composite(this, SWT.NONE);
		confirmComposite.setLayoutData(BorderLayout.SOUTH);
		confirmComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
		
		Composite buttonsComposite = new Composite(confirmComposite, SWT.NONE);
		FillLayout fl_buttonsComposite = new FillLayout(SWT.HORIZONTAL);
		fl_buttonsComposite.marginHeight = 9;
		buttonsComposite.setLayout(fl_buttonsComposite);
		
		btnOk = new Button(buttonsComposite, SWT.NONE);
		btnOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				setChanges();
				if (edit) {
					editAction.recordNewData();
					canvas.recordAction(editAction);
				}
				canvas.redraw();
				btnOk.getShell().dispose();
			}
		});
		btnOk.setText(lang.ok);
		
		btnCancel = new Button(buttonsComposite, SWT.NONE);
		btnCancel.setSelection(true);
		btnCancel.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				btnCancel.getShell().dispose();
			}
		});
		btnCancel.setText(lang.cancel);
		
		Composite settingsComposite = new Composite(this, SWT.NONE);
		settingsComposite.setLayoutData(BorderLayout.CENTER);
		settingsComposite.setLayout(new BorderLayout(0, 0));
		
		Composite messagesComposite = new Composite(settingsComposite, SWT.NONE);
		messagesComposite.setLayoutData(BorderLayout.SOUTH);
		messagesComposite.setLayout(new FillLayout(SWT.VERTICAL));
		
		lblMessagesLabel = new Label(messagesComposite, SWT.CENTER);
		
		Composite conditionComposite = new Composite(settingsComposite, SWT.NONE);
		conditionComposite.setLayoutData(BorderLayout.CENTER);
		RowLayout rl_conditionComposite = new RowLayout(SWT.HORIZONTAL);
		rl_conditionComposite.wrap = false;
		rl_conditionComposite.marginTop = 8;
		rl_conditionComposite.justify = true;
		conditionComposite.setLayout(rl_conditionComposite);
		
		Group grpFirstOperand = new Group(conditionComposite, SWT.NONE);
		grpFirstOperand.setText(lang.firstOperand);
		grpFirstOperand.setLayout(new GridLayout(2, false));
		
		lblVariable = new Label(grpFirstOperand, SWT.NONE);
		lblVariable.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblVariable.setText(lang.variable);
		
		firstVariableCombo = new Combo(grpFirstOperand, SWT.READ_ONLY);
		firstVariableCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				firstVariableSelection();
			}
		});
		
		GridData firstVariableGridData = new GridData();
		firstVariableGridData.horizontalAlignment = GridData.FILL;
		firstVariableCombo.setLayoutData(firstVariableGridData);
		
		Group grpSecondOperand = new Group(conditionComposite, SWT.NONE);
		grpSecondOperand.setText(lang.secondOperand);
		grpSecondOperand.setLayout(new GridLayout(2, false));
		
		btnSecondVariable = new Button(grpSecondOperand, SWT.RADIO);
		secondVariableCombo = new Combo(grpSecondOperand, SWT.READ_ONLY);
		btnSecondValue = new Button(grpSecondOperand, SWT.RADIO);
		secondValueText = new Text(grpSecondOperand, SWT.BORDER);
		btnSecondExpression = new Button(grpSecondOperand, SWT.RADIO);
		
		Composite secondExpressionComposite = new Composite(grpSecondOperand, SWT.NONE);
		RowLayout rl_secondExpressionComposite = new RowLayout(SWT.HORIZONTAL);
		rl_secondExpressionComposite.marginBottom = 0;
		rl_secondExpressionComposite.marginTop = 0;
		rl_secondExpressionComposite.marginRight = 0;
		rl_secondExpressionComposite.marginLeft = 0;
		secondExpressionComposite.setLayout(rl_secondExpressionComposite);
		
		GridData secondExpressionGridData = new GridData();
		secondExpressionGridData.horizontalAlignment = GridData.FILL;
		secondExpressionComposite.setLayoutData(secondExpressionGridData);
		secondExpressionText = new Text(secondExpressionComposite, SWT.BORDER | SWT.READ_ONLY);
		btnCreateEditSecondExpression = new Button(secondExpressionComposite, SWT.NONE);
		
		btnSecondVariable.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectVariable(secondVariableCombo, secondValueText, btnCreateEditSecondExpression);
				updateFinalOperands();
			}
		});
		btnSecondVariable.setText(lang.variable);
		btnSecondVariable.setSelection(true);
		
		secondVariableCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				secondVariableSelection();
			}
		});
		
		GridData secondVariableGridData = new GridData();
		secondVariableGridData.horizontalAlignment = GridData.FILL;
		secondVariableCombo.setLayoutData(secondVariableGridData);
		
		updateVariableList();
		
		Operand o = assignmentCopy.getFirstOperand();
		if ((o != null) && (o.isVariable())) {
			int i = variables.indexOf(o) + 1;
			firstVariableCombo.select(i);
			if (i != 0) {
				variableOp1 = (ComparableVariable)o;
				finalOp1 = variableOp1;
			}
		}
		
		btnSecondValue.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectValue(secondVariableCombo, secondValueText, btnCreateEditSecondExpression);
				updateFinalOperands();
			}
		});
		btnSecondValue.setText(lang.value);
		
		secondValueText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setSecondValueOperand();
			}
		});
		secondValueText.setEnabled(false);
		
		GridData secondValueGridData = new GridData();
		secondValueGridData.horizontalAlignment = GridData.FILL;
		secondValueText.setLayoutData(secondValueGridData);
		
		btnSecondExpression.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectExpression(secondVariableCombo, secondValueText, btnCreateEditSecondExpression);
				updateFinalOperands();
			}
		});
		btnSecondExpression.setText(lang.expression);
		
		secondExpressionText.setEnabled(false);
		
		o = assignmentCopy.getSecondOperand();
		if (o != null) {
			if (o.isValue()) {
				secondValueText.setText(o.print());
				valueOp2 = (ComparableValue)o;
				finalOp2 = valueOp2;
			} else if (o.isExpression()) {
				secondExpressionText.setText(o.print());
				expressionOp2 = (NumberExpression)o;
				finalOp2 = expressionOp2;
			} else if (o.isVariable()) {
				int i = variables.indexOf(o) + 1;
				secondVariableCombo.select(i);
				if (i != 0) {
					variableOp2 = (ComparableVariable)o;
					finalOp2 = variableOp2;
				}
			}
		}
		
		btnCreateEditSecondExpression.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (expressionOp2 == null) showNumberExpressionShell(new NumberExpression());
				else showNumberExpressionShell(expressionOp2.deepCopy());
			}
		});
		btnCreateEditSecondExpression.setEnabled(false);
		btnCreateEditSecondExpression.setText(lang.create + Const.SLASH + lang.edit + Const.ELLIPSIS);
		
		if (! secondValueText.getText().equals("")) {
			btnSecondValue.setSelection(true);
			btnSecondVariable.setSelection(false);
			btnSecondExpression.setSelection(false);
			selectValue(secondVariableCombo, secondValueText, btnCreateEditSecondExpression);
		}
		
		if (! secondExpressionText.getText().equals("")) {
			btnSecondValue.setSelection(false);
			btnSecondVariable.setSelection(false);
			btnSecondExpression.setSelection(true);
			selectExpression(secondVariableCombo, secondValueText, btnCreateEditSecondExpression);
		}
		
		adjustOkButton();
		createContents();
	
	}

	/**
	 * Create contents of the shell.
	 */
	protected void createContents() {
		setText(lang.assignmentSettings);
		setSize(Const.ASSIGNMENT_SETTINGS_SHELL_WIDTH, Const.ASSIGNMENT_SETTINGS_SHELL_HEIGHT);
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

	@Override
	public void updateVariableList() {
		variables = canvas.getVariables();
		String[] leftVariableComboItems = Functions.getVariableComboItems(variables, lang, newVariable);
		String[] rightVariableComboItems = Functions.getVariableComboItems(variables, lang, false);
		firstVariableCombo.setItems(leftVariableComboItems);
		secondVariableCombo.setItems(rightVariableComboItems);
		firstVariableCombo.select(0);
		secondVariableCombo.select(0);
	}

	@Override
	public MainShell getMainShell() {
		return mainShell;
	}

	@Override
	public void setNumberExpression(NumberExpression e) {
		if (e.usable()) {
			expressionOp2 = e;
			secondExpressionText.setText(expressionOp2.print());
			updateSecondFinalOp();
			setFinalOp2();
		}
	}

	@Override
	public void updateVariableOperands() {
		setFirstVariableOperand();
		setSecondVariableOperand();
	}
	
}