package jflowchart.ui.windows;

import jflowchart.engine.common.Const;
import jflowchart.engine.common.Language;
import jflowchart.engine.history.ResizingAction;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import swing2swt.layout.BorderLayout;
import org.eclipse.swt.widgets.Composite;
import swing2swt.layout.FlowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.DisposeEvent;

public class CanvasSizeShell extends Shell {
	
	private MainShell parent;
	private Language lang;
	private ResizingAction resizingAction;
	
	private Spinner widthSpinner;
	private Spinner heightSpinner;
	
	private Button btnOk;
	private Button btnCancel;
	
	public CanvasSizeShell(Display display, MainShell par) {
		super(par.getShell(), SWT.CLOSE|SWT.TITLE|SWT.APPLICATION_MODAL);
		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				parent.discardCanvasSizeShell();
				parent.getCanvas().unlock();
			}
		});
		
		parent = par;
		lang = parent.getLanguage();
		resizingAction = new ResizingAction(parent.getCanvas(), parent.getCanvasClient().width, parent.getCanvasClient().height);
		
		setLayout(new BorderLayout(0, 0));
		
		Composite confirmComposite = new Composite(this, SWT.NONE);
		confirmComposite.setLayoutData(BorderLayout.SOUTH);
		confirmComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 4, 0));
		
		Composite buttonsComposite = new Composite(confirmComposite, SWT.NONE);
		FillLayout fl_buttonsComposite = new FillLayout(SWT.HORIZONTAL);
		fl_buttonsComposite.marginHeight = 8;
		buttonsComposite.setLayout(fl_buttonsComposite);
		
		btnOk = new Button(buttonsComposite, SWT.NONE);
		btnOk.setSelection(false);
		btnOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				
				int width = widthSpinner.getSelection();
				int height = heightSpinner.getSelection();
				
				parent.changeCanvasSize(width, height);
				resizingAction.setNewSize(width, height);
				parent.getCanvas().recordAction(resizingAction);
				btnOk.getShell().dispose();
			
			}
		});
		btnOk.setText(lang.ok);
		
		btnCancel = new Button(buttonsComposite, SWT.NONE);
		btnCancel.setSelection(true);
		btnCancel.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				btnCancel.getShell().dispose();
			}
		});
		btnCancel.setText(lang.cancel);
		
		this.setDefaultButton(btnCancel);
		
		Composite sizeComposite = new Composite(this, SWT.NONE);
		sizeComposite.setLayoutData(BorderLayout.CENTER);
		sizeComposite.setLayout(new FillLayout(SWT.HORIZONTAL));
		
		Composite widthComposite = new Composite(sizeComposite, SWT.NONE);
		widthComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 3));
		
		Composite widthControl = new Composite(widthComposite, SWT.NONE);
		RowLayout rl_widthControl = new RowLayout(SWT.VERTICAL);
		rl_widthControl.marginBottom = 0;
		rl_widthControl.center = true;
		widthControl.setLayout(rl_widthControl);
		
		widthSpinner = new Spinner(widthControl, SWT.BORDER);
		widthSpinner.setMaximum(Const.MAXIMUM_CANVAS_SIZE);
		widthSpinner.setMinimum(parent.getCanvasMinimalWidth());
		widthSpinner.setSelection(parent.getCanvasClient().width);
		
		Label lblWidth = new Label(widthControl, SWT.NONE);
		lblWidth.setText(lang.width);
		
		Composite heightComposite = new Composite(sizeComposite, SWT.NONE);
		heightComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 3));
		
		Composite heightControl = new Composite(heightComposite, SWT.NONE);
		RowLayout rl_heightControl = new RowLayout(SWT.VERTICAL);
		rl_heightControl.marginBottom = 0;
		rl_heightControl.center = true;
		heightControl.setLayout(rl_heightControl);
		
		heightSpinner = new Spinner(heightControl, SWT.BORDER);
		heightSpinner.setMaximum(Const.MAXIMUM_CANVAS_SIZE);
		heightSpinner.setMinimum(parent.getCanvasMinimalHeight());
		heightSpinner.setSelection(parent.getCanvasClient().height);
		
		Label lblHeight = new Label(heightControl, SWT.NONE);
		lblHeight.setText(lang.height);
		createContents();
	}

	/**
	 * Create contents of the shell.
	 */
	protected void createContents() {
		setText(lang.canvasSize);
		setSize(Const.CANVAS_SIZE_SHELL_XSIZE, Const.CANVAS_SIZE_SHELL_YSIZE);
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
}
