package jflowchart.ui.windows;

import java.util.LinkedList;

import jflowchart.engine.arithmetics.ComparableOperand;
import jflowchart.engine.arithmetics.ComparableValue;
import jflowchart.engine.arithmetics.ComparableVariable;
import jflowchart.engine.arithmetics.Comparison;
import jflowchart.engine.arithmetics.IncompatibleDataTypesException;
import jflowchart.engine.arithmetics.NumberExpression;
import jflowchart.engine.arithmetics.Operand;
import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.common.Const;
import jflowchart.engine.common.Functions;
import jflowchart.engine.common.Language;
import jflowchart.engine.history.EditingConditionAction;
import jflowchart.ui.canvas.FlowChartCanvas;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import swing2swt.layout.BorderLayout;
import org.eclipse.swt.widgets.Composite;
import swing2swt.layout.FlowLayout;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.layout.GridLayout;

public class ConditionSettingsShell extends Shell implements VariableSetting, NumberExpressionSetting {
	
	private Text firstValueText;
	private Combo firstVariableCombo;
	private Combo operatorCombo;
	private Text secondValueText;
	private Combo secondVariableCombo;
	private Button btnOk;
	private Button btnCancel;
	private Label lblMessagesLabel;
	
	private MainShell mainShell;
	private FlowChartCanvas canvas;
	private LinkedList<ComparableVariable> variables;
	private Language lang;
	private boolean edit;
	private Comparison conditionOrig;
	private Comparison conditionCopy;
	private EditingConditionAction editAction;
	private Text firstExpressionText;
	private Button btnCreateEditFirstExpression;
	private Text secondExpressionText;
	private Button btnCreateEditSecondExpression;
	
	private Button btnFirstVariable;
	private Button btnFirstValue;
	private Button btnFirstExpression;
	
	private Button btnSecondVariable;
	private Button btnSecondValue;
	private Button btnSecondExpression;
	
	private ComparableOperand finalOp1;
	private ComparableOperand finalOp2;
	
	private ComparableVariable variableOp1;
	private ComparableVariable variableOp2;
	
	private ComparableValue valueOp1;
	private ComparableValue valueOp2;
	
	private NumberExpression expressionOp1;
	private NumberExpression expressionOp2;
	private boolean firstExpression;
	
	private void setMessage(String msg) {
		lblMessagesLabel.setText(msg);
	}
	
	private boolean emptyFinalOp1() {
		return (finalOp1 == null);
	}
	
	private boolean emptyFinalOp2() {
		return (finalOp2 == null);
	}
	
	private boolean adjustOkButton() {
		if (emptyFinalOp1() || emptyFinalOp2()) {
			btnOk.setEnabled(false);
			setMessage(lang.missingOperandsMsg);
			return false;
		} else {
			btnOk.setEnabled(true);
			setMessage(lang.allRightMsg);
			return true;
		}
	}
	
	private void incompatibleOperandsError() {
		btnOk.setEnabled(false);
		setMessage(lang.incomparableTypesMsg);
	}
	
	private void setFinalOp1() {
		try {
			conditionCopy.setFirstOperand(finalOp1);
		} catch (IncompatibleDataTypesException e) {
			incompatibleOperandsError();
		}
	}
	
	private void setFinalOp2() {
		try {
			conditionCopy.setSecondOperand(finalOp2);
		} catch (IncompatibleDataTypesException e) {
			incompatibleOperandsError();
		}
	}
	
	private void setFirstValueOperand() {
		valueOp1 = Functions.parseComparableValue(firstValueText.getText());
		updateFirstFinalOp();
		if (! emptyFinalOp1()) setFinalOp1();
	}
	
	private void setSecondValueOperand() {
		valueOp2 = Functions.parseComparableValue(secondValueText.getText());
		updateSecondFinalOp();
		if (! emptyFinalOp2()) setFinalOp2();
	}
	
	private void setFirstVariableOperand() {
		int selected = firstVariableCombo.getSelectionIndex();
		if (selected > 0) {
			variableOp1 = variables.get(selected - 1);
			updateFirstFinalOp();
			if (! emptyFinalOp1()) setFinalOp1();
		}
	}
	
	private void setSecondVariableOperand() {
		int selected = secondVariableCombo.getSelectionIndex();
		if (selected > 0) {
			variableOp2 = variables.get(selected - 1);
			updateSecondFinalOp();
			if (! emptyFinalOp2()) setFinalOp2();
		}
	}
	
	private void setChanges() {
		try {
			conditionOrig.setFirstOperand(conditionCopy.getFirstOperand());
			conditionOrig.setSecondOperand(conditionCopy.getSecondOperand());
		} catch (IncompatibleDataTypesException e) {}
		conditionOrig.setOperator(conditionCopy.getOperator());
		
	}
	
	private void selectVariable(Combo c, Text t, Button b) {
		c.setEnabled(true);
		t.setEnabled(false);
		b.setEnabled(false);
	}
	
	private void selectValue(Combo c, Text t, Button b) {
		c.setEnabled(false);
		t.setEnabled(true);
		b.setEnabled(false);
	}
	
	private void selectExpression(Combo c, Text t, Button b) {
		c.setEnabled(false);
		t.setEnabled(false);
		b.setEnabled(true);
	}
	
	private void firstVariableSelection() {
		int selected = firstVariableCombo.getSelectionIndex();
		if (selected == 0) {
			variableOp1 = null;
			updateFirstFinalOp();
		} else setFirstVariableOperand();
	}
	
	private void secondVariableSelection() {
		int selected = secondVariableCombo.getSelectionIndex();
		if (selected == 0) {
			variableOp2 = null;
			updateSecondFinalOp();
		} else setSecondVariableOperand();
	}
	
	private void updateFirstFinalOp() {
		if (btnFirstVariable.getSelection()) finalOp1 = variableOp1;
		else if (btnFirstValue.getSelection()) finalOp1 = valueOp1;
		else if (btnFirstExpression.getSelection()) finalOp1 = expressionOp1;
		adjustOkButton();
	}
	
	private void updateSecondFinalOp() {
		if (btnSecondVariable.getSelection()) finalOp2 = variableOp2;
		else if (btnSecondValue.getSelection()) finalOp2 = valueOp2;
		else if (btnSecondExpression.getSelection()) finalOp2 = expressionOp2;
		adjustOkButton();
	}
	
	private void updateFinalOperands() {
		updateFirstFinalOp();
		updateSecondFinalOp();
	}
	
	private void showNumberExpressionShell(NumberExpression e) {
		mainShell.showNewNumberExpressionShell(this, this, e);
	}

	/**
	 * Create the shell.
	 * @param display
	 */
	public ConditionSettingsShell(MainShell main, Shell parent, Comparison cond, boolean editing) {
		
		super(parent, SWT.CLOSE | SWT.TITLE | SWT.APPLICATION_MODAL);
		setMinimumSize(new Point(Const.NODE_SETTINGS_SHELL_WIDTH, Const.NODE_SETTINGS_SHELL_HEIGHT));
		
		mainShell = main;
		
		canvas = mainShell.getCanvas();
		variables = Functions.getComparableVariables(canvas.getVariables());
		lang = mainShell.getLanguage();
		edit = editing;
		conditionOrig = cond;
		conditionCopy = conditionOrig.deepCopy();
		
		finalOp1 = null;
		finalOp2 = null;
		
		variableOp1 = null;
		variableOp2 = null;
		
		valueOp1 = null;
		valueOp2 = null;
		
		expressionOp1 = null;
		expressionOp2 = null;
		firstExpression = false;
		
		if (edit) editAction = new EditingConditionAction(canvas, conditionOrig);
		else editAction = null;
		
		setLayout(new BorderLayout(0, 0));
		
		Composite confirmComposite = new Composite(this, SWT.NONE);
		confirmComposite.setLayoutData(BorderLayout.SOUTH);
		confirmComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
		
		Composite buttonsComposite = new Composite(confirmComposite, SWT.NONE);
		FillLayout fl_buttonsComposite = new FillLayout(SWT.HORIZONTAL);
		fl_buttonsComposite.marginHeight = 9;
		buttonsComposite.setLayout(fl_buttonsComposite);
		
		btnOk = new Button(buttonsComposite, SWT.NONE);
		btnOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				setChanges();
				if (edit) {
					editAction.recordNewData();
					canvas.recordAction(editAction);
				}
				canvas.redraw();
				btnOk.getShell().dispose();
			}
		});
		btnOk.setText(lang.ok);
		
		btnCancel = new Button(buttonsComposite, SWT.NONE);
		btnCancel.setSelection(true);
		btnCancel.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				btnCancel.getShell().dispose();
			}
		});
		btnCancel.setText(lang.cancel);
		
		Composite settingsComposite = new Composite(this, SWT.NONE);
		settingsComposite.setLayoutData(BorderLayout.CENTER);
		settingsComposite.setLayout(new BorderLayout(0, 0));
		
		Composite messagesComposite = new Composite(settingsComposite, SWT.NONE);
		messagesComposite.setLayoutData(BorderLayout.SOUTH);
		messagesComposite.setLayout(new FillLayout(SWT.VERTICAL));
		
		lblMessagesLabel = new Label(messagesComposite, SWT.CENTER);
		
		Composite conditionComposite = new Composite(settingsComposite, SWT.NONE);
		conditionComposite.setLayoutData(BorderLayout.CENTER);
		RowLayout rl_conditionComposite = new RowLayout(SWT.HORIZONTAL);
		rl_conditionComposite.wrap = false;
		rl_conditionComposite.marginTop = 8;
		rl_conditionComposite.justify = true;
		conditionComposite.setLayout(rl_conditionComposite);
		
		Group grpFirstOperand = new Group(conditionComposite, SWT.NONE);
		grpFirstOperand.setText(lang.firstOperand);
		grpFirstOperand.setLayout(new GridLayout(2, false));
		
		btnFirstVariable = new Button(grpFirstOperand, SWT.RADIO);
		firstVariableCombo = new Combo(grpFirstOperand, SWT.READ_ONLY);
		btnFirstValue = new Button(grpFirstOperand, SWT.RADIO);
		firstValueText = new Text(grpFirstOperand, SWT.BORDER);
		btnFirstExpression = new Button(grpFirstOperand, SWT.RADIO);
		
		Composite firstExpressionComposite = new Composite(grpFirstOperand, SWT.NONE);
		RowLayout rl_firstExpressionComposite = new RowLayout(SWT.HORIZONTAL);
		rl_firstExpressionComposite.marginBottom = 0;
		rl_firstExpressionComposite.marginTop = 0;
		rl_firstExpressionComposite.marginRight = 0;
		rl_firstExpressionComposite.marginLeft = 0;
		firstExpressionComposite.setLayout(rl_firstExpressionComposite);
		
		GridData firstExpressionGridData = new GridData();
		firstExpressionGridData.horizontalAlignment = GridData.FILL;
		firstExpressionComposite.setLayoutData(firstExpressionGridData);
		
		firstExpressionText = new Text(firstExpressionComposite, SWT.BORDER | SWT.READ_ONLY);
		btnCreateEditFirstExpression = new Button(firstExpressionComposite, SWT.NONE);
		
		btnFirstVariable.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectVariable(firstVariableCombo, firstValueText, btnCreateEditFirstExpression);
				updateFinalOperands();
			}
		});
		btnFirstVariable.setText(lang.variable);
		btnFirstVariable.setSelection(true);
		
		firstVariableCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				firstVariableSelection();
			}
		});
		
		GridData firstVariableGridData = new GridData();
		firstVariableGridData.horizontalAlignment = GridData.FILL;
		firstVariableCombo.setLayoutData(firstVariableGridData);

		btnFirstValue.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectValue(firstVariableCombo, firstValueText, btnCreateEditFirstExpression);
				updateFinalOperands();
			}
		});
		btnFirstValue.setText(lang.value);
		
		firstValueText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setFirstValueOperand();
			}
		});
		firstValueText.setEnabled(false);
		
		GridData firstValueGridData = new GridData();
		firstValueGridData.horizontalAlignment = GridData.FILL;
		firstValueText.setLayoutData(firstValueGridData);
		
		btnFirstExpression.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectExpression(firstVariableCombo, firstValueText, btnCreateEditFirstExpression);
				updateFinalOperands();
			}
		});
		btnFirstExpression.setText(lang.expression);
		
		firstExpressionText.setEnabled(false);
		
		btnCreateEditFirstExpression.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				firstExpression = true;
				if (expressionOp1 == null) showNumberExpressionShell(new NumberExpression());
				else showNumberExpressionShell(expressionOp1.deepCopy());
			}
		});
		btnCreateEditFirstExpression.setEnabled(false);
		btnCreateEditFirstExpression.setText(lang.create + Const.SLASH + lang.edit + Const.ELLIPSIS);
		
		Group grpOperator = new Group(conditionComposite, SWT.NONE);
		grpOperator.setText(lang.operator);
		grpOperator.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
		
		Composite operatorComposite = new Composite(grpOperator, SWT.NONE);
		operatorComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
		
		operatorCombo = new Combo(operatorComposite, SWT.READ_ONLY);
		operatorCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				conditionCopy.setOperator(operatorCombo.getSelectionIndex());
			}
		});
		operatorCombo.setItems(Const.COMPARISON_OPERATION_SIGNS);
		int op = conditionCopy.getOperator();
		if (op == Const.UNSELECTED_CODE) {
			operatorCombo.select(Const.EQUAL_OPERATION_CODE);
			conditionCopy.setOperator(Const.EQUAL_OPERATION_CODE);
		} else operatorCombo.select(op);
		
		Group grpSecondOperand = new Group(conditionComposite, SWT.NONE);
		grpSecondOperand.setText(lang.secondOperand);
		grpSecondOperand.setLayout(new GridLayout(2, false));
		
		btnSecondVariable = new Button(grpSecondOperand, SWT.RADIO);
		secondVariableCombo = new Combo(grpSecondOperand, SWT.READ_ONLY);
		btnSecondValue = new Button(grpSecondOperand, SWT.RADIO);
		secondValueText = new Text(grpSecondOperand, SWT.BORDER);
		btnSecondExpression = new Button(grpSecondOperand, SWT.RADIO);
		
		Composite secondExpressionComposite = new Composite(grpSecondOperand, SWT.NONE);
		RowLayout rl_secondExpressionComposite = new RowLayout(SWT.HORIZONTAL);
		rl_secondExpressionComposite.marginBottom = 0;
		rl_secondExpressionComposite.marginTop = 0;
		rl_secondExpressionComposite.marginRight = 0;
		rl_secondExpressionComposite.marginLeft = 0;
		secondExpressionComposite.setLayout(rl_secondExpressionComposite);
		
		GridData secondExpressionGridData = new GridData();
		secondExpressionGridData.horizontalAlignment = GridData.FILL;
		secondExpressionComposite.setLayoutData(secondExpressionGridData);
		secondExpressionText = new Text(secondExpressionComposite, SWT.BORDER | SWT.READ_ONLY);
		btnCreateEditSecondExpression = new Button(secondExpressionComposite, SWT.NONE);
		
		btnSecondVariable.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectVariable(secondVariableCombo, secondValueText, btnCreateEditSecondExpression);
				updateFinalOperands();
			}
		});
		btnSecondVariable.setText(lang.variable);
		btnSecondVariable.setSelection(true);
		
		secondVariableCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				secondVariableSelection();
			}
		});
		
		GridData secondVariableGridData = new GridData();
		secondVariableGridData.horizontalAlignment = GridData.FILL;
		secondVariableCombo.setLayoutData(secondVariableGridData);
		
		updateVariableList();
		
		Operand o = conditionCopy.getFirstOperand();
		if (o != null) {
			if (o.isValue()) {
				firstValueText.setText(o.print());
				valueOp1 = (ComparableValue)o;
				finalOp1 = valueOp1;
			} else if (o.isExpression()) {
				firstExpressionText.setText(o.print());
				expressionOp1 = (NumberExpression)o;
				finalOp1 = expressionOp1;
			} else if (o.isVariable()) {
				int i = variables.indexOf(o) + 1;
				firstVariableCombo.select(i);
				if (i != 0) {
					variableOp1 = (ComparableVariable)o;
					finalOp1 = variableOp1;
				}
			}
		}
		
		btnSecondValue.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectValue(secondVariableCombo, secondValueText, btnCreateEditSecondExpression);
				updateFinalOperands();
			}
		});
		btnSecondValue.setText(lang.value);
		
		secondValueText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setSecondValueOperand();
			}
		});
		secondValueText.setEnabled(false);
		
		GridData secondValueGridData = new GridData();
		secondValueGridData.horizontalAlignment = GridData.FILL;
		secondValueText.setLayoutData(secondValueGridData);
		
		btnSecondExpression.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectExpression(secondVariableCombo, secondValueText, btnCreateEditSecondExpression);
				updateFinalOperands();
			}
		});
		btnSecondExpression.setText(lang.expression);
		
		secondExpressionText.setEnabled(false);
		
		o = conditionCopy.getSecondOperand();
		if (o != null) {
			if (o.isValue()) {
				secondValueText.setText(o.print());
				valueOp2 = (ComparableValue)o;
				finalOp2 = valueOp2;
			} else if (o.isExpression()) {
				secondExpressionText.setText(o.print());
				expressionOp2 = (NumberExpression)o;
				finalOp2 = expressionOp2;
			} else if (o.isVariable()) {
				int i = variables.indexOf(o) + 1;
				secondVariableCombo.select(i);
				if (i != 0) {
					variableOp2 = (ComparableVariable)o;
					finalOp2 = variableOp2;
				}
			}
		}
		
		btnCreateEditSecondExpression.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				firstExpression = false;
				if (expressionOp2 == null) showNumberExpressionShell(new NumberExpression());
				else showNumberExpressionShell(expressionOp2.deepCopy());
			}
		});
		btnCreateEditSecondExpression.setEnabled(false);
		btnCreateEditSecondExpression.setText(lang.create + Const.SLASH + lang.edit + Const.ELLIPSIS);
		
		if (! firstValueText.getText().equals("")) {
			btnFirstValue.setSelection(true);
			btnFirstVariable.setSelection(false);
			btnFirstExpression.setSelection(false);
			selectValue(firstVariableCombo, firstValueText, btnCreateEditFirstExpression);
		}
		
		if (! secondValueText.getText().equals("")) {
			btnSecondValue.setSelection(true);
			btnSecondVariable.setSelection(false);
			btnSecondExpression.setSelection(false);
			selectValue(secondVariableCombo, secondValueText, btnCreateEditSecondExpression);
		}
		
		if (! firstExpressionText.getText().equals("")) {
			btnFirstValue.setSelection(false);
			btnFirstVariable.setSelection(false);
			btnFirstExpression.setSelection(true);
			selectExpression(firstVariableCombo, firstValueText, btnCreateEditFirstExpression);
		}
		
		if (! secondExpressionText.getText().equals("")) {
			btnSecondValue.setSelection(false);
			btnSecondVariable.setSelection(false);
			btnSecondExpression.setSelection(true);
			selectExpression(secondVariableCombo, secondValueText, btnCreateEditSecondExpression);
		}
		
		adjustOkButton();
		createContents();
	
	}

	/**
	 * Create contents of the shell.
	 */
	protected void createContents() {
		setText(lang.conditionSettings);
		setSize(Const.NODE_SETTINGS_SHELL_WIDTH, Const.NODE_SETTINGS_SHELL_HEIGHT);
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

	@Override
	public void updateVariableList() {
		LinkedList<Variable> vars = canvas.getVariables();
		variables = Functions.getComparableVariables(vars);
		String[] variableComboItems = Functions.getVariableComboItems(Functions.filterNonComparableVariables(vars), lang, false);
		firstVariableCombo.setItems(variableComboItems);
		secondVariableCombo.setItems(variableComboItems);
		firstVariableCombo.select(0);
		secondVariableCombo.select(0);
	}

	@Override
	public MainShell getMainShell() {
		return mainShell;
	}

	@Override
	public void setNumberExpression(NumberExpression e) {
		if (e.usable()) {
			if (firstExpression) {
				expressionOp1 = e;
				firstExpressionText.setText(expressionOp1.print());
				updateFirstFinalOp();
				setFinalOp1();
			} else {
				expressionOp2 = e;
				secondExpressionText.setText(expressionOp2.print());
				updateSecondFinalOp();
				setFinalOp2();
			}
		}
	}

	@Override
	public void updateVariableOperands() {
		// TODO Auto-generated method stub
		
	}
	
}
