package jflowchart.ui.windows;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import jflowchart.engine.common.Language;

import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.widgets.Shell;
import swing2swt.layout.BorderLayout;

import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Listener;

import swing2swt.layout.FlowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.browser.Browser;


public class HelpShell extends Shell {
	
	private Language lang;
	private Button btnClose;
	private List contentsList;
	private Browser browser;
	private String[] urls;
	private Shell parent;
	private PleaseWaitMessage wait;
	
	private String getLocalURL(String file) throws MalformedURLException {
		return new File(lang.helpPath.concat(file)).toURI().toURL().toString();
	}
	
	private void showWaitMessage() {
		try {
			Display display = Display.getDefault();
			wait = new PleaseWaitMessage(parent, lang);
			wait.open();
			wait.layout();
			while (!wait.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
		} catch (Exception e) {
			throw e;
		}
	}

	public HelpShell(Shell parent, Language l) throws Exception {
		
		super(parent, SWT.SHELL_TRIM);
		
		this.parent = parent;
		lang = l;
		
		if (! new File("help/").exists()) {
			
			showWaitMessage();
			
			InputStream is = getClass().getResourceAsStream("/help.zip");
			ZipInputStream zis = new ZipInputStream(is);
			ZipEntry entry;
			File f;
			FileOutputStream fout;
			String name;
			while ((entry = zis.getNextEntry()) != null) {
				name = entry.getName();
				f = new File(name);
				if (!name.contains(".")) f.mkdirs();
				else f.createNewFile();
				if (f.isFile()) {
					fout = new FileOutputStream(name);
					for (int c = zis.read(); c != -1; c = zis.read())
						fout.write(c);
					zis.closeEntry();
					fout.close();
				}
			}
			zis.close();
			wait.dispose();
		}
		
		try {
			urls = new String[] {getLocalURL("addRemove.html"), getLocalURL("edit.html"), getLocalURL("connect.html"), getLocalURL("running.html"), getLocalURL("canvasResize.html")};
		} catch (MalformedURLException e2) {
			throw e2;
		}
		
		setLayout(new BorderLayout(3, 3));
		
		contentsList = new List(this, SWT.BORDER);
		contentsList.setItems(new String[] {lang.addRemoveNodes, lang.editNodes, lang.connectNodes, lang.running, lang.canvasResize});
		contentsList.setLayoutData(BorderLayout.WEST);
		contentsList.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				int index = contentsList.getSelectionIndex();
				browser.setUrl(urls[index]);
	        }
		});
		
		Composite closeComposite = new Composite(this, SWT.NONE);
		closeComposite.setLayoutData(BorderLayout.SOUTH);
		closeComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
		
		Composite buttonComposite = new Composite(closeComposite, SWT.NONE);
		buttonComposite.setLayout(new RowLayout(SWT.HORIZONTAL));
		
		btnClose = new Button(buttonComposite, SWT.NONE);
		btnClose.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				btnClose.getShell().dispose();
			}
		});
		btnClose.setText(lang.close);
		
		try {
	         browser = new Browser(this, SWT.NONE);
	         browser.setLayoutData(BorderLayout.CENTER);
	         browser.setUrl(urls[0]);
	         contentsList.setSelection(0);
		} catch (SWTError e) {
			throw new Exception(e.getMessage());
	    }
		
		createContents();
	}

	/**
	 * Create contents of the shell.
	 */
	protected void createContents() {
		setText(lang.help);
		setSize(840, 480);

	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
