package jflowchart.ui.windows;

import java.util.LinkedList;

import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.common.Const;
import jflowchart.engine.common.Functions;
import jflowchart.engine.common.Language;
import jflowchart.engine.history.EditingInputOutputAction;
import jflowchart.engine.nodes.InputOutputNode;
import jflowchart.ui.canvas.FlowChartCanvas;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import swing2swt.layout.BorderLayout;
import org.eclipse.swt.widgets.Composite;
import swing2swt.layout.FlowLayout;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

public class InputOutputSettingsShell extends Shell implements VariableSetting {

	private MainShell mainShell;
	private Language lang;
	private FlowChartCanvas canvas;
	private InputOutputNode clientNode;
	private boolean edit;
	private EditingInputOutputAction editAction;
	private boolean input;
	
	private LinkedList<Variable> variables;
	private Variable variable;
	
	private Combo variableCombo;
	private Button btnOk;
	private Button btnCancel;
	
	private boolean lastSelected(Combo c) {
		return (c.getSelectionIndex() == (c.getItemCount() - 1));
	}
	
	private void adjustOkButton() {
		btnOk.setEnabled((variable != null));
	}
	
	private void setVariable() {
		boolean lastSelected = lastSelected(variableCombo);
		if (! input) lastSelected = false;
		if (! lastSelected) {
			int selected = variableCombo.getSelectionIndex();
			if (selected > 0) 
				variable = variables.get(selected - 1);
			adjustOkButton();
		}
	}
	
	private void variableSelection() {
		int selected = variableCombo.getSelectionIndex();
		if (selected == 0) {
			variable = null;
			adjustOkButton();
		} else if ((lastSelected(variableCombo)) && input) 
			mainShell.showNewVariableShell(this, this, variableCombo, variables);
		else setVariable();
	}
	
	/**
	 * Create the shell.
	 * @param display
	 */
	public InputOutputSettingsShell(MainShell main, Language l, InputOutputNode n, boolean in, boolean editing) {
		super(main.getShell(), SWT.SHELL_TRIM | SWT.APPLICATION_MODAL);
		setLayout(new BorderLayout(0, 0));
		
		mainShell = main;
		lang = l;
		canvas = mainShell.getCanvas();
		clientNode = n;
		edit = editing;
		input = in;
		
		if (edit) editAction = new EditingInputOutputAction(canvas, clientNode);
		else editAction = null;
		
		Composite confirmComposite = new Composite(this, SWT.NONE);
		confirmComposite.setLayoutData(BorderLayout.SOUTH);
		confirmComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
		
		Composite buttonsComposite = new Composite(confirmComposite, SWT.NONE);
		FillLayout fl_buttonsComposite = new FillLayout(SWT.HORIZONTAL);
		fl_buttonsComposite.marginHeight = 9;
		buttonsComposite.setLayout(fl_buttonsComposite);
		
		btnOk = new Button(buttonsComposite, SWT.NONE);
		btnOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				clientNode.setVariable(variable);
				if (edit) {
					editAction.recordNewData();
					canvas.recordAction(editAction);
				}
				btnOk.getShell().dispose();
			}
		});
		btnOk.setText(lang.ok);
		
		btnCancel = new Button(buttonsComposite, SWT.NONE);
		btnCancel.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				btnCancel.getShell().dispose();
			}
		});
		btnCancel.setText(lang.cancel);
		
		Composite settingsComposite = new Composite(this, SWT.NONE);
		settingsComposite.setLayoutData(BorderLayout.CENTER);
		settingsComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
		
		Composite variableComposite = new Composite(settingsComposite, SWT.NONE);
		RowLayout rl_variableComposite = new RowLayout(SWT.HORIZONTAL);
		rl_variableComposite.center = true;
		variableComposite.setLayout(rl_variableComposite);
		
		Label lblVariable = new Label(variableComposite, SWT.NONE);
		lblVariable.setText(lang.variable + Const.COLON);
		
		variableCombo = new Combo(variableComposite, SWT.READ_ONLY);
		variableCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				variableSelection();
			}
		});
		
		updateVariableList();
		
		variable = clientNode.getVariable();
		if (variable != null) {
			int i = variables.indexOf(variable) + 1;
			variableCombo.select(i);
		}
		
		adjustOkButton();
		
		createContents();
	}

	/**
	 * Create contents of the shell.
	 */
	protected void createContents() {
		if (input) setText(lang.inputSettings);
		else setText(lang.outputSettings);
		setSize(Const.INPUT_OUTPUT_SETTINGS_SHELL_WIDTH, Const.INPUT_OUTPUT_SETTINGS_SHELL_HEIGHT);
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

	@Override
	public void updateVariableList() {
		variables = canvas.getVariables();
		String[] variableComboItems = Functions.getVariableComboItems(variables, lang, input);
		variableCombo.setItems(variableComboItems);
		variableCombo.select(0);
	}

	@Override
	public MainShell getMainShell() {
		return mainShell;
	}

	@Override
	public void updateVariableOperands() {
		setVariable();
	}

}
