package jflowchart.ui.windows;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.LinkedList;

import jflowchart.engine.arithmetics.Assignment;
import jflowchart.engine.arithmetics.Comparison;
import jflowchart.engine.arithmetics.NumberExpression;
import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.codegen.c.CGenerator;
import jflowchart.engine.common.Const;
import jflowchart.engine.common.Language;
import jflowchart.engine.nodes.InputOutputNode;
import jflowchart.engine.runtime.Program;
import jflowchart.engine.runtime.Semaphore;
import jflowchart.engine.runtime.SemaphoreState;
import jflowchart.engine.xml.XMLReader;
import jflowchart.engine.xml.XMLWriter;
import jflowchart.ui.canvas.CanvasEnvironment;
import jflowchart.ui.canvas.FlowChartCanvas;

import org.eclipse.swt.dnd.*;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import swing2swt.layout.BorderLayout;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.ExpandBar;
import org.eclipse.swt.widgets.ExpandItem;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.wb.swt.SWTResourceManager;
import org.eclipse.swt.events.DragDetectListener;
import org.eclipse.swt.events.DragDetectEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;

public class MainShell {

	protected static Shell shell;
	private Display display;
	private static Language lang;
	private int selectedTool;
	private int selectedActivity;
	private FlowChartCanvas canvas;
	private Composite canvasComposite;
	private ScrolledComposite canvasScrolledComposite;
	private MenuItem mntmAddingMode;
	private MenuItem mntmSelectionMode;
	private MenuItem mntmConnectingMode;
	private ToolItem addItem;
	private ToolItem selectionItem;
	private ToolItem connectingItem;
	private boolean canvasSizeShellActive;
	private Clipboard clipBoard;
	private MenuItem mntmCut;
	private MenuItem mntmCopy;
	private MenuItem mntmPaste;
	private ToolItem tltmCut;
	private ToolItem tltmCopy;
	private ToolItem tltmPaste;
	private MenuItem mntmUndo;
	private MenuItem mntmRedo;
	private ToolItem tltmUndo;
	private ToolItem tltmRedo;
	private ExpandItem xpndtmBasic;
	private Composite basicComposite;
	private ToolBar basicBar;
	private ToolItem tltmStart;
	private ToolItem tltmEnd;
	private ExpandItem xpndtmRuntimeDirection;
	private Composite directionComposite;
	private ToolBar directionBar;
	private ToolItem tltmDecision;
	private ToolItem tltmPreparation;
	private ToolItem tltmProcess;
	private ExpandItem xpndtmInputOutput;
	private Composite inputOutputComposite;
	private ToolBar inputOutputBar;
	private ToolItem tltmManualInput;
	private ToolItem tltmDisplay;
	private MenuItem mntmEditNode;
	private MenuItem mntmToggleBreakpoint;
	private Composite statusbarComposite;
	private Label lblNotifLabel;
	private Composite semaphoreComposite;
	private Composite notifComposite;
	private Composite semaphoreLightComposite;
	private Label lblSemaphoreLabel;
	private Label semaphoreLightLabel;
	private ToolItem playItem;
	private ToolItem pauseItem;
	private ToolItem stopItem;
	private ToolItem stepItem;
	private ToolItem breakpointItem;
	private MenuItem mntmStepping;
	private Menu steppingMenu;
	private MenuItem mntmPlay;
	private MenuItem mntmPause;
	private MenuItem mntmStop;
	private MenuItem mntmNextStep;
	private Table variablesTable;
	private TableColumn tblclmnName;
	private TableColumn tblclmnValue;
	private Composite centerComposite;
	private Text consoleText;
	private MenuItem mntmSave;
	private MenuItem mntmSaveAs;
	private String currentFile;
	private boolean savedFlag;
	@SuppressWarnings("unused")
	private ToolItem fileHistorySeparator;
	private ToolItem tltmSave;
	private MenuItem mntmNew;
	private ToolItem tltmNew;
	private MenuItem mntmOpen;
	private File activeFile;
	private ToolItem tltmOpen;
	private MenuItem mntmGenerateCSource;
	private MenuItem mntmExportPNG;
	private ToolItem tltmPng;
	private ToolItem tltmCSource;
	private Font boldFont;
	
	/**
	 * Launch the application.
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			MainShell window = new MainShell();
			Display display = Display.getDefault();
			window.open(display);
		} catch (Exception e) {
			e.printStackTrace();
			showErrorMessage(lang.swtError);
		}
	}

	/**
	 * Open the window.
	 */
	public void open(Display display) {
		createContents(display);
		shell.open();
		shell.layout();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}
	
	private ToolItem getToolItem(final int code) {
		ToolItem ret = null;
		if (code == Const.START_SELECTED) ret = tltmStart;
		else if (code == Const.END_SELECTED) ret = tltmEnd;
		else if (code == Const.DECISION_SELECTED) ret = tltmDecision;
		else if (code == Const.PROCESS_SELECTED) ret = tltmProcess;
		else if (code == Const.PREPARATION_SELECTED) ret = tltmPreparation;
		else if (code == Const.MANUAL_INPUT_SELECTED) ret = tltmManualInput;
		else if (code == Const.DISPLAY_SELECTED) ret = tltmDisplay;
		return ret;
	}
	
	private void setSelectedTool(final int code) {
		discardNotification();
		ToolItem ti = getToolItem(code);
		adjustPalette(ti);
		selectedTool = code;
	}
	
	private boolean addingMode() {
		return (selectedActivity == Const.ADDING_ACTIVE);
	}
	
	private boolean selectionMode() {
		return (selectedActivity == Const.SELECTING_ACTIVE);
	}
	
	private void switchActivity(int code) {
		discardNotification();
		selectedActivity = code;
		canvas.discardGhost();
		discardConnection();
		canvas.redraw();
		mntmAddingMode.setSelection(addingMode());
		mntmSelectionMode.setSelection((! addingMode()) && selectionMode());
		mntmConnectingMode.setSelection((! addingMode()) && (! selectionMode()));
		addItem.setSelection(addingMode());
		selectionItem.setSelection((! addingMode()) && selectionMode());
		connectingItem.setSelection((! addingMode()) && (! selectionMode()));
	}
	
	private void switchAddingMode() {
		switchActivity(Const.ADDING_ACTIVE);
	}
	
	private void switchSelectionMode() {
		switchActivity(Const.SELECTING_ACTIVE);
	}
	
	private void switchConnectingMode() {
		switchActivity(Const.CONNECTING_ACTIVE);
	}
	
	public void changeCanvasSize(int width, int height) {
		if (((canvas.getMinimalWidth() <= width) && (width <= Const.MAXIMUM_CANVAS_SIZE)) 
				&& ((canvas.getMinimalHeight() <= height) && (height <= Const.MAXIMUM_CANVAS_SIZE))) { 
			canvas.setLayoutData(new RowData(width, height));
			canvasComposite.pack();
			canvasScrolledComposite.setMinSize(width, height);
			adjustImageExportItems();
		}
	}
	
	public Rectangle getCanvasClient() {
		return canvas.getClientArea();
	}
	
	public Language getLanguage() {
		return lang;
	}
	
	public void showCanvasSizeShell() {
		if (! canvasSizeShellActive) {
			try {
				CanvasSizeShell canvasSizeShell = new CanvasSizeShell(display, this);
				canvasSizeShell.open();
				canvasSizeShell.layout();
				canvasSizeShellActive = true;
				while (!canvasSizeShell.isDisposed()) {
					if (!display.readAndDispatch()) {
						display.sleep();
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
				showErrorMessage(lang.swtError);
			}
		}
	}
	
	public void showConditionSettingsShell(Shell parent, Comparison cond, boolean editing) {
		try {
			ConditionSettingsShell shell = new ConditionSettingsShell(this, parent, cond, editing);
			shell.open();
			shell.layout();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch()) {}
			}
			canvas.updateSemaphore();
		} catch (Exception e) {
			e.printStackTrace();
			showErrorMessage(lang.swtError);
		}
	}
	
	public void showAssignmentSettingsShell(Shell parent, Assignment a, boolean newVar, boolean editing) {
		try {
			AssignmentSettingsShell shell = new AssignmentSettingsShell(this, parent, a, newVar, editing);
			shell.open();
			shell.layout();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
			canvas.updateSemaphore();
		} catch (Exception e) {
			e.printStackTrace();
			showErrorMessage(lang.swtError);
		}
	}
	
	public void showNewVariableShell(Shell parent, VariableSetting client, Combo parentCombo, LinkedList<Variable> vars) {
		try {
			NewVariableShell shell = new NewVariableShell(parent, client, lang, parentCombo, vars);
			shell.open();
			shell.layout();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			showErrorMessage(lang.swtError);
		}
	}
	
	public void showNewNumberExpressionShell(Shell parent, NumberExpressionSetting client, NumberExpression exp) {
		try {
			NewNumberExpressionShell shell = new NewNumberExpressionShell(this, parent, client, exp);
			shell.open();
			shell.layout();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			showErrorMessage(lang.swtError);
		}
	}
	
	public void showPreparationSettingsShell(Comparison c, Assignment a, boolean editing) {
		try {
			PreparationSettingsShell shell = new PreparationSettingsShell(this, lang, c, a, editing);
			shell.open();
			shell.layout();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
			canvas.updateSemaphore();
		} catch (Exception e) {
			e.printStackTrace();
			showErrorMessage(lang.swtError);
		}
	}
	
	public void showInputOutputSettingsShell(InputOutputNode client, boolean input, boolean editing) {
		try {
			InputOutputSettingsShell shell = new InputOutputSettingsShell(this, lang, client, input, editing);
			shell.open();
			shell.layout();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
			canvas.updateSemaphore();
		} catch (Exception e) {
			e.printStackTrace();
			showErrorMessage(lang.swtError);
		}
	}
	
	public void showManualInputShell(Program p, Variable v) {
		try {
			ManualInputShell shell = new ManualInputShell(this, p, v);
			shell.open();
			shell.layout();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			showErrorMessage(lang.swtError);
		}
	}
	
	public void showHelpShell() {
		try {
			Display display = Display.getDefault();
			HelpShell help = new HelpShell(shell, lang);
			help.open();
			help.layout();
			while (!help.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
		} catch (Exception e) {
			showErrorMessage(lang.helpError);
		}
	}
	
	public void discardCanvasSizeShell() {
		canvasSizeShellActive = false;
	}
	
	public int getCanvasMinimalWidth() {
		return canvas.getMinimalWidth();
	}
	
	public int getCanvasMinimalHeight() {
		return canvas.getMinimalHeight();
	}
	
	public void copy() {
		discardNotification();
		String s = canvas.copy();
		if (s.length() > 0) {
			TextTransfer textTransfer = TextTransfer.getInstance();
			clipBoard.setContents(new Object[]{s}, new Transfer[]{textTransfer});
		}
	}
	
	public void cut() {
		copy();
		canvas.deleteSelected();
	}
	
	public void paste() {
		discardNotification();
		TextTransfer transfer = TextTransfer.getInstance();
		String s = (String)clipBoard.getContents(transfer);
		if (s != null) {
			if (s.contains(Const.START_NODE_CODE) && canvas.hasStart())
				setNotification(lang.startSet);
			else canvas.paste(s);
		}
	}
	
	private void setCopyClip(boolean val) {
		mntmCopy.setEnabled(val);
		mntmCut.setEnabled(val);
		tltmCopy.setEnabled(val);
		tltmCut.setEnabled(val);
		if (canvas != null) canvas.setCopyClip(val);
	}
	
	public FlowChartCanvas getCanvas() {
		return canvas;
	}
	
	public void adjustUndoRedo() {
		mntmUndo.setEnabled(! canvas.isUndoStackEmpty());
		mntmRedo.setEnabled(! canvas.isRedoStackEmpty());
		tltmUndo.setEnabled(! canvas.isUndoStackEmpty());
		tltmRedo.setEnabled(! canvas.isRedoStackEmpty());
		canvas.setUndoRedo();
	}
	
	public void undo() {
		discardNotification();
		discardConnection();
		canvas.undo();
		adjustUndoRedo();
	}
	
	public void redo() {
		discardNotification();
		discardConnection();
		canvas.redo();
		adjustUndoRedo();
	}
	
	public Point getMainShellLocation() {
		return shell.getLocation();
	}
	
	public Point getMainShellSize() {
		return shell.getSize();
	}
	
	private void unselectPaletteItem(ToolItem tm) {
		tm.setSelection(false);
	}
	
	private void adjustPalette(ToolItem butThis) {
		unselectPaletteItem(tltmStart);
		unselectPaletteItem(tltmEnd);
		unselectPaletteItem(tltmDecision);
		unselectPaletteItem(tltmProcess);
		unselectPaletteItem(tltmPreparation);
		unselectPaletteItem(tltmManualInput);
		unselectPaletteItem(tltmDisplay);
		butThis.setSelection(true);
	}
	
	public void setEditNodeMenuItem(boolean enabled) {
		mntmEditNode.setEnabled(enabled);
	}
	
	public void setNotification(String notif) {
		lblNotifLabel.setText(notif);
	}
	
	public void discardNotification() {
		lblNotifLabel.setText("");
	}
	
	private void discardConnection() {
		if ((canvas.getConnectingFrom() != null) && (canvas.getConnectingTo() == null) 
				&& (canvas.getPointedToConnect() == null)) {
			if (canvas.editingConnection()) canvas.discardEditConnectionMode();
			else canvas.discardConnectingFrom();
		}
	}
	
	private void setConnectingHint() {
		setNotification(lang.connectingMessage);
	}
	
	private void adjustToggleBreakpointOption() {
		boolean one = canvas.oneNodeSelected();
		this.breakpointItem.setEnabled(one);
		this.mntmToggleBreakpoint.setEnabled(one);
		canvas.setToggleBreakpoint(one);
	}
	
	private void updateWindowTitle() {
		String s = Const.PROGRAM_NAME;
		if (currentFile != null)
			s = s.concat(" - ").concat(currentFile);
		if (!savedFlag)
			s = "* ".concat(s);
		shell.setText(s);
	}
	
	private boolean saveDialog() {
		FileDialog fd = new FileDialog(shell, SWT.SAVE);
        fd.setText(lang.saveAs);
        fd.setFilterPath(System.getProperty("user.home"));
        String[] filterExt = { "*.xml" };
        fd.setFilterExtensions(filterExt);
        String filename = fd.open();
        if (filename != null) {
    		activeFile = new File(filename);
        	currentFile = activeFile.getName();
    		updateWindowTitle();
    		return true;
        } else return false;
	}
	
	public void setSavedFlag() {
		savedFlag = false;
		updateWindowTitle();
	}
	
	private static void showErrorMessage(String message) {
		MessageBox messageBox = new MessageBox(shell, SWT.ICON_ERROR);
		messageBox.setMessage(message);
	    messageBox.open();
	}
	
	private void showInfoMessage(String message) {
		MessageBox messageBox = new MessageBox(shell, SWT.ICON_INFORMATION);
		messageBox.setMessage(message);
	    messageBox.open();
	}
	
	private boolean save() {
		try {
			FileOutputStream saveStream = new FileOutputStream(activeFile);
			XMLWriter wr = new XMLWriter();
			wr.save(saveStream, canvas);
			saveStream.flush();
			saveStream.close();
			savedFlag = true;
			updateWindowTitle();
			return true;
		} catch (Exception e2) {
			String message = lang.saveError;
			if (currentFile != null)
				message = message.concat(currentFile).concat(" ");
			message = message.concat(lang.hdFreeSpace);
		    e2.printStackTrace();
			showErrorMessage(message);
		    return false;
		}
	}
	
	private boolean saveAs() {
		discardNotification();
		if (saveDialog()) return save();
		else return false;
	}
	
	private boolean performSave() {
		if (currentFile == null) return saveAs();
		else return save();
	}
	
	private CanvasEnvironment clearCanvas() {
		boolean f = savedFlag;
		CanvasEnvironment e = canvas.empty();
		e.setCurrentFile(activeFile);
		currentFile = null;
		e.setSavedFlag(f);
		savedFlag = true;
		updateWindowTitle();
		return e;
	}
	
	private void promptedClear() {
		discardNotification();
		if (!savedFlag) {
			UnsavedChangesDialog d = new UnsavedChangesDialog(shell, lang);
			UnsavedChangesOption o = d.getResult();
			if (o == UnsavedChangesOption.SAVE) {
				if (performSave()) clearCanvas();
			} else if (o == UnsavedChangesOption.DISCARD)
				clearCanvas();
		} else {
			MessageBox messageBox = new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.YES | SWT.NO | SWT.ICON_QUESTION);
	        messageBox.setText(lang.newCanvas);
	        messageBox.setMessage(lang.newCanvasPrompt);
	        if (messageBox.open() == SWT.YES)
	        	clearCanvas();
		}
	}
	
	private void recover(CanvasEnvironment e) {
		canvas.recover(e);
		activeFile = e.getActiveFile();
		if (activeFile != null) 
			currentFile = activeFile.getName();
		else currentFile = null;
		savedFlag = e.getSavedFlag();
		updateWindowTitle();
	}
	
	private void openFile() {
		FileDialog fd = new FileDialog(shell, SWT.OPEN);
        fd.setText(lang.open);
        fd.setFilterPath(System.getProperty("user.home"));
        String[] filterExt = { "*.xml" };
        fd.setFilterExtensions(filterExt);
        String filename = fd.open();
        CanvasEnvironment e = null;
        if (filename != null) {
        	try {
    			File f = new File(filename);
    			FileInputStream s = new FileInputStream(f);
    			XMLReader r = new XMLReader();
    			e = clearCanvas();
    			r.load(s, canvas);
    			s.close();
    			activeFile = f;
    			currentFile = activeFile.getName();
    			savedFlag = true;
    			updateWindowTitle();
    		} catch (Exception ex) {
    			ex.printStackTrace();
    			showErrorMessage(lang.loadError);
    			if (e != null) recover(e);
    		}
        }
	}
	
	private void promptedOpenFile() {
		discardNotification();
		if (!savedFlag) {
			UnsavedChangesDialog d = new UnsavedChangesDialog(shell, lang);
			UnsavedChangesOption o = d.getResult();
			if (o == UnsavedChangesOption.SAVE) {
				if (performSave()) openFile();
			} else if (o == UnsavedChangesOption.DISCARD)
				openFile();
		} else openFile();
	}
	
	private void adjustImageExportItems(boolean val) {
		mntmExportPNG.setEnabled(val);
		tltmPng.setEnabled(val);
		if (val) tltmPng.setToolTipText(lang.exportToPNG);
		else tltmPng.setToolTipText(lang.unableToExportPNG);
	}
	
	public void adjustImageExportItems() {
		Rectangle scrolls = canvasScrolledComposite.getBounds();
		boolean val = ((canvas.getMinimalWidth() < scrolls.width) && (canvas.getMinimalHeight() < scrolls.height));
		adjustImageExportItems(val);
	}
	
	private void exportToCSource() {
		
		CGenerator gen = new CGenerator();
		String code = gen.generate(canvas.getProgram());
		
		FileDialog fd = new FileDialog(shell, SWT.SAVE);
        fd.setText(lang.exportAsC);
        fd.setFilterPath(System.getProperty("user.home"));
        String[] filterExt = { "*.c" };
        fd.setFilterExtensions(filterExt);
        String filename = fd.open();
		if (filename != null) {
			PrintStream out = null;
			try {
			    out = new PrintStream(new FileOutputStream(filename));
			    out.print(code);
			    showInfoMessage(lang.codeGenerated);
			} catch (FileNotFoundException e1) {
				e1.printStackTrace();
				showErrorMessage(lang.savingError);
			} finally {
			    if (out != null) out.close();
			}
		}
		
	}
	
	private void exportToPNG(Display d) {
		
		FileDialog fd = new FileDialog(shell, SWT.SAVE);
        fd.setText(lang.exportToPNG);
        fd.setFilterPath(System.getProperty("user.home"));
        String[] filterExt = { "*.png" };
        fd.setFilterExtensions(filterExt);
        String filename = fd.open();
		if (filename != null) {
			Image image = new Image(d, canvasScrolledComposite.getBounds());
			GC gc = new GC(image);
			canvas.print(gc);
			ImageLoader loader = new ImageLoader();
			loader.data = new ImageData[] {image.getImageData()};
			loader.save(filename, SWT.IMAGE_PNG);
			image.dispose();
			gc.dispose();
			showInfoMessage(lang.imageCreated);
		}
		
	}

	/**
	 * Create contents of the window.
	 */
	protected void createContents(Display d) {
		
		shell = new Shell();
		shell.setMinimumSize(new Point(Const.MAIN_SHELL_DEFAULT_WIDTH, Const.MAIN_SHELL_DEFAULT_HEIGHT));
		shell.setLocation(Const.MAIN_SHELL_XLOCATION, Const.MAIN_SHELL_YLOCATION);
		
		shell.addListener(SWT.Close, new Listener() {
			public void handleEvent(Event event) {
				if (!savedFlag) {
					UnsavedChangesDialog d = new UnsavedChangesDialog(shell, lang);
					UnsavedChangesOption o = d.getResult();
					if (o == UnsavedChangesOption.SAVE) {
						if (performSave()) shell.dispose();
						else event.doit = false;
					} else if (o == UnsavedChangesOption.DISCARD)
						shell.dispose();
					else event.doit = false;
				} else {
			        MessageBox messageBox = new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.YES | SWT.NO | SWT.ICON_QUESTION);
			        messageBox.setText(lang.exit);
			        messageBox.setMessage(lang.exitPrompt);
			        event.doit = messageBox.open() == SWT.YES;
				}
		    }
		});
		
		display = d;
		canvasSizeShellActive = false;
		
		shell.setSize(923, 574);
		shell.setMaximized(true);
		savedFlag = true;
		updateWindowTitle();
		
		shell.setLayout(new BorderLayout(0, 5));
		
		Menu menu = new Menu(shell, SWT.BAR);
		shell.setMenuBar(menu);
		
		lang = new Language();
		
		selectedTool = Const.NONE_TOOL_SELECTED;
		selectedActivity = Const.ADDING_ACTIVE;
		
		clipBoard = new Clipboard(display);
	
		MenuItem mntmFile = new MenuItem(menu, SWT.CASCADE);
		mntmFile.setText(lang.file);
		
		Menu fileMenu = new Menu(mntmFile);
		mntmFile.setMenu(fileMenu);
		
		activeFile = null;
		
		mntmNew = new MenuItem(fileMenu, SWT.NONE);
		mntmNew.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				promptedClear();
			}
		});
		mntmNew.setText(lang.neww + "\tCtrl + N");
		mntmNew.setAccelerator(SWT.CTRL + 'n');
		
		new MenuItem(fileMenu, SWT.SEPARATOR);
		
		mntmOpen = new MenuItem(fileMenu, SWT.NONE);
		mntmOpen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				promptedOpenFile();
			}
		});
		mntmOpen.setText(lang.open + "\tCtrl + 0");
		mntmOpen.setAccelerator(SWT.CTRL + 'o');
		
		new MenuItem(fileMenu, SWT.SEPARATOR);
		
		mntmSave = new MenuItem(fileMenu, SWT.NONE);
		mntmSave.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				performSave();
			}
		});
		mntmSave.setText(lang.save + "\tCtrl + S");
		mntmSave.setAccelerator(SWT.CTRL + 's');
		
		currentFile = null;
		
		mntmSaveAs = new MenuItem(fileMenu, SWT.NONE);
		mntmSaveAs.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				saveAs();
			}
		});
		mntmSaveAs.setText(lang.saveAs + "\tCtrl + Shift + S");
		mntmSaveAs.setAccelerator(SWT.CTRL + SWT.SHIFT + 's');
		
		MenuItem menuItem = new MenuItem(fileMenu, SWT.SEPARATOR);
		menuItem.setText("quitSeparator");
		
		MenuItem mntmQuit = new MenuItem(fileMenu, SWT.NONE);
		mntmQuit.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				shell.close();
			}
		});
		mntmQuit.setText(lang.quit + "\tCtrl + Q");
		mntmQuit.setAccelerator(SWT.CTRL + 'Q');
		
		MenuItem mntmEdit = new MenuItem(menu, SWT.CASCADE);
		mntmEdit.setText(lang.edit);
		
		Menu editMenu = new Menu(mntmEdit);
		mntmEdit.setMenu(editMenu);
		
		mntmUndo = new MenuItem(editMenu, SWT.NONE);
		mntmUndo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				undo();
			}
		});
		mntmUndo.setText(lang.undo + "\tCtrl + Z");
		mntmUndo.setAccelerator(SWT.CTRL + 'Z');
		
		mntmRedo = new MenuItem(editMenu, SWT.NONE);
		mntmRedo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				redo();
			}
		});
		mntmRedo.setText(lang.redo + "\tCtrl + Y");
		mntmRedo.setAccelerator(SWT.CTRL + 'Y');
		
		new MenuItem(editMenu, SWT.SEPARATOR);
		
		mntmCut = new MenuItem(editMenu, SWT.NONE);
		mntmCut.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				cut();
			}
		});
		mntmCut.setText(lang.cut + "\tCtrl + X");
		mntmCut.setAccelerator(SWT.CTRL + 'X');
		
		mntmCopy = new MenuItem(editMenu, SWT.NONE);
		mntmCopy.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				copy();
			}
		});
		mntmCopy.setText(lang.copy + "\tCtrl + C");
		mntmCopy.setAccelerator(SWT.CTRL + 'C');
		
		mntmPaste = new MenuItem(editMenu, SWT.NONE);
		mntmPaste.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				paste();
			}
		});
		mntmPaste.setText(lang.paste + "\tCtrl + V");
		mntmPaste.setAccelerator(SWT.CTRL + 'V');
		
		new MenuItem(editMenu, SWT.SEPARATOR);
		
		mntmEditNode = new MenuItem(editMenu, SWT.NONE);
		mntmEditNode.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				canvas.showSettings();
			}
		});
		mntmEditNode.setText(lang.editNode + Const.ELLIPSIS + "\tCtrl + E");
		mntmEditNode.setAccelerator(SWT.CTRL + 'E');
		setEditNodeMenuItem(false);
		
		mntmToggleBreakpoint = new MenuItem(editMenu, SWT.NONE);
		mntmToggleBreakpoint.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				canvas.toggleBreakpoint();
			}
		});
		mntmToggleBreakpoint.setText(lang.toggleBreakpoint + "\tCtrl + B");
		mntmToggleBreakpoint.setAccelerator(SWT.CTRL + 'B');
		
		new MenuItem(editMenu, SWT.SEPARATOR);
		
		MenuItem mntmMode = new MenuItem(editMenu, SWT.CASCADE);
		mntmMode.setText(lang.mode);
		
		Menu modeMenu = new Menu(mntmMode);
		mntmMode.setMenu(modeMenu);
		
		mntmAddingMode = new MenuItem(modeMenu, SWT.RADIO);
		mntmAddingMode.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				switchAddingMode();
			}
		});
		mntmAddingMode.setSelection(true);
		mntmAddingMode.setText(lang.adding + "\tCtrl + Alt + A");
		mntmAddingMode.setAccelerator(SWT.CTRL + SWT.ALT + 'A');
		
		mntmSelectionMode = new MenuItem(modeMenu, SWT.RADIO);
		mntmSelectionMode.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				switchSelectionMode();
			}
		});
		mntmSelectionMode.setText(lang.selecting + "\tCtrl + Alt + S");
		mntmSelectionMode.setAccelerator(SWT.CTRL + SWT.ALT + 'S');
		
		mntmConnectingMode = new MenuItem(modeMenu, SWT.RADIO);
		mntmConnectingMode.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				switchConnectingMode();
			}
		});
		mntmConnectingMode.setText(lang.connecting + "\tCtrl + Alt + C");
		mntmConnectingMode.setAccelerator(SWT.CTRL + SWT.ALT + 'C');
		
		MenuItem mntmCanvasSize = new MenuItem(editMenu, SWT.NONE);
		mntmCanvasSize.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				discardConnection();
				showCanvasSizeShell();
			}
		});
		mntmCanvasSize.setText(lang.canvasSize + Const.ELLIPSIS);
		
		mntmStepping = new MenuItem(menu, SWT.CASCADE);
		mntmStepping.setText(lang.stepping);
		
		steppingMenu = new Menu(mntmStepping);
		mntmStepping.setMenu(steppingMenu);
		
		mntmPlay = new MenuItem(steppingMenu, SWT.NONE);
		mntmPlay.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				discardNotification();
				canvas.play();
			}
		});
		mntmPlay.setText(lang.playDiagram + "\tP");
		mntmPlay.setAccelerator('P');
		
		mntmPause = new MenuItem(steppingMenu, SWT.NONE);
		mntmPause.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				canvas.pause();
			}
		});
		mntmPause.setText(lang.pauseDiagram + "\tP");
		mntmPause.setAccelerator('P');
		
		mntmStop = new MenuItem(steppingMenu, SWT.NONE);
		mntmStop.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				canvas.stop();
			}
		});
		mntmStop.setText(lang.stopDiagram + "\tS");
		mntmStop.setAccelerator('S');
		
		mntmNextStep = new MenuItem(steppingMenu, SWT.NONE);
		mntmNextStep.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				canvas.nextStep();
			}
		});
		mntmNextStep.setText(lang.nextStep + "\tN");
		mntmStop.setAccelerator('N');
		
		MenuItem mntmExport = new MenuItem(menu, SWT.CASCADE);
		mntmExport.setText(lang.export);
		
		Menu exportMenu = new Menu(mntmExport);
		mntmExport.setMenu(exportMenu);
		
		mntmGenerateCSource = new MenuItem(exportMenu, SWT.NONE);
		mntmGenerateCSource.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				exportToCSource();
			}
		});
		mntmGenerateCSource.setText(lang.generateC);
		
		mntmExportPNG = new MenuItem(exportMenu, SWT.NONE);
		mntmExportPNG.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {	
				exportToPNG(e.display);
			}
		});
		mntmExportPNG.setText(lang.exportToPNG);
		
		MenuItem mntmHelp = new MenuItem(menu, SWT.CASCADE);
		mntmHelp.setText(lang.help);
		
		Menu helpMenu = new Menu(mntmHelp);
		mntmHelp.setMenu(helpMenu);
		
		MenuItem mntmContents = new MenuItem(helpMenu, SWT.NONE);
		mntmContents.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				showHelpShell();
			}
		});
		mntmContents.setText(lang.contents + "\tF1");
		mntmContents.setAccelerator(SWT.F1);
		
		 ExpandBar expandBar = new ExpandBar(shell, SWT.V_SCROLL);
		 expandBar.setSpacing(Const.EXPAND_BAR_SPACING);
		 expandBar.setLayoutData(BorderLayout.WEST);
		 
		 xpndtmBasic = new ExpandItem(expandBar, SWT.NONE);
		 xpndtmBasic.setExpanded(true);
		 xpndtmBasic.setText(lang.basic);
		 
		 basicComposite = new Composite(expandBar, SWT.NONE);
		 xpndtmBasic.setControl(basicComposite);
		 RowLayout rl_basicComposite = new RowLayout(SWT.HORIZONTAL);
		 rl_basicComposite.justify = true;
		 rl_basicComposite.center = true;
		 rl_basicComposite.fill = true;
		 basicComposite.setLayout(rl_basicComposite);
		 
		 basicBar = new ToolBar(basicComposite, SWT.FLAT | SWT.RIGHT | SWT.VERTICAL);
		 
		 tltmStart = new ToolItem(basicBar, SWT.RADIO);
		 tltmStart.addSelectionListener(new SelectionAdapter() {
		 	@Override
		 	public void widgetSelected(SelectionEvent e) {
		 		setSelectedTool(Const.START_SELECTED);
		 	}
		 });
		 tltmStart.setText(lang.start);
		 
		 tltmEnd = new ToolItem(basicBar, SWT.RADIO);
		 tltmEnd.addSelectionListener(new SelectionAdapter() {
		 	@Override
		 	public void widgetSelected(SelectionEvent e) {
		 		setSelectedTool(Const.END_SELECTED);
		 	}
		 });
		 tltmEnd.setText(lang.end);
		 
		 tltmProcess = new ToolItem(basicBar, SWT.RADIO);
		 tltmProcess.addSelectionListener(new SelectionAdapter() {
		 	@Override
		 	public void widgetSelected(SelectionEvent e) {
		 		setSelectedTool(Const.PROCESS_SELECTED);
		 	}
		 });
		 tltmProcess.setText(lang.process);
		 xpndtmBasic.setHeight(xpndtmBasic.getControl().computeSize(SWT.DEFAULT, SWT.DEFAULT).y);
		 
		 xpndtmRuntimeDirection = new ExpandItem(expandBar, SWT.NONE);
		 xpndtmRuntimeDirection.setExpanded(true);
		 xpndtmRuntimeDirection.setText(lang.runtimeDirection);
		 
		 directionComposite = new Composite(expandBar, SWT.NONE);
		 xpndtmRuntimeDirection.setControl(directionComposite);
		 
		 RowLayout rl_directionComposite = new RowLayout(SWT.HORIZONTAL);
		 rl_directionComposite.justify = true;
		 rl_directionComposite.center = true;
		 rl_directionComposite.fill = true;
		 directionComposite.setLayout(rl_directionComposite);
		 
		 directionBar = new ToolBar(directionComposite, SWT.FLAT | SWT.RIGHT | SWT.VERTICAL);
		 
		 tltmDecision = new ToolItem(directionBar, SWT.RADIO);
		 tltmDecision.addSelectionListener(new SelectionAdapter() {
		 	@Override
		 	public void widgetSelected(SelectionEvent e) {
		 		setSelectedTool(Const.DECISION_SELECTED);
		 	}
		 });
		 tltmDecision.setText(lang.decision);
		 
		 tltmPreparation = new ToolItem(directionBar, SWT.RADIO);
		 tltmPreparation.addSelectionListener(new SelectionAdapter() {
		 	@Override
		 	public void widgetSelected(SelectionEvent e) {
		 		setSelectedTool(Const.PREPARATION_SELECTED);
		 	}
		 });
		 tltmPreparation.setText(lang.preparation);
		 
		 xpndtmRuntimeDirection.setHeight(xpndtmRuntimeDirection.getControl().computeSize(SWT.DEFAULT, SWT.DEFAULT).y);
		 
		 xpndtmInputOutput = new ExpandItem(expandBar, SWT.NONE);
		 xpndtmInputOutput.setExpanded(true);
		 xpndtmInputOutput.setText(lang.input + Const.SPACE + Const.SLASH + Const.SPACE + lang.output);
		 
		 inputOutputComposite = new Composite(expandBar, SWT.NONE);
		 xpndtmInputOutput.setControl(inputOutputComposite);
		 
		 RowLayout rl_inputOutputComposite = new RowLayout(SWT.HORIZONTAL);
		 rl_inputOutputComposite.justify = true;
		 rl_inputOutputComposite.fill = true;
		 rl_inputOutputComposite.center = true;
		 inputOutputComposite.setLayout(rl_inputOutputComposite);
		 
		 inputOutputBar = new ToolBar(inputOutputComposite, SWT.FLAT | SWT.RIGHT | SWT.VERTICAL);
		 
		 tltmManualInput = new ToolItem(inputOutputBar, SWT.RADIO);
		 tltmManualInput.addSelectionListener(new SelectionAdapter() {
		 	@Override
		 	public void widgetSelected(SelectionEvent e) {
		 		setSelectedTool(Const.MANUAL_INPUT_SELECTED);
		 	}
		 });
		 tltmManualInput.setText(lang.manualInput);
		 
		 tltmDisplay = new ToolItem(inputOutputBar, SWT.RADIO);
		 tltmDisplay.addSelectionListener(new SelectionAdapter() {
		 	@Override
		 	public void widgetSelected(SelectionEvent e) {
		 		setSelectedTool(Const.DISPLAY_SELECTED);
		 	}
		 });
		 tltmDisplay.setText(lang.display);
		 
		 xpndtmInputOutput.setHeight(xpndtmInputOutput.getControl().computeSize(SWT.DEFAULT, SWT.DEFAULT).y);
		
		ToolBar toolBar = new ToolBar(shell, SWT.FLAT | SWT.RIGHT);
		toolBar.setLayoutData(BorderLayout.NORTH);
		
		tltmNew = new ToolItem(toolBar, SWT.NONE);
		tltmNew.setToolTipText(lang.neww);
		tltmNew.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				promptedClear();
			}
		});
		tltmNew.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/new.png"));
		
		tltmOpen = new ToolItem(toolBar, SWT.NONE);
		tltmOpen.setToolTipText(lang.open);
		tltmOpen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				promptedOpenFile();
			}
		});
		tltmOpen.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/open.png"));
		
		tltmSave = new ToolItem(toolBar, SWT.NONE);
		tltmSave.setToolTipText(lang.save);
		tltmSave.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				performSave();
			}
		});
		tltmSave.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/save.png"));
		
		fileHistorySeparator = new ToolItem(toolBar, SWT.SEPARATOR);
		
		tltmUndo = new ToolItem(toolBar, SWT.NONE);
		tltmUndo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				undo();
			}
		});
		tltmUndo.setToolTipText(lang.undo);
		tltmUndo.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/undo.png"));
		
		tltmRedo = new ToolItem(toolBar, SWT.NONE);
		tltmRedo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				redo();
			}
		});
		tltmRedo.setToolTipText(lang.redo);
		tltmRedo.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/redo.png"));
		
		centerComposite = new Composite(shell, SWT.NONE);
		centerComposite.setLayoutData(BorderLayout.CENTER);
		centerComposite.setLayout(new BorderLayout(0, 5));
		
		canvasScrolledComposite = new ScrolledComposite(centerComposite, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		canvasScrolledComposite.addControlListener(new ControlAdapter() {
			@Override
			public void controlResized(ControlEvent e) {
				adjustImageExportItems();
			}
		});
		canvasScrolledComposite.setLayoutData(BorderLayout.CENTER);
		canvasScrolledComposite.setExpandHorizontal(true);
		canvasScrolledComposite.setExpandVertical(true);
		
		canvasComposite = new Composite(canvasScrolledComposite, SWT.NONE);
		canvasComposite.setLayout(new RowLayout(SWT.HORIZONTAL));

		canvas = new FlowChartCanvas(display, this, canvasComposite, SWT.NO_BACKGROUND, lang);
		canvas.addControlListener(new ControlAdapter() {
			@Override
			public void controlResized(ControlEvent e) {
				adjustImageExportItems();
			}
		});
		canvas.setLayoutData(new RowData(Const.DEFAULT_CANVAS_WIDTH, Const.DEFAULT_CANVAS_HEIGHT));
		canvas.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		canvas.addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent e) {
				
				if (selectedActivity == Const.SELECTING_ACTIVE) {
					if (e.character == SWT.DEL) canvas.deleteSelected();
					else if (e.keyCode == SWT.CTRL) canvas.setMultipleSelection(true);
				}
				
				if (e.keyCode == SWT.ESC)
					discardConnection();
			}
			@Override
			public void keyReleased(KeyEvent e) {
				if ((selectedActivity == Const.SELECTING_ACTIVE) && (e.keyCode == SWT.CTRL))
					canvas.setMultipleSelection(false);
			}
		});
		canvas.addDragDetectListener(new DragDetectListener() {
			public void dragDetected(DragDetectEvent e) {
				if ((selectedActivity == Const.SELECTING_ACTIVE) && (e.button == Const.LEFT_MOUSE)) {
					canvas.dragOn(e.x, e.y);
				}
			}
		});
		canvas.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDown(MouseEvent e) {
				
				canvas.hideContextMenu();
				
				if ((selectedActivity == Const.ADDING_ACTIVE) && (e.button == Const.LEFT_MOUSE)) {
					if (selectedTool == Const.START_SELECTED) {
						if (canvas.hasStart()) 
							canvas.getMainShell().setNotification(lang.startSet);
						else canvas.addStartGraphNode(e.x, e.y);
					} else if (selectedTool == Const.END_SELECTED) 
						canvas.addEndGraphNode(e.x, e.y);
					else if (selectedTool == Const.DECISION_SELECTED) 
						canvas.addDecisionGraphNode(e.x, e.y);
					else if (selectedTool == Const.PROCESS_SELECTED) 
						canvas.addProcessGraphNode(e.x, e.y);
					else if (selectedTool == Const.PREPARATION_SELECTED) 
						canvas.addPreparationGraphNode(e.x, e.y);
					else if (selectedTool == Const.MANUAL_INPUT_SELECTED)
						canvas.addManualInputGraphNode(e.x, e.y);
					else if (selectedTool == Const.DISPLAY_SELECTED)
						canvas.addDisplayGraphNode(e.x, e.y);
				} else if ((selectedActivity == Const.CONNECTING_ACTIVE) && (e.button == Const.LEFT_MOUSE)) {
					
					if (canvas.getPointedToRemove() != null) canvas.removePointedConnection();
					else if (canvas.getPointedToEdit() != null) {
						canvas.editConnectionMode();
						setConnectingHint();
					} else if ((canvas.getConnectingFrom() != null) && (canvas.getConnectingTo() == null) 
								&& (canvas.getPointedToConnect() == null)) {
						canvas.placeAssistantConnectPoint();
						setConnectingHint();
					} else if ((canvas.getConnectingFrom() == null) && (canvas.getPointedToConnect() != null)) {
						canvas.setConnectingFrom();
						setConnectingHint();
					} else if ((canvas.getConnectingFrom() != null) && (canvas.getPointedToConnect() != null)) { 
						if (canvas.editingConnection() && (canvas.getPointedToConnect().equals(canvas.getEditingConnectPoint()))) 
							canvas.discardEditConnectionMode();
						else if (canvas.editingConnection()) canvas.commitConnectionEdit();
						else canvas.setConnectingTo();
					}
				
				}
				
				if (e.button == Const.RIGHT_MOUSE) {
					canvas.setContextMenuAxis(display.getCursorLocation());
					canvas.showContextMenu(e.x, e.y);
				}
				
				setCopyClip(canvas.nodesSelected());
				
			}
			
			@Override
			public void mouseUp(MouseEvent e) {
				if ((selectedActivity == Const.SELECTING_ACTIVE) && (e.button == Const.LEFT_MOUSE)) { 
					if (canvas.isDraggingOn() || canvas.isMarqueeOn()) {
						canvas.dragOff();
					} else canvas.select(e.x, e.y);
					setCopyClip(canvas.nodesSelected());
				}
			}
			
			@Override
			public void mouseDoubleClick(MouseEvent e) {
				discardConnection();
				if (selectedActivity == Const.SELECTING_ACTIVE) 
					canvas.showSettings();
			}
			
		});
		canvas.addMouseMoveListener(new MouseMoveListener() {
			public void mouseMove(MouseEvent e) {
				
				if (! canvas.locked()) {
					
					if (selectedActivity == Const.ADDING_ACTIVE) {
						
						if (selectedTool == Const.START_SELECTED) canvas.setStartGhost(e.x, e.y);
						else if (selectedTool == Const.END_SELECTED) canvas.setEndGhost(e.x, e.y);
						else if (selectedTool == Const.DECISION_SELECTED) canvas.setDecisionGhost(e.x, e.y);
						else if (selectedTool == Const.PROCESS_SELECTED) canvas.setProcessGhost(e.x, e.y);
						else if (selectedTool == Const.PREPARATION_SELECTED) canvas.setPreparationGhost(e.x, e.y);
						else if (selectedTool == Const.MANUAL_INPUT_SELECTED) canvas.setManualInputGhost(e.x, e.y);
						else if (selectedTool == Const.DISPLAY_SELECTED) canvas.setDisplayGhost(e.x, e.y);
						
						if (selectedTool != Const.NONE_TOOL_SELECTED) canvas.drawGridLines(e.x, e.y);
						
					} else if (selectedActivity == Const.SELECTING_ACTIVE) {
						canvas.moveSelected(e.x, e.y);
						canvas.marqueeSelect(e.x, e.y);
						canvas.updateMultipleIndicator(e.x, e.y);
						canvas.setDndResize(e.x, e.y);
						canvas.dndResize(e.x, e.y);
						canvas.trackMousePosition(e.x, e.y);
					} else if (selectedActivity == Const.CONNECTING_ACTIVE) {
						canvas.updateGhostConnectPoint(e.x, e.y);
						canvas.findConnectPoint(e.x, e.y);
						if (! canvas.validActiveConnectPoint()) 
							canvas.getMainShell().setNotification(lang.unconnectableNotif);
					}
					
				}
				
			}
		});
		canvas.addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent e) {
				Image bufferImage = new Image(display, canvas.getBounds().width, canvas.getBounds().height);
				GC gc = new GC(bufferImage);
				gc.setAntialias(SWT.ON);
				canvas.recenter();
				canvas.drawNodes(gc);
				canvas.drawConnections(gc);
				canvas.drawVerticalGridLine(gc);
				canvas.drawHorizontalGridLine(gc);
				canvas.indicateMultipleSelection(gc);
				canvas.drawMarqueeRect(gc);
				canvas.drawConnectPoint(gc);
				e.gc.drawImage(bufferImage, 0, 0);
				bufferImage.dispose();
				gc.dispose();
				adjustToggleBreakpointOption();
				if (canvas.isIdle()) updateGraphSemaphore();
			}
		});
		
		canvasScrolledComposite.setContent(canvasComposite);
		canvasScrolledComposite.setMinSize(canvasComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		
		adjustUndoRedo();
		
		@SuppressWarnings("unused")
		ToolItem historyClipboardSeparator = new ToolItem(toolBar, SWT.SEPARATOR);
		
		tltmCut = new ToolItem(toolBar, SWT.NONE);
		tltmCut.setToolTipText(lang.cut);
		tltmCut.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				cut();
			}
		});
		tltmCut.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/cut.png"));
		
		tltmCopy = new ToolItem(toolBar, SWT.NONE);
		tltmCopy.setToolTipText(lang.copy);
		tltmCopy.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				copy();
			}
		});
		tltmCopy.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/copy.png"));
		
		tltmPaste = new ToolItem(toolBar, SWT.NONE);
		tltmPaste.setToolTipText(lang.paste);
		tltmPaste.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				paste();
			}
		});
		tltmPaste.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/paste.png"));
		
		@SuppressWarnings("unused")
		ToolItem clipboardModeSeparator = new ToolItem(toolBar, SWT.SEPARATOR);

		addItem = new ToolItem(toolBar, SWT.RADIO);
		addItem.setToolTipText(lang.addingMode);
		addItem.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/add.png"));
		addItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				switchAddingMode();
			}
		});
		addItem.setSelection(true);
		
		selectionItem = new ToolItem(toolBar, SWT.RADIO);
		selectionItem.setToolTipText(lang.selectionMode);
		selectionItem.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/select.png"));
		
		connectingItem = new ToolItem(toolBar, SWT.RADIO);
		connectingItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				switchConnectingMode();
			}
		});
		connectingItem.setToolTipText(lang.connectingMode);
		connectingItem.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/connect.png"));
		
		@SuppressWarnings("unused")
		ToolItem steppingSeparator = new ToolItem(toolBar, SWT.SEPARATOR);
		
		playItem = new ToolItem(toolBar, SWT.NONE);
		playItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				discardNotification();
				canvas.play();
			}
		});
		playItem.setToolTipText(lang.playDiagram);
		playItem.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/play.png"));
		
		pauseItem = new ToolItem(toolBar, SWT.NONE);
		pauseItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				canvas.pause();
			}
		});
		pauseItem.setToolTipText(lang.pauseDiagram);
		pauseItem.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/pause.png"));
		
		stopItem = new ToolItem(toolBar, SWT.NONE);
		stopItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				canvas.stop();
			}
		});
		stopItem.setToolTipText(lang.stopDiagram);
		stopItem.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/stop.png"));
		
		stepItem = new ToolItem(toolBar, SWT.NONE);
		stepItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				canvas.nextStep();
			}
		});
		stepItem.setToolTipText(lang.nextStep);
		stepItem.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/step.png"));
		
		breakpointItem = new ToolItem(toolBar, SWT.NONE);
		breakpointItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				canvas.toggleBreakpoint();
			}
		});
		breakpointItem.setToolTipText(lang.toggleBreakpoint);
		breakpointItem.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/breakpoint.png"));
		
		@SuppressWarnings("unused")
		ToolItem toolItem = new ToolItem(toolBar, SWT.SEPARATOR);
		
		tltmCSource = new ToolItem(toolBar, SWT.NONE);
		tltmCSource.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				exportToCSource();
			}
		});
		tltmCSource.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/sourceC.png"));
		
		tltmPng = new ToolItem(toolBar, SWT.NONE);
		tltmPng.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				exportToPNG(e.display);
			}
		});
		tltmPng.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/png.png"));
		
		statusbarComposite = new Composite(shell, SWT.NONE);
		statusbarComposite.setLayoutData(BorderLayout.SOUTH);
		FillLayout fl_statusbarComposite = new FillLayout(SWT.HORIZONTAL);
		fl_statusbarComposite.marginWidth = 5;
		fl_statusbarComposite.marginHeight = 2;
		statusbarComposite.setLayout(fl_statusbarComposite);
		
		semaphoreComposite = new Composite(statusbarComposite, SWT.NONE);
		semaphoreComposite.setLayout(new GridLayout(2, false));
		
		semaphoreLightComposite = new Composite(semaphoreComposite, SWT.NONE);
		semaphoreLightComposite.setLayout(new FillLayout(SWT.HORIZONTAL));
		GridData gd_semaphoreLightComposite = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_semaphoreLightComposite.heightHint = 16;
		gd_semaphoreLightComposite.widthHint = 16;
		semaphoreLightComposite.setLayoutData(gd_semaphoreLightComposite);
		
		semaphoreLightLabel = new Label(semaphoreLightComposite, SWT.NONE);
		semaphoreLightLabel.setAlignment(SWT.CENTER);
		semaphoreLightLabel.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/redLight.png"));
		
		lblSemaphoreLabel = new Label(semaphoreComposite, SWT.NONE);
		lblSemaphoreLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		adjustSteppingItems();
		updateGraphSemaphore();
		
		notifComposite = new Composite(statusbarComposite, SWT.NONE);
		notifComposite.setLayout(new GridLayout(1, false));
		
		lblNotifLabel = new Label(notifComposite, SWT.NONE);
		lblNotifLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		setNotification(lang.welcomeText);
		selectionItem.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				switchSelectionMode();
			}
		});
		
		Composite variablesMonitorComposite = new Composite(shell, SWT.NONE);
		variablesMonitorComposite.setLayoutData(BorderLayout.EAST);
		FillLayout fl_variablesMonitorComposite = new FillLayout(SWT.VERTICAL);
		fl_variablesMonitorComposite.marginWidth = 5;
		variablesMonitorComposite.setLayout(fl_variablesMonitorComposite);
		
		variablesTable = new Table(variablesMonitorComposite, SWT.BORDER);
		variablesTable.setHeaderVisible(true);
		
		tblclmnName = new TableColumn(variablesTable, SWT.NONE);
		tblclmnName.setWidth(89);
		tblclmnName.setText(lang.variable);
		
		tblclmnValue = new TableColumn(variablesTable, SWT.NONE);
		tblclmnValue.setWidth(89);
		tblclmnValue.setText(lang.value);
		
		consoleText = new Text(centerComposite, SWT.BORDER | SWT.READ_ONLY | SWT.V_SCROLL | SWT.MULTI);
		consoleText.setLayoutData(BorderLayout.SOUTH);
		
		setCopyClip(false);
		adjustImageExportItems();

	}
	
	private void appendConsoleLine(String s) {
		if (!consoleText.getText().equals(""))
			consoleText.append("\n");
		consoleText.append(s);
		consoleText.setTopIndex(consoleText.getLineCount() - 1);
	}
	
	public void displayVariable(Variable v) {
		appendConsoleLine(lang.variableValue + Const.SPACE + v.getName() + Const.SPACE + lang.is + Const.SPACE + v.getValue().print());
	}
	
	public void clearConsole() {
		consoleText.setText("");
	}
	
	public void updateVariableMonitor(LinkedList<Variable> vars, Variable lastChanged) {
		variablesTable.removeAll();
		for (Variable v : vars) {
			TableItem item = new TableItem(variablesTable, SWT.NULL);
			item.setText(0, v.getName());
			item.setText(1, v.getValue().print());
			if (v.equals(lastChanged)) {
				Font defFont = item.getFont();
				FontData defFontData = defFont.getFontData()[0];
				if (boldFont != null) boldFont.dispose();
				boldFont = new Font(defFont.getDevice(), defFontData.getName(), defFontData.getHeight(), SWT.BOLD);
				item.setFont(boldFont);
			}
		}
	}
	
	private void setSemaphoreMessage(String msg) {
		lblSemaphoreLabel.setText(" ".concat(msg));
	}
	
	public void adjustNonSteppingItems(boolean val) {
		
		if (val) {
			adjustUndoRedo();
			setCopyClip(canvas.nodesSelected());
			adjustCodeGenerationItems();
			adjustImageExportItems();
		} else {
			mntmUndo.setEnabled(val);
			mntmRedo.setEnabled(val);
			tltmUndo.setEnabled(val);
			tltmRedo.setEnabled(val);
			canvas.setUndoRedo(val);
			setCopyClip(val);
			adjustCodeGenerationItems(val);
			adjustImageExportItems(val);
		}
		
		mntmPaste.setEnabled(val);
		tltmPaste.setEnabled(val);
		canvas.setPasteClip(val);
		
		mntmAddingMode.setEnabled(val);
		mntmSelectionMode.setEnabled(val);
		mntmConnectingMode.setEnabled(val);
		addItem.setEnabled(val);
		selectionItem.setEnabled(val);
		connectingItem.setEnabled(val);
		
		tltmStart.setEnabled(val);
		tltmEnd.setEnabled(val);
		tltmDecision.setEnabled(val);
		tltmPreparation.setEnabled(val);
		tltmProcess.setEnabled(val);
		tltmManualInput.setEnabled(val);
		tltmDisplay.setEnabled(val);
		
		mntmSave.setEnabled(val);
		mntmSaveAs.setEnabled(val);
		tltmSave.setEnabled(val);
		
		mntmNew.setEnabled(val);
		tltmNew.setEnabled(val);
		
		mntmOpen.setEnabled(val);
		tltmOpen.setEnabled(val);
		
	}
	
	public void adjustSteppingItems() {
		
		playItem.setEnabled(!canvas.isRunning());
		mntmPlay.setEnabled(!canvas.isRunning());
		
		pauseItem.setEnabled(canvas.isRunning());
		mntmPause.setEnabled(canvas.isRunning());
		
		stopItem.setEnabled(!canvas.isIdle());
		mntmStop.setEnabled(!canvas.isIdle());
		
		stepItem.setEnabled(canvas.isPaused());
		mntmNextStep.setEnabled(canvas.isPaused());
		
	}
	
	private void adjustCodeGenerationItems(boolean val) {
		mntmGenerateCSource.setEnabled(val);
		tltmCSource.setEnabled(val);
		if (val) tltmCSource.setToolTipText(lang.exportAsC);
		else tltmCSource.setToolTipText(lang.unableToGenerateC);
	}
	
	private void adjustCodeGenerationItems() {
		adjustCodeGenerationItems(canvas.getSemaphore().greenLight());
	}
	
	private void updateGraphSemaphore() {
		
		Semaphore s = canvas.getSemaphore();
		SemaphoreState st = s.getState();
		boolean greenLight = s.greenLight();
		
		switch (st) {
			case MISSING_START:
				setSemaphoreMessage(lang.missingStart);
				break;
			case UNSET_VARIABLE:
				setSemaphoreMessage(lang.unsetVariable.concat(s.getUnsetVariable().getName()));
				break;
			case UNREACHABLE_END:
				setSemaphoreMessage(lang.unreachableEnd);
				break;
			case REACHABLE_END:
				if (s.hasInfiniteLoop()) setSemaphoreMessage(lang.infiniteLoop);
				else setSemaphoreMessage(lang.reachableEnd);
				break;
		}
		
		if (greenLight) 
			semaphoreLightLabel.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/greenLight.png"));
		else semaphoreLightLabel.setImage(SWTResourceManager.getImage(MainShell.class, "/icons/redLight.png"));
		
		playItem.setEnabled(greenLight);
		mntmPlay.setEnabled(greenLight);
		
		adjustCodeGenerationItems();
		
	}
	
	public Shell getShell() {
		return shell;
	}
}
