package jflowchart.ui.windows;

import jflowchart.engine.arithmetics.Value;
import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.common.Const;
import jflowchart.engine.common.Functions;
import jflowchart.engine.common.Language;
import jflowchart.engine.runtime.Program;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import swing2swt.layout.BorderLayout;
import org.eclipse.swt.widgets.Composite;
import swing2swt.layout.FlowLayout;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

public class ManualInputShell extends Shell {
	
	private Text valueText;
	private Button btnOk;
	private Label lblNotifLabel;
	
	private Language lang;
	private Program program;
	private Variable var;
	private Value val;
	private String formText;

	/**
	 * Create the shell.
	 * @param display
	 */
	public ManualInputShell(MainShell main, Program p, Variable v) {
		
		super(main.getShell(), SWT.SHELL_TRIM | SWT.APPLICATION_MODAL);
		this.addListener(SWT.Close, new Listener() {
			public void handleEvent(Event event) {
			}
		});
		
		lang = main.getLanguage();
		program = p;
		var = v;
		val = null;
		formText = lang.variableValue + Const.SPACE + var.getName() + Const.COLON;
		
		setLayout(new BorderLayout(0, 2));
		
		Composite confirmComposite = new Composite(this, SWT.NONE);
		confirmComposite.setLayoutData(BorderLayout.SOUTH);
		confirmComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
		
		Composite buttonComposite = new Composite(confirmComposite, SWT.NONE);
		FillLayout fl_buttonComposite = new FillLayout(SWT.HORIZONTAL);
		fl_buttonComposite.marginWidth = 3;
		fl_buttonComposite.marginHeight = 5;
		buttonComposite.setLayout(fl_buttonComposite);
		
		btnOk = new Button(buttonComposite, SWT.NONE);
		btnOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				program.setValue(var, val);
				program.updateMonitor(var);
				btnOk.getShell().dispose();
			}
		});
		btnOk.setSelection(true);
		btnOk.setEnabled(false);
		btnOk.setText(lang.ok);
		
		Composite settingComposite = new Composite(this, SWT.NONE);
		settingComposite.setLayoutData(BorderLayout.CENTER);
		settingComposite.setLayout(new BorderLayout(0, 0));
		
		lblNotifLabel = new Label(settingComposite, SWT.NONE);
		lblNotifLabel.setAlignment(SWT.CENTER);
		lblNotifLabel.setLayoutData(BorderLayout.SOUTH);
		lblNotifLabel.setText(Const.EMPTY_STRING);
		
		Composite formComposite = new Composite(settingComposite, SWT.NONE);
		formComposite.setLayoutData(BorderLayout.CENTER);
		formComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
		
		Composite valueComposite = new Composite(formComposite, SWT.NONE);
		RowLayout rl_valueComposite = new RowLayout(SWT.HORIZONTAL);
		rl_valueComposite.spacing = 5;
		rl_valueComposite.center = true;
		valueComposite.setLayout(rl_valueComposite);
		
		Label lblValue = new Label(valueComposite, SWT.HORIZONTAL | SWT.CENTER);
		lblValue.setAlignment(SWT.CENTER);
		lblValue.setText(formText);
		
		valueText = new Text(valueComposite, SWT.BORDER);
		valueText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				val = Functions.parseComparableValue(valueText.getText());
				if ((val != null) && (val.getDataType().equals(var.getDataType()))) {
					btnOk.setEnabled(true);
					lblNotifLabel.setText(Const.EMPTY_STRING);
				} else {
					btnOk.setEnabled(false);
					lblNotifLabel.setText(lang.incompatibleTypesMsg);
				}
			}
		});
		
		createContents();
	}

	/**
	 * Create contents of the shell.
	 */
	protected void createContents() {
		setText(lang.variableValue);
		setSize(410, 141);

	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
