package jflowchart.ui.windows;

import java.util.LinkedList;

import jflowchart.engine.arithmetics.Variable;
import jflowchart.engine.common.Const;
import jflowchart.engine.common.Functions;
import jflowchart.engine.common.Language;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import swing2swt.layout.BorderLayout;
import org.eclipse.swt.widgets.Composite;
import swing2swt.layout.FlowLayout;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.DisposeEvent;

public class NewVariableShell extends Shell {

	private Language lang;
	private Button btnOk;
	private Button btnCancel;
	private Combo dataTypeCombo;
	private Text variableNameText;
	private Combo parentCombo;
	private LinkedList<Variable> variables;
	private VariableSetting clientShell;
	private boolean created;
	private Label lblNotif;
	
	private boolean nameExists;
	private boolean beginningWithNumber;
	
	private boolean beginningWithNumber() {
		beginningWithNumber = ((variableNameText.getText().trim().length() > 0) && Character.isDigit(variableNameText.getText().trim().charAt(0)));
		return beginningWithNumber;
	}
	
	private boolean nameExists(String name) {
		nameExists = false;
		for (Variable v : variables)
			if (name.equals(v.getName())) {
				nameExists = true;
				return nameExists;
			}
		return nameExists;
	}
	
	private boolean acceptable() {
		return (! ((dataTypeCombo.getSelectionIndex() == 0) || variableNameText.getText().trim().equals("") || beginningWithNumber || nameExists));
	}
	
	private void adjustOkButton() {
		String name = variableNameText.getText().trim();
		if (beginningWithNumber())
			lblNotif.setText(lang.beginningWithNumber);
		else if (nameExists(name))
			lblNotif.setText(lang.variableNamed + name + lang.alreadyExists);
		else lblNotif.setText("");
		btnOk.setEnabled(acceptable());
	}
	
	private void adjustParentCombo() {
		if (created) {
			parentCombo.select(variables.size());
			clientShell.updateVariableOperands();
		} else parentCombo.select(0);
	}
	
	private void newVariable(final int typeLabel, String name) {
		
		Variable v = null;
		if (typeLabel == Const.INTEGER_TYPE_LABEL_CODE) v = Functions.newIntegerVariable(name);
		else if (typeLabel == Const.FLOAT_TYPE_LABEL_CODE) v = Functions.newFloatVariable(name);
		else if (typeLabel == Const.STRING_TYPE_LABEL_CODE) v = Functions.newStringVariable(name);
		
		if (v != null) {
			variables.add(v);
			created = true;
		}
		
	}
	
	/**
	 * Create the shell.
	 * @param display
	 */
	public NewVariableShell(Shell parent, VariableSetting client, Language l, Combo c, LinkedList<Variable> vars) {
		
		super(parent, SWT.SHELL_TRIM | SWT.APPLICATION_MODAL);
		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				if (created) clientShell.updateVariableList();
				adjustParentCombo();
			}
		});
		
		clientShell = client;
		lang = l;
		parentCombo = c;
		variables = vars;
		created = false;
		
		nameExists = false;
		beginningWithNumber = false;
		
		setLayout(new BorderLayout(0, 0));
		
		Composite newVariableComposite = new Composite(this, SWT.NONE);
		newVariableComposite.setLayoutData(BorderLayout.CENTER);
		newVariableComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
		
		Composite variableSettingsComposite = new Composite(newVariableComposite, SWT.NONE);
		variableSettingsComposite.setLayoutData(BorderLayout.CENTER);
		RowLayout rl_variableSettingsComposite = new RowLayout(SWT.HORIZONTAL);
		rl_variableSettingsComposite.center = true;
		variableSettingsComposite.setLayout(rl_variableSettingsComposite);
		
		Composite dataTypeComposite = new Composite(variableSettingsComposite, SWT.NONE);
		RowLayout rl_dataTypeComposite = new RowLayout(SWT.HORIZONTAL);
		rl_dataTypeComposite.center = true;
		dataTypeComposite.setLayout(rl_dataTypeComposite);
		
		Label lblDataType = new Label(dataTypeComposite, SWT.NONE);
		lblDataType.setText(lang.dataType + Const.COLON);
		
		dataTypeCombo = new Combo(dataTypeComposite, SWT.READ_ONLY);
		dataTypeCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				adjustOkButton();
			}
		});
		dataTypeCombo.setItems(new String[] {lang.chooseDataType + Const.ELLIPSIS, lang.integer, lang.decimal, lang.string});
		dataTypeCombo.select(0);
		
		Composite variableNameComposite = new Composite(variableSettingsComposite, SWT.NONE);
		RowLayout rl_variableNameComposite = new RowLayout(SWT.HORIZONTAL);
		rl_variableNameComposite.center = true;
		variableNameComposite.setLayout(rl_variableNameComposite);
		
		Label lblVariableName = new Label(variableNameComposite, SWT.NONE);
		lblVariableName.setText(lang.variableName + Const.COLON);
		
		variableNameText = new Text(variableNameComposite, SWT.BORDER);
		
		Composite southComposite = new Composite(this, SWT.NONE);
		southComposite.setLayoutData(BorderLayout.SOUTH);
		southComposite.setLayout(new BorderLayout(0, 0));
		
		lblNotif = new Label(southComposite, SWT.NONE);
		lblNotif.setAlignment(SWT.CENTER);
		lblNotif.setLayoutData(BorderLayout.NORTH);
		
		Composite confirmComposite = new Composite(southComposite, SWT.NONE);
		confirmComposite.setLayoutData(BorderLayout.CENTER);
		confirmComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
		
		Composite buttonsComposite = new Composite(confirmComposite, SWT.NONE);
		FillLayout fl_buttonsComposite = new FillLayout(SWT.HORIZONTAL);
		fl_buttonsComposite.marginHeight = 9;
		buttonsComposite.setLayout(fl_buttonsComposite);
		
		btnOk = new Button(buttonsComposite, SWT.NONE);
		btnOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				newVariable(dataTypeCombo.getSelectionIndex() - 1, variableNameText.getText().trim());
				btnOk.getShell().dispose();
			}
		});
		btnOk.setText(lang.ok);
		btnOk.setEnabled(false);
		
		btnCancel = new Button(buttonsComposite, SWT.NONE);
		btnCancel.setSelection(true);
		btnCancel.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				btnCancel.getShell().dispose();
			}
		});
		btnCancel.setText(lang.cancel);
		
		variableNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				adjustOkButton();
			}
		});
		
		createContents();
	
	}

	/**
	 * Create contents of the shell.
	 */
	protected void createContents() {
		setText(lang.newVariable);
		setSize(457, 154);
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
