package jflowchart.ui.windows;

import jflowchart.engine.arithmetics.Assignment;
import jflowchart.engine.arithmetics.Comparison;
import jflowchart.engine.arithmetics.IncompatibleDataTypesException;
import jflowchart.engine.common.Const;
import jflowchart.engine.common.Language;
import jflowchart.engine.history.EditingPreparationAction;
import jflowchart.ui.canvas.FlowChartCanvas;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import swing2swt.layout.BorderLayout;
import org.eclipse.swt.widgets.Composite;
import swing2swt.layout.FlowLayout;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;

public class PreparationSettingsShell extends Shell {
	
	private Text conditionText;
	private Text operationText;
	private Button btnOk;
	private Button btnCancel;
	private Button btnCreateEditCondition;
	private Button btnCreateEditOperation;
	
	private Language lang;
	private MainShell mainShell;
	private FlowChartCanvas canvas;
	
	private Comparison conditionOrig;
	private Comparison conditionCopy;
	
	private Assignment assignmentOrig;
	private Assignment assignmentCopy;
	
	private boolean edit;
	private EditingPreparationAction editAction;
	
	private void adjustOkButton() {
		btnOk.setEnabled(conditionCopy.usable() && assignmentCopy.usable());
	}
	
	private void setChanges() {
		try {
			conditionOrig.setFirstOperand(conditionCopy.getFirstOperand());
			conditionOrig.setOperator(conditionCopy.getOperator());
			conditionOrig.setSecondOperand(conditionCopy.getSecondOperand());
			assignmentOrig.setFirstOperand(assignmentCopy.getFirstOperand());
			assignmentOrig.setSecondOperand(assignmentCopy.getSecondOperand());
		} catch (IncompatibleDataTypesException e) {}
	}

	public PreparationSettingsShell(MainShell main, Language l, Comparison c, Assignment a, boolean editing) {
		
		super(main.getShell(), SWT.CLOSE | SWT.MIN | SWT.MAX | SWT.TITLE | SWT.APPLICATION_MODAL);
		setMinimumSize(new Point(Const.PREPARATION_SETTINGS_SHELL_WIDTH, Const.PREPARATION_SETTINGS_SHELL_HEIGHT));
		
		lang = l;
		mainShell = main;
		canvas = mainShell.getCanvas();
		
		conditionOrig = c;
		conditionCopy = conditionOrig.deepCopy();
		
		assignmentOrig = a;
		assignmentCopy = assignmentOrig.deepCopy();
		
		edit = editing;
		if (edit) editAction = new EditingPreparationAction(canvas, conditionOrig, assignmentOrig);
		else editAction = null;
		
		setLayout(new BorderLayout(0, 0));
		
		Composite confirmationComposite = new Composite(this, SWT.NONE);
		confirmationComposite.setLayoutData(BorderLayout.SOUTH);
		confirmationComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
		
		Composite buttonsComposite = new Composite(confirmationComposite, SWT.NONE);
		FillLayout fl_buttonsComposite = new FillLayout(SWT.HORIZONTAL);
		fl_buttonsComposite.marginHeight = 9;
		buttonsComposite.setLayout(fl_buttonsComposite);
		
		btnOk = new Button(buttonsComposite, SWT.NONE);
		btnOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				setChanges();
				if (edit) {
					editAction.recordNewData();
					canvas.recordAction(editAction);
				}
				btnOk.getShell().dispose();
			}
		});
		btnOk.setText(lang.ok);
		adjustOkButton();
		
		btnCancel = new Button(buttonsComposite, SWT.NONE);
		btnCancel.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				btnCancel.getShell().dispose();
			}
		});
		btnCancel.setSelection(true);
		btnCancel.setText(lang.cancel);
		
		Composite settingsComposite = new Composite(this, SWT.NONE);
		settingsComposite.setLayoutData(BorderLayout.CENTER);
		settingsComposite.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
		
		Composite gridComposite = new Composite(settingsComposite, SWT.NONE);
		gridComposite.setLayout(new GridLayout(2, false));
		
		Label lblCondition = new Label(gridComposite, SWT.NONE);
		lblCondition.setText(lang.condition + Const.COLON);
		
		Composite conditionComposite = new Composite(gridComposite, SWT.NONE);
		conditionComposite.setLayout(new RowLayout(SWT.HORIZONTAL));
		
		conditionText = new Text(conditionComposite, SWT.BORDER | SWT.READ_ONLY);
		conditionText.setEnabled(false);
		if (conditionCopy.usable()) conditionText.setText(conditionCopy.print());
		
		btnCreateEditCondition = new Button(conditionComposite, SWT.NONE);
		btnCreateEditCondition.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				mainShell.showConditionSettingsShell(btnCreateEditCondition.getShell(), conditionCopy, false);
				if (conditionCopy.usable()) conditionText.setText(conditionCopy.print());
				adjustOkButton();
			}
		});
		btnCreateEditCondition.setText(lang.create + Const.SLASH + lang.edit + Const.ELLIPSIS);
		
		Label lblStepOperation = new Label(gridComposite, SWT.NONE);
		lblStepOperation.setText(lang.stepOperation + Const.COLON);
		
		Composite operationComposite = new Composite(gridComposite, SWT.NONE);
		operationComposite.setLayout(new RowLayout(SWT.HORIZONTAL));
		
		operationText = new Text(operationComposite, SWT.BORDER | SWT.READ_ONLY);
		operationText.setEnabled(false);
		if (assignmentCopy.usable()) operationText.setText(assignmentCopy.print());
		
		btnCreateEditOperation = new Button(operationComposite, SWT.NONE);
		btnCreateEditOperation.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				mainShell.showAssignmentSettingsShell(btnCreateEditOperation.getShell(), assignmentCopy, false, false);
				if (assignmentCopy.usable()) operationText.setText(assignmentCopy.print());
				adjustOkButton();
			}
		});
		btnCreateEditOperation.setText(lang.create + Const.SLASH + lang.edit + Const.ELLIPSIS);
		createContents();
	}

	/**
	 * Create contents of the shell.
	 */
	protected void createContents() {
		setText(lang.preparationSettings);
		setSize(Const.PREPARATION_SETTINGS_SHELL_WIDTH, Const.PREPARATION_SETTINGS_SHELL_HEIGHT);
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
